package more.ssaqms.xiamen.controller;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import more.commons.utils.response.ResponseResult;
import more.commons.utils.response.exception.MoreException;
import more.ssaqms.xiamen.handler.month.traffic.TrafficMonthReport;
import more.ssaqms.xiamen.model.report.ReportOutput;
import more.ssaqms.xiamen.model.supper.htmlInput;
import more.ssaqms.xiamen.model.traffic.TrafficInput;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.WordTransformUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

/**
 * @author zuo
 * @date 2024/8/28 15:56
 */

@Api(value = "厦门达良日报接口")
@RestController
@RequestMapping(value = "/TrafficReport")
@Slf4j
public class TrafficReportController {

    @Value("${cloud.report.path}")
    private String reportPath;

    @Autowired
    private TrafficMonthReport trafficMonthReport;

    @ApiOperation(value = "获取交通站月报html字符串", notes = "获取交通站月报html字符串", produces = "application/json")
    @PostMapping(value = "/html")
    public ResponseResult<String> getReachGoodReport(@RequestBody @Validated TrafficInput input) {
        return ResponseResult.success(trafficMonthReport.creatMonthHtmlReport(input).getHtml());
    }

    @ApiOperation(value = "获取超站月报html字符串", notes = "获取超站月报html字符串", produces = "application/octet-stream")
    @PostMapping(value = "/word")
    public void exportMonthReport(@RequestBody @Validated htmlInput input, HttpServletResponse response) {
        Date date = input.getDate();
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        String localHtmlStream = WordTransformUtils.dayReportHtmlAll(input.getHtml());
        String idStr = IdWorker.getIdStr();
        String path = reportPath + "TrafficReport/" + idStr;
        String filePathStr = WordTransformUtils.writeWordFile(localHtmlStream, path, path + "/", idStr, false, null);


        File file = new File(filePathStr);

        if (!file.exists()) {//如果文件不存在
            throw new MoreException("文件未找到");
        }
        String fileName = "厦门市空气质量组分监测简报【" + ca.get(Calendar.YEAR) + "】第" + (ca.get(Calendar.MONTH) + 1) + "期(交通站).docx";
        //输入处理流
        InputStream inputStream = null;
        //输出处理流
        OutputStream outputStream = null;
        try {
            //定义处理流
            inputStream = new BufferedInputStream(Files.newInputStream(file.toPath()));
            outputStream = new BufferedOutputStream(response.getOutputStream());

            response.reset();
            //解决跨域文件名问题
            response.addHeader("Access-Control-Expose-Headers", "content-disposition");
            // 设置在下载框默认显示的文件名
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            //写明文件大小
            response.addHeader("Content-Length", "" + file.length());
            // 指明response的返回对象是文件流
            response.setContentType("application/octet-stream");
            //response.setContentType("multipart/form-data");

            //"开始读取文件流"
            //返回从该输入流中可以读取（或跳过）的字节数的估计值，而不会被下一次调用此输入流的方法阻塞
            byte[] bytes = new byte[inputStream.available()];
            //将pdf的内容写入bytes中
            inputStream.read(bytes);
            //将bytes中的内容写入
            outputStream.write(bytes);
            //刷新输出流，否则不会写出数据
            outputStream.flush();

            //文件下载处理完成
            FileUtils.delFile(new File(path));


        } catch (FileNotFoundException e) {
            log.error("文件未找到"+e.getMessage());
        } catch (IOException e) {
            log.error("文件读写异常"+e.getMessage());
        }finally {//关闭输入流、输出流
            try {
                if (inputStream != null){
                    inputStream.close();
                    log.debug("输入流关闭成功！");
                }
                if (outputStream != null){
                    outputStream.close();
                    log.debug("输出流关闭成功！");
                }
            } catch (IOException e) {
                log.error("关闭处理流出现异常"+e.getMessage());
            }
        }
    }
}
