package more.ssaqms.xiamen.handler.day.good;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.ssaqms.xiamen.model.factor.Factor;
import more.ssaqms.xiamen.model.reachGood.ReachGoodAqiData;
import more.ssaqms.xiamen.model.reachGood.ReachGoodPollutantData;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.report.TimeSeriesChart;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.RestTemplateUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Component;

import java.util.*;

import static more.ssaqms.xiamen.util.MonthReportUtils.image;
import static more.ssaqms.xiamen.util.MonthReportUtils.textCenter;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class ReachGoodChapter1 {

    public String html(TimeRange dayTimeRange, ReachGoodAqiData reachGoodAqiData, String chartUrl, String path, Map<String, Integer> countMap) {
        Integer imageCount = countMap.get("imageCount");
        Integer tableCount = countMap.get("tableCount");
        Date beginTime = dayTimeRange.getBeginTime();
        Float dayAqiValue = reachGoodAqiData.getDayAqiValue();
        String dayAqiValueStr = Objects.isNull(dayAqiValue) ? "XX" : dayAqiValue.toString();
        String gradeStr = reachGoodAqiData.getGradeStr();
        gradeStr = Objects.isNull(gradeStr) ? "XX" : gradeStr;
        String dayPrimaryPollutant = reachGoodAqiData.getDayPrimaryPollutant();
        String dayPrimaryPollutantStr = "无首要污染物";
        if (StringUtils.isNotBlank(dayPrimaryPollutant)) {
            dayPrimaryPollutantStr = "首要污染物为" + dayPrimaryPollutant;
        }
        List<ReachGoodPollutantData> dayPollutantDataList = reachGoodAqiData.getDayPollutantDataList();
        List<String> names = new ArrayList<>();
        List<String> values = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(dayPollutantDataList)) {
            for (ReachGoodPollutantData reachGoodPollutantData : dayPollutantDataList) {
                String factorId = reachGoodPollutantData.getFactorId();
                String stdUnit = "μg/m³";
                if ("1005".equals(factorId)) stdUnit = "mg/m³";
                values.add(reachGoodPollutantData.getValue() + stdUnit);
                names.add(reachGoodPollutantData.getName());
            }
        }
        String nameStr = "XXX";
        if (CollectionUtils.isNotEmpty(names)) {
            nameStr = String.join("、", names);
        }
        String valueStr = "XXX";
        if (CollectionUtils.isNotEmpty(values)) {
            valueStr = String.join("、", values);
        }


        StringBuilder sb = new StringBuilder();
        sb.append(headline1("一、空气质量整体概况"));
        sb.append(textPart(DateFormatUtils.format(beginTime, "yyyy-MM-dd") + "，厦门市AQI指数为" + dayAqiValueStr +
                "，空气质量等级为" + gradeStr + "，" + dayPrimaryPollutantStr + "。当日常规污染物（" + nameStr + "）浓度分别为：" + valueStr + "。"));


        Map<String, ReachGoodPollutantData> maxMap = reachGoodAqiData.getMaxMap();
        Map<String, ReachGoodPollutantData> minMap = reachGoodAqiData.getMinMap();

        String s = "SO2于14时达到峰值浓度XXμg/m3，于11时达到谷值浓度XXμg/m3；" +
                "NO2于14时达到峰值浓度XXμg/m3，于11时达到谷值浓度XXμg/m3；" +
                "CO于14时达到峰值浓度XXμg/m3，于11时达到谷值XXμg/m3；" +
                "O3于14时达到峰值浓度XXμg/m3，于23时达到谷值浓度XXμg/m3；" +
                "PM10于14时达到峰值浓度XXμg/m3，于23时达到谷值浓度XXμg/m3；" +
                "PM2.5于10时达到峰值浓度XXμg/m3，于23时达到谷值浓度XXμg/m3。";

        if (CollectionUtils.isNotEmpty(maxMap)){
            List<String> list = new ArrayList<>();
            for (Map.Entry<String, ReachGoodPollutantData> entry : maxMap.entrySet()) {
                String factorId = entry.getKey();
                ReachGoodPollutantData maxData = entry.getValue();
                String name = maxData.getName();
                Date maxDataTime = maxData.getDataTime();
                Float maxValue = maxData.getValue();
                String stdUnit = "μg/m³";
                if ("1005".equals(factorId)) stdUnit = "mg/m³";
                ReachGoodPollutantData minData = minMap.get(factorId);
                Date minDataTime = minData.getDataTime();
                Float minValue = minData.getValue();
                list.add(name + "于"+DateFormatUtils.format(maxDataTime,"HH时")+"达到峰值浓度"+ maxValue + stdUnit +
                        "，于"+DateFormatUtils.format(minDataTime,"HH时")+"达到谷值浓度"+ minValue + stdUnit);
            }
            s = String.join("；",list);
        }
        sb.append(textPart("从图1厦门市常规监测因子时序图可知：" + s + "。"));

        Map<String, Object> imageMap = reachGoodAqiData.getHourPollutantChartMap();
        if (Objects.isNull(imageMap)) imageMap = new HashMap<>();

        imageMap.put("width", 600);
        imageMap.put("height", 225);
        String imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/xy", imageMap);
        String fileName = IdWorker.getIdStr() + ".jpg";
        String wordImagePath = "image/" + fileName;
        if (StringUtils.isNotBlank(imageBase64)){
            FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
            sb.append(image(wordImagePath, null, null));
        }
        sb.append(textCenter("图"+ imageCount++ +" 厦门市常规监测因子时间序列图"));



        List<TimeSeriesChart> timeSeriesCharts = reachGoodAqiData.getTimeSeriesCharts();
        if (CollectionUtils.isNotEmpty(timeSeriesCharts)){
            Map<String, Map<String, String>> siteComparisonMap = reachGoodAqiData.getSiteComparison();
            int count = 0;
            for (Map.Entry<String, Map<String, String>> entry : siteComparisonMap.entrySet()) {
                Map<String, String> value = entry.getValue();
                String factorName = value.get("factorName");
                String maxSiteName = value.get("maxSiteName");
                String minSiteName = value.get("minSiteName");
                sb.append(textPart("对比图"+ (imageCount + count) +"各国控点位"+factorName+"小时浓度变化，"+maxSiteName+"点位浓度相对最高，"+minSiteName+"点位相对最低，可能受XX影响。"));
                count++;
            }

            for (TimeSeriesChart timeSeriesChart : timeSeriesCharts) {
                Factor factor = timeSeriesChart.getDimensions().get(0);
                imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/line", timeSeriesChart);
                fileName = IdWorker.getIdStr() + ".jpg";
                wordImagePath = "image/" + fileName;
                if (imageBase64 != null) {
                    FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
                }
                sb.append(image(wordImagePath, null, null));
                sb.append(textCenter("图"+ imageCount++ +" 国控点"+DateFormatUtils.format(beginTime,"MM月dd日")+factor.getCnName()+"小时浓度序列图"));
            }
        }
        countMap.put("imageCount",imageCount);
        countMap.put("tableCount",tableCount);
        return sb.toString();
    }


    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

}
