package more.ssaqms.xiamen.handler.day.good;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.ssaqms.xiamen.model.reachGood.ReachGoodVocData;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.statistics.AvgData;
import more.ssaqms.xiamen.model.statistics.PollutantData;
import more.ssaqms.xiamen.util.CalculationUtils;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.RestTemplateUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static more.ssaqms.xiamen.util.MonthReportUtils.textCenter;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class ReachGoodChapter3 {

    public String html(TimeRange dayTimeRange, String chartUrl, String path, Map<String, Integer> countMap, ReachGoodVocData vocData) {
        Integer imageCount = countMap.get("imageCount");
        Integer tableCount = countMap.get("tableCount");
        Date beginTime = dayTimeRange.getBeginTime();


        StringBuilder sb = new StringBuilder();
        sb.append(headline1("三、在线监测VOCs特征分析"));
        sb.append(headline2("3.1 VOCs浓度水平及构成"));
        String maxTvocValue = vocData.getMaxTvocValue();
        String avgTvocValue = vocData.getAvgTvocValue();
        sb.append(textPart("厦门市超级站VOCs监测时序图显示，" + DateFormatUtils.format(beginTime, "dd日") +
                "TVOC峰值浓度" + (StringUtils.isBlank(maxTvocValue) ? "XX" : maxTvocValue) + "μg/m³，" +
                "日均浓度" + (StringUtils.isBlank(avgTvocValue) ? "XX" : avgTvocValue) + "μg/m³，" +
                "XX~XX时VOCs浓度明显XX，其中以.......。"));

        Map<String, Object> imageMap = vocData.getGroupLineChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        String imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/xy", imageMap);
        String fileName = IdWorker.getIdStr() + ".jpg";
        String wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));

        imageMap = vocData.getGroupAvgPieChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/pie", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));

        sb.append(textCenter("图" + imageCount++ + " 厦门超站VOCs化学组分浓度时间序列及百分占比图"));


        sb.append(headline2("3.2 VOCs关键组分筛选"));
        List<AvgData> vocTop10AvgDataList = vocData.getVocTop10AvgDataList();
        String text1 = "xxxx";
        String text2 = "xxxx";
        String text3 = "xxxx";
        for (int i = 0; i < Math.min(vocTop10AvgDataList.size(), 3); i++) {
            AvgData avgData = vocTop10AvgDataList.get(i);
            String factorName = avgData.getFactorName();
            Float value = avgData.getValue();
            if (0 == i) {
                text1 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else if (1 == i) {
                text2 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else {
                text3 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            }
        }
        sb.append(textPart("如图所示，VOCs组分测定排名前三的物质为" + text1 + "、" + text2 + "和" + text3 + "，主要来源为.......。"));

        imageMap = vocData.getVocTop10AvgChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/factor/bar", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));

        sb.append(textCenter("图" + imageCount++ + " 厦门超站VOCs组分浓度贡献排名前十的物质"));

        sb.append(headline2("3.3 VOCs活性评估"));

        List<PollutantData> pollutantData = vocData.getPollutantData();
        String ofp1 = "xxxx";
        String ofp2 = "xxxx";
        String ofp3 = "xxxx";
        for (int i = 0; i < Math.min(pollutantData.size(), 3); i++) {
            PollutantData pollutantData1 = pollutantData.get(i);
            String factorName = pollutantData1.getFactorName();
            Float value = pollutantData1.getIOfp();
            if (0 == i) {
                ofp1 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else if (1 == i) {
                ofp2 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else {
                ofp3 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            }
        }

        sb.append(textPart("VOCs组分中，不同物质对臭氧的影响贡献存在较大差异。" +
                "对厦门超站VOCs物种进行OFP（臭氧生成潜势）测定，" +
                "排名前三的物质为" + ofp1 + "、" + ofp2 + "和" + ofp3 + "，主要来源为.......。"));


        imageMap = vocData.getOfpTop10ChartMap();

        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/bar", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));

        sb.append(textCenter("图" + imageCount++ + " 厦门超站OFP排名前十的物质"));

        countMap.put("imageCount", imageCount);
        countMap.put("tableCount", tableCount);
        return sb.toString();
    }


    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    /**
     * 图片
     *
     * @param width  宽
     * @param height 高
     * @return
     */
    public static String imageTwo(String path1, String path2, Double width, Double height) {
        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 2.34375;
        return "<p style=\"text-align:center;\">" +
                "<img src=\"" + path1 + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "<img src=\"" + path2 + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "</p>";
    }

    /**
     * 图片
     *
     * @param width  宽
     * @param height 高
     * @return
     */
    public static String image(String path, Double width, Double height) {
        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 2.34375;
        return "<p style=\"text-align:center;\">" +
                "<img src=\"" + path + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "</p>";
    }

}
