package more.ssaqms.xiamen.handler.day.good;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import more.ssaqms.xiamen.model.data.HistoricalData;
import more.ssaqms.xiamen.model.reachGood.ReachGoodParticleData;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.statistics.AvgData;
import more.ssaqms.xiamen.util.CalculationUtils;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.RestTemplateUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Component;

import java.util.*;

import static more.ssaqms.xiamen.util.MonthReportUtils.textCenter;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class ReachGoodChapter4 {

    public String html(TimeRange dayTimeRange, String chartUrl, String path, Map<String, Integer> countMap, ReachGoodParticleData particleData) {
        Integer imageCount = countMap.get("imageCount");
        Integer tableCount = countMap.get("tableCount");
        Date beginTime = dayTimeRange.getBeginTime();


        StringBuilder sb = new StringBuilder();
        sb.append(headline1("四、颗粒物组分特征分析"));
        Map<String, HistoricalData> pmMaxMap = particleData.getPmMaxMap();
        Map<String, HistoricalData> pmMinMap = particleData.getPmMinMap();

        String s = "PM10浓度于10时达到峰值85μg/m³，于22时达到谷值22μg/m³；" +
                "PM2.5浓度于01时达到峰值29μg/m³，于23时达到谷值12μg/m³。";

        if (CollectionUtils.isNotEmpty(pmMaxMap)) {
            List<String> list = new ArrayList<>();
            for (Map.Entry<String, HistoricalData> entry : pmMaxMap.entrySet()) {
                String factorId = entry.getKey();
                HistoricalData maxData = entry.getValue();
                String name = maxData.getFactorName();
                Date maxDataTime = maxData.getDataTime();
                Float maxValue = maxData.getValue();
                String stdUnit = "μg/m³";
                HistoricalData minData = pmMinMap.get(factorId);
                Date minDataTime = minData.getDataTime();
                Float minValue = minData.getValue();
                list.add(name + "浓度于" + DateFormatUtils.format(maxDataTime, "HH时") + "达到峰值" + maxValue + stdUnit +
                        "，于" + DateFormatUtils.format(minDataTime, "HH时") + "达到谷值" + minValue + stdUnit);
            }
            s = String.join("；", list);
        }


        sb.append(textPart(DateFormatUtils.format(beginTime, "MM月dd日") +
                "厦门市大气超级站PM10和PM2.5浓度变化趋势图显示，" + s + "PM2.5与PM10浓度变化XXX，以XXX为主。"));


        Map<String, Object> imageMap = particleData.getPmChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        String imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/xy", imageMap);
        String fileName = IdWorker.getIdStr() + ".jpg";
        String wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));
        sb.append(textCenter("图" + imageCount++ + " 厦门市超站PM10和PM2.5时间序列图"));


        List<AvgData> liAvgDataList = particleData.getLiAvgDataList();

        String name1 = "XX";
        String name2 = "XX";
        String name3 = "XX";
        String value1 = "XX";
        String value2 = "XX";
        String value3 = "XX";


        for (int i = 0; i < Math.min(liAvgDataList.size(), 3); i++) {
            AvgData avgData = liAvgDataList.get(i);
            String factorName = avgData.getFactorName();
            Float value = avgData.getValue();
            if (0 == i) {
                name1 = factorName;
                value1 = CalculationUtils.digitHalfEvenStr(value, 2);
            } else if (1 == i) {
                name2 = factorName;
                value2 = CalculationUtils.digitHalfEvenStr(value, 2);
            } else {
                name3 = factorName;
                value3 = CalculationUtils.digitHalfEvenStr(value, 2);
            }
        }

        sb.append(textPart("从水溶性离子组分时间序列图中可以看出，" +
                "排名前三的离子组分为" + name1 + "、" + name2 + "" + name3 + "，" +
                "日均浓度分别为" + value1 + "μg/m³、" + value2 + "μg/m³、" + value3 + "μg/m³，" +
                "XX-XX小时XX组分明显升高。其中，XX组分与XX源相关；XX组分与XX源相关。"));

        imageMap = particleData.getLiChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/xy", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));
        sb.append(textCenter("图" + imageCount++ + " 水溶性离子浓度时间序列图"));


        String ocMaxStr = "OC于XX时达到峰值浓度XXXμg/m³";
        String ecMaxStr = "EC于XX时达到峰值浓度XXXμg/m³";
        HistoricalData ocMax = particleData.getOcMax();
        if (!Objects.isNull(ocMax)) {
            ocMaxStr = "OC于" + DateFormatUtils.format(ocMax.getDataTime(), "HH时") + "达到峰值浓度" + CalculationUtils.digitHalfEvenStr(ocMax.getValue(), 2) + "μg/m³";
        }
        HistoricalData ecMax = particleData.getEcMax();
        if (!Objects.isNull(ecMax)) {
            ecMaxStr = "EC于" + DateFormatUtils.format(ecMax.getDataTime(), "HH时") + "达到峰值浓度" + CalculationUtils.digitHalfEvenStr(ecMax.getValue(), 2) + "μg/m³";
        }
        sb.append(textPart("从碳质组分时间序列图来看，" + ocMaxStr + "，" + ecMaxStr + "。"));

        imageMap = particleData.getOcEcChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/xy", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));
        sb.append(textCenter("图" + imageCount++ + " OC、EC时间序列"));

        List<AvgData> jsAvgDataList = particleData.getJsAvgDataList();
        jsAvgDataList.sort(Comparator.comparing(AvgData::getValue).reversed());
        double sum = jsAvgDataList.stream().mapToDouble(AvgData::getValue).sum();

        List<String> jsNames = new ArrayList<>();
        List<String> jsValues = new ArrayList<>();
        List<String> jsProportions = new ArrayList<>();
        for (int i = 0; i < Math.min(jsAvgDataList.size(), 6); i++) {
            AvgData avgData = jsAvgDataList.get(i);
            Float value = avgData.getValue();
            jsNames.add(avgData.getFactorName());
            jsValues.add(CalculationUtils.digitHalfEvenStr(value, 2));
            jsProportions.add(CalculationUtils.digitHalfEvenStr(Float.valueOf(String.valueOf((value / sum) * 100)), 2));
        }
        String jsNameStr = "XX、XX、XX、XX、XX、XX";
        String jsValueStr = "XXng/m³、XXng/m³、XXng/m³、XXng/m³、XXng/m³、XXng/m³";
        String jsProportionStr = "XX%、XX%、XX%、XX%、XX%、XX%";

        if (CollectionUtils.isNotEmpty(jsNames)){
            jsNameStr = String.join("、",jsNames);
        }
        if (CollectionUtils.isNotEmpty(jsValues)){
            jsValueStr = String.join("ng/m³、",jsValues) + "ng/m³";
        }
        if (CollectionUtils.isNotEmpty(jsProportions)){
            jsProportionStr = String.join("%、",jsProportions) + "%";
        }
        sb.append(textPart("从无机元素来看，5月16日厦门市日均浓度排名前六为" + jsNameStr + "，" +
                "日均浓度分别为" + jsValueStr + "；" +
                "占比分别为" + jsProportionStr + "。其中XX元素指示XX源，XX元素指示XX源。"));

        imageMap = particleData.getJsChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/xy", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));
        sb.append(textCenter("图" + imageCount++ + " 无机元素组分时间序列"));

        String klText = "XX、XX、XX、XX、XX、XX、XX、XX、XX";
        List<AvgData> klAvgDataList = particleData.getKlAvgDataList();
        if (CollectionUtils.isNotEmpty(klAvgDataList)){
            List<String> list = new ArrayList<>();
            for (AvgData avgData : klAvgDataList) {
                list.add(avgData.getFactorName() + CalculationUtils.digitHalfEvenStr(avgData.getValue(),2) + "%");
            }
            klText = String.join("、",list);
        }

        sb.append(textPart("超级站监测到的污染源占比从高到低依次为"+klText+"。"));


        imageMap = particleData.getJsChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/xy", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));
        sb.append(textCenter("图" + imageCount++ + " 单颗粒质谱来源时序分布特征"));

        imageMap = particleData.getKlAvgPieChartMap();
        imageMap.put("width", 600);
        imageMap.put("height", 225);
        imageBase64 = RestTemplateUtils.getHistogram(chartUrl + "/chart/pie", imageMap);
        fileName = IdWorker.getIdStr() + ".jpg";
        wordImagePath = "image/" + fileName;
        FileUtils.GenerateImage(imageBase64.substring(imageBase64.indexOf("base64,") + 7), path, fileName);
        sb.append(image(wordImagePath, null, null));

        sb.append(textCenter("图" + imageCount++ + " 单颗粒质谱来源占比"));


        countMap.put("imageCount", imageCount);
        countMap.put("tableCount", tableCount);
        return sb.toString();
    }


    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    /**
     * 图片
     *
     * @param width  宽
     * @param height 高
     * @return
     */
    public static String imageTwo(String path1, String path2, Double width, Double height) {
        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 2.34375;
        return "<p style=\"text-align:center;\">" +
                "<img src=\"" + path1 + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "<img src=\"" + path2 + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "</p>";
    }

    /**
     * 图片
     *
     * @param width  宽
     * @param height 高
     * @return
     */
    public static String image(String path, Double width, Double height) {
        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 2.34375;
        return "<p style=\"text-align:center;\">" +
                "<img src=\"" + path + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "</p>";
    }

}
