package more.ssaqms.xiamen.handler.day.good;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.ssaqms.xiamen.model.reachGood.*;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.statistics.AvgData;
import more.ssaqms.xiamen.model.statistics.PollutantData;
import more.ssaqms.xiamen.util.CalculationUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class ReachGoodChapter6 {

    public String html(TimeRange dayTimeRange, ReachGoodAqiData reachGoodAqiData, ReachGoodWeatherData weatherData, ReachGoodVocData vocData, ReachGoodParticleData particleData) {
        Date beginTime = dayTimeRange.getBeginTime();

        Float dayAqiValue = reachGoodAqiData.getDayAqiValue();
        String dayAqiValueStr = Objects.isNull(dayAqiValue) ? "XX" : dayAqiValue.toString();
        String gradeStr = reachGoodAqiData.getGradeStr();
        gradeStr = Objects.isNull(gradeStr) ? "XX" : gradeStr;
        String dayPrimaryPollutant = reachGoodAqiData.getDayPrimaryPollutant();
        String dayPrimaryPollutantStr = "无首要污染物";
        if (StringUtils.isNotBlank(dayPrimaryPollutant)) {
            dayPrimaryPollutantStr = "首要污染物为" + dayPrimaryPollutant;
        }
        List<ReachGoodPollutantData> dayPollutantDataList = reachGoodAqiData.getDayPollutantDataList();
        List<String> names = new ArrayList<>();
        List<String> values = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(dayPollutantDataList)) {
            for (ReachGoodPollutantData reachGoodPollutantData : dayPollutantDataList) {
                String factorId = reachGoodPollutantData.getFactorId();
                String stdUnit = "μg/m³";
                if ("1005".equals(factorId)) stdUnit = "mg/m³";
                values.add(reachGoodPollutantData.getValue() + stdUnit);
                names.add(reachGoodPollutantData.getName());
            }
        }
        String nameStr = "XXX";
        if (CollectionUtils.isNotEmpty(names)) {
            nameStr = String.join("、", names);
        }
        String valueStr = "XXX";
        if (CollectionUtils.isNotEmpty(values)) {
            valueStr = String.join("、", values);
        }

        StringBuilder sb = new StringBuilder();
        sb.append(headline1("六、小结"));
        sb.append(textPart("（1） " + DateFormatUtils.format(beginTime, "yyyy年MM月dd日") + "，厦门市AQI指数为" + dayAqiValueStr +
                "，空气质量等级为" + gradeStr + "，" + dayPrimaryPollutantStr + "。当日常规污染物（" + nameStr + "）浓度分别为：" + valueStr + "。"));
        sb.append(textPart("（2） 整体来看，" + DateFormatUtils.format(beginTime, "dd日") + "臭氧/颗粒物污染成因是本地生成/区域传输。"));

        String wdString = weatherData.getWdString();
        sb.append(textPart("（3）从气象因素来看，厦门市天气形势受XX系统影响，地面主导风向为"+(StringUtils.isBlank(wdString) ? "XX风" : wdString)+
                "，天气XX，水平扩散条件XX，垂直扩散条件XX，光化学反应条件XX。"));

        String maxTvocValue = vocData.getMaxTvocValue();
        String avgTvocValue = vocData.getAvgTvocValue();

        sb.append(textPart("（4）" + DateFormatUtils.format(beginTime, "yyyy年MM月dd日") +
                "TVOC峰值浓度"+ (StringUtils.isBlank(maxTvocValue) ? "XX" : maxTvocValue) +"μg/m³，" +
                "日均浓度"+ (StringUtils.isBlank(avgTvocValue) ? "XX" : avgTvocValue) +"μg/m³，" +
                "XX~XX时VOCs浓度明显XX，其中以.......。"));


        List<PollutantData> pollutantData = vocData.getPollutantData();
        String ofp1 = "xxxx";
        String ofp2 = "xxxx";
        String ofp3 = "xxxx";
        for (int i = 0; i < Math.min(pollutantData.size(), 3); i++) {
            PollutantData pollutantData1 = pollutantData.get(i);
            String factorName = pollutantData1.getFactorName();
            Float value = pollutantData1.getIOfp();
            if (0 == i) {
                ofp1 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else if (1 == i) {
                ofp2 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else {
                ofp3 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            }
        }

        List<AvgData> vocTop10AvgDataList = vocData.getVocTop10AvgDataList();
        String text1 = "xxxx";
        String text2 = "xxxx";
        String text3 = "xxxx";
        for (int i = 0; i < Math.min(vocTop10AvgDataList.size(), 3); i++) {
            AvgData avgData = vocTop10AvgDataList.get(i);
            String factorName = avgData.getFactorName();
            Float value = avgData.getValue();
            if (0 == i) {
                text1 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else if (1 == i) {
                text2 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            } else {
                text3 = factorName + "（" + CalculationUtils.digitHalfEvenStr(value, 3) + "μg/m³)";
            }
        }
        sb.append(textPart("（5）VOCs组分测定排名前三的物质为" + text1 + "、" + text2 + "和" + text3 + "，主要来源为.......。"+
                "VOCs物种进行OFP（臭氧生成潜势）测定，排名前三的物质为" + ofp1 + "、" + ofp2 + "和" + ofp3 + "，主要来源为.......。"));

        List<AvgData> liAvgDataList = particleData.getLiAvgDataList();

        String name1 = "XX";
        String name2 = "XX";
        String name3 = "XX";
        String value1 = "XX";
        String value2 = "XX";
        String value3 = "XX";


        for (int i = 0; i < Math.min(liAvgDataList.size(), 3); i++) {
            AvgData avgData = liAvgDataList.get(i);
            String factorName = avgData.getFactorName();
            Float value = avgData.getValue();
            if (0 == i) {
                name1 = factorName;
                value1 = CalculationUtils.digitHalfEvenStr(value, 2);
            } else if (1 == i) {
                name2 = factorName;
                value2 = CalculationUtils.digitHalfEvenStr(value, 2);
            } else {
                name3 = factorName;
                value3 = CalculationUtils.digitHalfEvenStr(value, 2);
            }
        }

        List<AvgData> jsAvgDataList = particleData.getJsAvgDataList();
        jsAvgDataList.sort(Comparator.comparing(AvgData::getValue).reversed());
        double sum = jsAvgDataList.stream().mapToDouble(AvgData::getValue).sum();

        List<String> jsNames = new ArrayList<>();
        List<String> jsValues = new ArrayList<>();
        List<String> jsProportions = new ArrayList<>();
        for (int i = 0; i < Math.min(jsAvgDataList.size(), 6); i++) {
            AvgData avgData = jsAvgDataList.get(i);
            Float value = avgData.getValue();
            jsNames.add(avgData.getFactorName());
            jsValues.add(CalculationUtils.digitHalfEvenStr(value, 2));
            jsProportions.add(CalculationUtils.digitHalfEvenStr(Float.valueOf(String.valueOf((value / sum) * 100)), 2));
        }
        String jsNameStr = "XX、XX、XX、XX、XX、XX";
//        String jsValueStr = "XXng/m³、XXng/m³、XXng/m³、XXng/m³、XXng/m³、XXng/m³";
        String jsProportionStr = "XX%、XX%、XX%、XX%、XX%、XX%";

        if (CollectionUtils.isNotEmpty(jsNames)){
            jsNameStr = String.join("、",jsNames);
        }
//        if (CollectionUtils.isNotEmpty(jsValues)){
//            jsValueStr = String.join("ng/m³、",jsValues) + "ng/m³";
//        }
        if (CollectionUtils.isNotEmpty(jsProportions)){
            jsProportionStr = String.join("%、",jsProportions) + "%";
        }

        sb.append(textPart("（6）颗粒物离子组分中，排名前三的为" + name1 + "、" + name2 + "和" + name3 + "，" +
                "日均浓度分别为" + value1 + "μg/m³、" + value2 + "μg/m³、" + value3 + "，" +
                "XX-XX小时XX组分明显升高，主要来自XX。" +
                "无机元素中，日均浓度排名前六为" + jsNameStr + "，占比分别为"+jsProportionStr+"，其中X元素指示XX，X元素指示XX。"));

//        String klText = "XX、XX、XX、XX、XX、XX、XX、XX、XX";
//        List<AvgData> klAvgDataList = particleData.getKlAvgDataList();
//        if (CollectionUtils.isNotEmpty(klAvgDataList)){
//            List<String> list = new ArrayList<>();
//            for (AvgData avgData : klAvgDataList) {
//                list.add(avgData.getFactorName() + CalculationUtils.digitHalfEvenStr(avgData.getValue(),2) + "%");
//            }
//            klText = String.join("、",list);
//        }
        sb.append(textPart("（8）从臭氧雷达图可知，XXXX。"));
        sb.append(textPart("（9）从颗粒物雷达图可知，XXXX。"));

        return sb.toString();
    }


    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    /**
     * 图片
     *
     * @param width  宽
     * @param height 高
     * @return
     */
    public static String imageTwo(String path1, String path2, Double width, Double height) {
        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 2.34375;
        return "<p style=\"text-align:center;\">" +
                "<img src=\"" + path1 + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "<img src=\"" + path2 + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "</p>";
    }

    /**
     * 图片
     *
     * @param width  宽
     * @param height 高
     * @return
     */
    public static String image(String path, Double width, Double height) {
        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 2.34375;
        return "<p style=\"text-align:center;\">" +
                "<img src=\"" + path + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "</p>";
    }

}
