package more.ssaqms.xiamen.handler.month.airquality;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import more.ssaqms.xiamen.model.airquality.SiteAqiData;
import more.ssaqms.xiamen.model.airquality.SiteComplexData;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.traffic.GradeData;
import more.ssaqms.xiamen.model.traffic.PollutantData;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.*;

@Component
public class AirQualityChapter1 {
    public static Map<String, String> factorNameMap = new HashMap<>();

    static {
        factorNameMap.put("O₃", "臭氧");
        factorNameMap.put("PM₂.₅", "细颗粒物");
        factorNameMap.put("PM₁₀", "可吸入颗粒物");
        factorNameMap.put("NO₂", "二氧化氮");
        factorNameMap.put("SO₂", "二氧化硫");
        factorNameMap.put("CO", "一氧化碳");
    }

    private static String SiteId = "1721488148709826562";

    public String html(TimeRange monthTimeRange, List<SiteAqiData> siteAqiDatas, List<SiteAqiData> monthSiteAqiDatas, SiteComplexData siteComplexData) {
        StringBuilder sb = new StringBuilder();
        sb.append(headline("一、全市环境空气质量"));
        Date beginTime = monthTimeRange.getBeginTime();
        Calendar ca = Calendar.getInstance();
        ca.setTime(beginTime);
        int month = ca.get(Calendar.MONTH) + 1;

        String content = createAqi(siteAqiDatas, String.valueOf(month), 1, beginTime, siteComplexData);
        if (!content.equals("")) {
            sb.append(textBoldPart2(content));
        }

        content = createAqi(monthSiteAqiDatas, MessageFormat.format("1-{0}", month), 2, beginTime, siteComplexData);
        if (!content.equals("")) {
            sb.append(textBoldPart2(content));
        }
        return sb.toString();
    }

    public String createAqi(List<SiteAqiData> siteAqiDatas, String month, int index, Date dateTime, SiteComplexData siteComplexData) {
        if (CollectionUtils.isEmpty(siteAqiDatas)) {
            return "";
        }
        StringBuilder stringBuilder = new StringBuilder();

        //获取厦门市的AQI数据
        Optional<SiteAqiData> siteAqiDataOptional = siteAqiDatas.stream().filter(a -> a.getSiteId().equals("1")).findFirst();
        if (siteAqiDataOptional.isPresent()) {
            SiteAqiData siteAqiData = siteAqiDataOptional.get();
            String indexValue = "xxxx";
            String flag = "下降（转好）/上升（变差）/持平";
            String offset = "xxxx";
            String percent = "xxxx";
            String sort = "xxxx";

            ///1=按月查询真气网综合指数排名 2= 按年查综合指数排名
            if (siteComplexData != null) {
                sort = String.valueOf(siteComplexData.getRank());
            }

            if (siteAqiData.getValue() != null) {
                indexValue = String.valueOf(siteAqiData.getValue());
            }

            if (siteAqiData.getYoyPercent() != null) {
                flag = siteAqiData.getYoyPercent() > 0 ? "上升（变差）" : (siteAqiData.getYoyPercent() < 0 ? "下降（转好）" : "持平");
                percent = String.valueOf(Math.abs(siteAqiData.getYoyPercent()));
            }

            if (siteAqiData.getOffset() != null) {
                offset = String.valueOf(Math.abs(siteAqiData.getOffset()));
            }

            stringBuilder.append(MessageFormat.format("{0}月，厦门市空气质量综合指数{1}，同比{2}{3}（{4}%），在全国168个重点城市中排名第{5}（真气网）。", month, indexValue, flag, offset, percent, sort));

            List<GradeData> gradeDatas = siteAqiData.getGradeDatas();
            List<String> list = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(gradeDatas)) {
                gradeDatas.forEach(gradeData -> {
                    Map<String, Integer> factorDayMap = new HashMap<>();
                    StringBuilder strBuilder = new StringBuilder();
                    List<PollutantData> pollutantDatas = gradeData.getPollutantDatas();
                    List<String> factorDays = new ArrayList<>();
                    if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                        pollutantDatas.forEach(p -> {
                            factorDays.add(MessageFormat.format("{0}{1}天", factorNameMap.containsKey(p.getFactorName()) ? factorNameMap.get(p.getFactorName()) : p.getFactorName(), p.getDay()));
                        });
                    }
                    strBuilder.append(MessageFormat.format("{0}的天数{1}天", gradeData.getGradeName(), gradeData.getDay()));
                    if (CollectionUtils.isNotEmpty(factorDays)) {
                        strBuilder.append(MessageFormat.format("（首要污染物为{0}）", String.join("、", factorDays)));
                    }
                    list.add(strBuilder.toString());
                });
            }

            if (CollectionUtils.isNotEmpty(list)) {
                stringBuilder.append(MessageFormat.format("空气质量指数（AQI）{0}；", String.join("，", list)));
            } else {
                stringBuilder.append("空气质量指数（AQI）优/良/轻度污染的天数xxx天（首要污染物为xxx天）；");
            }

            String rate = "xxxx";
            String rateFlag = "xxxx";
            if (siteAqiData.getExcellentRate() != null) {
                rate = String.valueOf(siteAqiData.getExcellentRate());
            }

            if (siteAqiData.getExcellentRatePercent() != null) {
                if (siteAqiData.getExcellentRatePercent() > 0.0f) {
                    rateFlag = MessageFormat.format("上升（{0}%）", Math.abs(siteAqiData.getExcellentRatePercent()));
                } else if (siteAqiData.getExcellentRatePercent() < 0.0f) {
                    rateFlag = MessageFormat.format("下降（{0}%）", Math.abs(siteAqiData.getExcellentRatePercent()));
                } else {
                    rateFlag = "持平";
                }
            }

            stringBuilder.append(MessageFormat.format("空气质量达标天数比率{0}%，同比{1}（详见附表{2}）。", rate, rateFlag, index));
        }

        return stringBuilder.toString();
    }

    private String textBoldPart2(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>" +
                "</p>";
    }

    private String headline(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }
}
