package more.ssaqms.xiamen.handler.month.airquality;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import more.ssaqms.xiamen.model.airquality.RegionPrimaryData;
import more.ssaqms.xiamen.model.airquality.SiteAqiData;
import more.ssaqms.xiamen.model.report.TimeRange;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

@Component
public class AirQualityChapter2 {
    public static Map<String,String> factorNameMap = new HashMap<>();

    static {
        factorNameMap.put("1006","臭氧");
        factorNameMap.put("1008","细颗粒物");
        factorNameMap.put("1007","可吸入颗粒物");
        factorNameMap.put("1003","二氧化氮");
        factorNameMap.put("1001","二氧化硫");
        factorNameMap.put("1005","一氧化碳");
    }

    public String html(TimeRange monthTimeRange, List<SiteAqiData> regionSiteAqiDatas, List<SiteAqiData> monthRegionSiteAqiDatas){
        StringBuilder sb = new StringBuilder();
        sb.append(headline("二、各行政区环境空气质量"));
        Date beginTime = monthTimeRange.getBeginTime();
        Calendar ca = Calendar.getInstance();
        ca.setTime(beginTime);
        int month = ca.get(Calendar.MONTH)+1;

        String content = createAqi(regionSiteAqiDatas,String.valueOf(month),3);
        if(!content.equals("")){
            sb.append(textBoldPart2(content));
        }

        content = createAqi(monthRegionSiteAqiDatas,MessageFormat.format("1-{0}",String.valueOf(month)),4);
        if(!content.equals("")){
            sb.append(textBoldPart2(content));
        }

        return sb.toString();
    }

    public String createAqi(List<SiteAqiData> siteAqiDatas,String month,int index){
        if(CollectionUtils.isEmpty(siteAqiDatas)){
            return "";
        }
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(MessageFormat.format("{0}月，厦门市",month));
        int count = siteAqiDatas.size();
        //统计每个区域的达标天数比例
        Map<Float,List<SiteAqiData>> rateMap = siteAqiDatas.stream().filter(a->a.getExcellentRate()!=null).collect(Collectors.groupingBy(SiteAqiData::getExcellentRate));
        if(CollectionUtils.isNotEmpty(rateMap)){
            List<String> content = new ArrayList<>();
            List<Float> rates = rateMap.keySet().stream().collect(Collectors.toList());
            Collections.sort(rates, Comparator.reverseOrder());

            for(int i = 0 ;i < rates.size();i++){
                Float rate = rates.get(i);
                if(!rateMap.containsKey(rate)){
                    continue;
                }
                List<SiteAqiData> siteAqiDataList = rateMap.get(rate);
                if(CollectionUtils.isEmpty(siteAqiDataList)){
                    continue;
                }
                List<String> regionNames = siteAqiDataList.stream().map(a->a.getSiteName()).collect(Collectors.toList());
                StringBuilder builder = new StringBuilder();
                builder.append(MessageFormat.format("{0}的环境空气质量达标天数比例{1}{2}%，",String.join("、",regionNames),regionNames.size()>1?"均为":"为",String.valueOf(rate)));
                Map<Float,List<SiteAqiData>> ratePercentMap = siteAqiDataList.stream().collect(Collectors.groupingBy(SiteAqiData::getExcellentRatePercent));
                if(CollectionUtils.isNotEmpty(ratePercentMap)){
                    builder.append("同比");
                    List<String> strPercent = new ArrayList<>();
                    ratePercentMap.forEach((percent,datas)->{
                        if(null == percent || CollectionUtils.isEmpty(datas)){
                            return;
                        }
                        if(datas.size() == siteAqiDataList.size()){
                            strPercent.add(MessageFormat.format("{0}{1}",siteAqiDataList.size() == 1 ? "":"均",createStrPercent(percent)));
                        }
                        else {
                            List<String> names = datas.stream().map(SiteAqiData::getSiteName).collect(Collectors.toList());
                            strPercent.add(MessageFormat.format("{0}{1}", String.join("、", names), createStrPercent(percent)));
                        }
                    });
                    if(CollectionUtils.isNotEmpty(strPercent)){
                        builder.append(String.join("、", strPercent));
                    }
                }
                content.add(builder.toString());
            }

            stringBuilder.append(MessageFormat.format("{0}个行政区中{1}。",count,String.join("；",content)));
        }

        // 统计每个区综合指数范围
        List<Float> indexs = siteAqiDatas.stream().filter(a->a.getValue()!=null).map(SiteAqiData::getValue).collect(Collectors.toList());
        Float max = Collections.max(indexs);
        Float min = Collections.min(indexs);

        List<RegionPrimaryData>regionPrimaryList = new ArrayList<>();
        siteAqiDatas.forEach(a->{
            List<String> ids = a.getPrimaryPollutantFactorIds();
            if(CollectionUtils.isEmpty(ids)){
                return;
            }
            List<String>factorNames = new ArrayList<>();
            ids.forEach(id->{
                if(factorNameMap.containsKey(id)){
                    factorNames.add(factorNameMap.get(id));
                }
            });
            if(CollectionUtils.isNotEmpty(factorNames)){
                RegionPrimaryData data = new RegionPrimaryData();
                data.setFactorName(String.join("、",factorNames));
                data.setRegionName(a.getSiteName());
                regionPrimaryList.add(data);
            }
        });

        String strPrimary = "各区的首要污染物为xxxx";
        if(CollectionUtils.isNotEmpty(regionPrimaryList)){
            Map<String,List<RegionPrimaryData>> primaryMap = regionPrimaryList.stream().collect(Collectors.groupingBy(RegionPrimaryData::getFactorName));
            List<String> list = new ArrayList<>();
            primaryMap.forEach((name,primarys)->{
                if(CollectionUtils.isEmpty(primarys)){
                    return;
                }
                if(primarys.size() == regionPrimaryList.size()){
                    list.add(MessageFormat.format("各区的首要污染物均为{0}",name));
                }
                else {
                    List<String> names = primarys.stream().map(RegionPrimaryData::getRegionName).collect(Collectors.toList());
                    list.add(MessageFormat.format("{0}首要污染物为{1}", String.join("、", names), name));
                }
            });

            if(CollectionUtils.isNotEmpty(list)){
                strPrimary = String.join("、",list);
            }
        }

        stringBuilder.append(MessageFormat.format("{0}个区环境空气质量综合指数范围为{1}~{2}，{3}。",count,min,max,strPrimary));

        List<SiteAqiData> siteAqiDataList = siteAqiDatas.stream().filter(a->a.getValue()!=null).collect(Collectors.toList());
        Map<Float,List<SiteAqiData>> datas = siteAqiDataList.stream().collect(Collectors.groupingBy(SiteAqiData::getValue));
        List<Float> values = datas.keySet().stream().collect(Collectors.toList());

        Collections.sort(values);
        List<String> regionNames = new ArrayList<>();
        values.forEach(a->{
            if(!datas.containsKey(a)){
                return;
            }
            List<SiteAqiData> dataList = datas.get(a);
            if(CollectionUtils.isEmpty(dataList)){
                return;
            }
            List<String> names = dataList.stream().map(b->b.getSiteName()).collect(Collectors.toList());
            if(names.size()>1){
                regionNames.add(MessageFormat.format("{0}并列",String.join("与",names)));
            }
            else{
                regionNames.add(names.get(0));
            }
        });

        stringBuilder.append(MessageFormat.format("各区环境空气质量由好至差依次为：{0}（详见附表{1}）",String.join("、",regionNames),index));
        return stringBuilder.toString();
    }

    private String createStrPercent(Float percent){
        return percent > 0.0f ? MessageFormat.format("上升{0}个百分点",String.valueOf(Math.abs(percent))) : (percent < 0.0f? MessageFormat.format("下降{0}个百分点",String.valueOf(Math.abs(percent))) : "持平");
    }

    private String headline(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    private String textBoldPart2(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>" +
                "</p>";
    }
}
