package more.ssaqms.xiamen.handler.month.airquality;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import more.commons.utils.tool.DateUtils;
import more.ssaqms.xiamen.model.airquality.SiteAqiData;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.traffic.PollutantData;
import more.ssaqms.xiamen.util.CalculationUtils;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

import static more.ssaqms.xiamen.util.MonthReportUtils.table;
import static more.ssaqms.xiamen.util.MonthReportUtils.tr;

@Component
public class AirQualityChapter3 {

    public static final Map<String, String> FACTOR_NAME_MAPPING = new HashMap() {{
        put("1001", "SO₂");
        put("1003", "NO₂");
        put("1006", "O₃");
        put("1007", "PM₁₀");
        put("1008", "PM₂.₅");
        put("1005", "CO");
    }};

    public String html(TimeRange monthTimeRange, List<SiteAqiData> siteAqiDatas, List<SiteAqiData> monthSiteAqiDatas,int index,int index1,String title) {
        StringBuilder sb = new StringBuilder();
        Date beginTime = monthTimeRange.getBeginTime();
        Calendar ca = Calendar.getInstance();
        ca.setTime(beginTime);
        int month = ca.get(Calendar.MONTH)+1;

        String aqiTable = createAqiTable(index,monthTimeRange,String.valueOf(month),siteAqiDatas,title);
        if(!aqiTable.equals("")){
            sb.append(aqiTable);
        }

        aqiTable = createAqiTable(index1,monthTimeRange,MessageFormat.format("1-{0}",month),monthSiteAqiDatas,title);
        if(!aqiTable.equals("")){
            sb.append(aqiTable);
        }
        return sb.toString();
    }

    public String htmlSort(TimeRange monthTimeRange, List<SiteAqiData> siteAqiDatas, List<SiteAqiData> monthSiteAqiDatas,int index,int index1,String title) {
        StringBuilder sb = new StringBuilder();
        Date beginTime = monthTimeRange.getBeginTime();
        Calendar ca = Calendar.getInstance();
        ca.setTime(beginTime);
        int month = ca.get(Calendar.MONTH)+1;

        String aqiTable = createAqiTableSort(index,monthTimeRange,String.valueOf(month),siteAqiDatas,title);
        if(!aqiTable.equals("")){
            sb.append(aqiTable);
        }

        aqiTable = createAqiTableSort(index1,monthTimeRange,MessageFormat.format("1-{0}",month),monthSiteAqiDatas,title);
        if(!aqiTable.equals("")){
            sb.append(aqiTable);
        }
        return sb.toString();
    }

    private String createAqiTable(int index,TimeRange monthTimeRange,String month,List<SiteAqiData> siteAqiDatas,String title){
        if(CollectionUtils.isEmpty(siteAqiDatas)){
            return "";
        }

        StringBuilder trStr = new StringBuilder();
        trStr.append(textLeft(MessageFormat.format("附表{0}：",index)));
        trStr.append(textCenter(MessageFormat.format("{0}{1}月{2}", DateUtils.formatDate(monthTimeRange.getBeginTime(),"yyyy年"),month,title)));
        StringBuilder tdBuffer = new StringBuilder();
        tdBuffer.append(overstrikingTd(1,1,"站点"));
        tdBuffer.append(overstrikingTd(1,1,"SO₂"));
        tdBuffer.append(overstrikingTd(1,1,"NO₂"));
        tdBuffer.append(overstrikingTd(1,1,"CO-95per"));
        tdBuffer.append(overstrikingTd(1,1,"O₃_8h-90per"));
        tdBuffer.append(overstrikingTd(1,1,"PM₁₀"));
        tdBuffer.append(overstrikingTd(1,1,"PM₂.₅"));
        tdBuffer.append(overstrikingTd(1,1,"综合指数"));
        tdBuffer.append(overstrikingTd(1,1,"达标天数比例（%）"));
        tdBuffer.append(overstrikingTd(1,1,"首要污染物"));
        trStr.append(tr(tdBuffer.toString()));

        for(int i=0;i<siteAqiDatas.size();i++){
            SiteAqiData siteAqiData=siteAqiDatas.get(i);
            List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
            Map<String, PollutantData> pollutantDataMapMap = new HashMap<>();

            for (PollutantData pollutantData : pollutantDatas) {
                pollutantDataMapMap.put(pollutantData.getFactorId(), pollutantData);
            }

            tdBuffer = new StringBuilder();
            tdBuffer.append(overstrikingTd(1,1,siteAqiData.getSiteName()));
            tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1001") ? CalculationUtils.digit(pollutantDataMapMap.get("1001").getValue(),0):"-"));
            tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1003") ? CalculationUtils.digit(pollutantDataMapMap.get("1003").getValue(),0):"-"));
            tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1005") ? CalculationUtils.digit(pollutantDataMapMap.get("1005").getValue(),1):"-"));
            tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1006") ? CalculationUtils.digit(pollutantDataMapMap.get("1006").getValue(),0):"-"));
            tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1007") ? CalculationUtils.digit(pollutantDataMapMap.get("1007").getValue(),0):"-"));
            tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1008") ? CalculationUtils.digit(pollutantDataMapMap.get("1008").getValue(),0):"-"));
            tdBuffer.append(overstrikingTd(1,1,null == siteAqiData.getValue() ? "-" : siteAqiData.getValue().toString()));
            tdBuffer.append(overstrikingTd(1,1,null == siteAqiData.getExcellentRate()?"-":siteAqiData.getExcellentRate().toString()));
            String primaryFactor = "-";
            List<String> factorIds = siteAqiData.getPrimaryPollutantFactorIds();
            if(CollectionUtils.isNotEmpty(factorIds)){
                List<String> names = new ArrayList<>();
                factorIds.forEach(a->{
                    String id = a;
                    if(FACTOR_NAME_MAPPING.containsKey(id)){
                        names.add(FACTOR_NAME_MAPPING.get(id));
                    }
                });
                if(CollectionUtils.isNotEmpty(names)){
                    primaryFactor = String.join("、", names);
                }
            }
            tdBuffer.append(overstrikingTd(1,1,primaryFactor));
            trStr.append(tr(tdBuffer.toString()));
        }

        return pageBreak() + table(trStr.toString()) + textPartTwo("备注：","1.综合指数为无量纲，CO浓度单位为mg/m³，其他浓度单位均为μg/m³；","2.综合指数越小，表示环境空气质量相对越好。");
    }

    private String createAqiTableSort(int index,TimeRange monthTimeRange,String month,List<SiteAqiData> siteAqiDatas,String title){
        if(CollectionUtils.isEmpty(siteAqiDatas)){
            return "";
        }

        StringBuilder trStr = new StringBuilder();
        trStr.append(textLeft(MessageFormat.format("附表{0}：",index)));
        trStr.append(textCenter(MessageFormat.format("{0}{1}月{2}", DateUtils.formatDate(monthTimeRange.getBeginTime(),"yyyy年"),month,title)));
        StringBuilder tdBuffer = new StringBuilder();
        tdBuffer.append(overstrikingTd(1,1,"排名"));
        tdBuffer.append(overstrikingTd(1,1,"行政区"));
        tdBuffer.append(overstrikingTd(1,1,"SO₂"));
        tdBuffer.append(overstrikingTd(1,1,"NO₂"));
        tdBuffer.append(overstrikingTd(1,1,"CO-95per"));
        tdBuffer.append(overstrikingTd(1,1,"O₃_8h-90per"));
        tdBuffer.append(overstrikingTd(1,1,"PM₁₀"));
        tdBuffer.append(overstrikingTd(1,1,"PM₂.₅"));
        tdBuffer.append(overstrikingTd(1,1,"综合指数"));
        tdBuffer.append(overstrikingTd(1,1,"达标天数比例（%）"));
        tdBuffer.append(overstrikingTd(1,1,"首要污染物"));
        trStr.append(tr(tdBuffer.toString()));

        List<SiteAqiData> siteAqiDataList = siteAqiDatas.stream().filter(a->a.getValue()!=null).collect(Collectors.toList());
        Map<Float,List<SiteAqiData>> datas = siteAqiDataList.stream().collect(Collectors.groupingBy(SiteAqiData::getValue));
        List<Float> values = datas.keySet().stream().collect(Collectors.toList());

        Collections.sort(values);
        int currentIndex = 1;
        for(int n =0;n < values.size();n++){
            if(!datas.containsKey(values.get(n))){
                continue;
            }
            List<SiteAqiData> dataList = datas.get(values.get(n));
            if(CollectionUtils.isEmpty(dataList)){
                continue;
            }
            for(int i=0;i<dataList.size();i++){
                SiteAqiData siteAqiData=dataList.get(i);
                List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
                Map<String, PollutantData> pollutantDataMapMap = new HashMap<>();

                for (PollutantData pollutantData : pollutantDatas) {
                    pollutantDataMapMap.put(pollutantData.getFactorId(), pollutantData);
                }

                tdBuffer = new StringBuilder();
                tdBuffer.append(overstrikingTd(1,1,String.valueOf(currentIndex)));
                tdBuffer.append(overstrikingTd(1,1,siteAqiData.getSiteName()));
                tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1001") ? CalculationUtils.digit(pollutantDataMapMap.get("1001").getValue(),0):"-"));
                tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1003") ? CalculationUtils.digit(pollutantDataMapMap.get("1003").getValue(),0):"-"));
                tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1005") ? CalculationUtils.digit(pollutantDataMapMap.get("1005").getValue(),1):"-"));
                tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1006") ? CalculationUtils.digit(pollutantDataMapMap.get("1006").getValue(),0):"-"));
                tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1007") ? CalculationUtils.digit(pollutantDataMapMap.get("1007").getValue(),0):"-"));
                tdBuffer.append(overstrikingTd(1,1,pollutantDataMapMap.containsKey("1008") ? CalculationUtils.digit(pollutantDataMapMap.get("1008").getValue(),0):"-"));
                tdBuffer.append(overstrikingTd(1,1,null == siteAqiData.getValue() ? "-" : siteAqiData.getValue().toString()));
                tdBuffer.append(overstrikingTd(1,1,null == siteAqiData.getExcellentRate()?"-":siteAqiData.getExcellentRate().toString()));
                String primaryFactor = "-";
                List<String> factorIds = siteAqiData.getPrimaryPollutantFactorIds();
                if(CollectionUtils.isNotEmpty(factorIds)){
                    List<String> names = new ArrayList<>();
                    factorIds.forEach(a->{
                        String id = a;
                        if(FACTOR_NAME_MAPPING.containsKey(id)){
                            names.add(FACTOR_NAME_MAPPING.get(id));
                        }
                    });
                    if(CollectionUtils.isNotEmpty(names)){
                        primaryFactor = String.join("、", names);
                    }
                }
                tdBuffer.append(overstrikingTd(1,1,primaryFactor));
                trStr.append(tr(tdBuffer.toString()));
            }
            currentIndex += dataList.size();
        }
        return pageBreak() + table(trStr.toString()) + textPartTwo("备注：","1.综合指数为无量纲，CO浓度单位为mg/m³，其他浓度单位均为μg/m³；","2.综合指数越小，表示环境空气质量相对越好。");
    }

    private String overstrikingTd(Integer colspan, Integer rowspan, String text) {
        return "<td colspan=\"" + colspan + "\" rowspan=\"" + rowspan + "\" style=\"text-align:center\">" +
                "<p style=\"text-align:center;\">" +
                "    <span style=\"font-size: 10pt; font-family: 仿宋, SimSun;\">" + text + "</span>" +
                "</p>" +
                "</td>";
    }

    private String textCenter(String text) {
        return "<p style=\"margin: 0px; text-align: center; line-height: 1.1;\">" +
                "<span style=\"font-size: 12pt; font-family: 黑体, SimSun;\">" + text + "</span>" +
                "</p>";
    }

    private String textLeft(String text) {
        return "<p style=\"margin: 0px; text-align: left; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</span>" +
                "</p>";
    }

    public String textPartTwo(String text1, String text2,String text3) {
        return "<p style=\"margin: 0px; text-align: center; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 10.5pt; font-family: 仿宋_GB2312, SimSun;\">" + text1 + "</strong>" +
                "</span>" +
                "<span style=\"font-size: 10.5pt; font-family: 仿宋_GB2312, SimSun;\">" + text2 + "</span>" +
                "<br>"+
                "<span style=\"font-size: 10.5pt; font-family: 仿宋_GB2312, SimSun;\">" + text3 + "</span>" +
                "</p>";
    }

    public String pageBreak(){
        return "<span lang=\"EN-US\" style=\"font-family: 宋体, SimSun;mso-bidi-font-family:KaiTi\"><br clear=\"all\" style=\"mso-special-character:line-break;page-break-before:always\"><o:p></o:p></span>";
    }
}
