package more.ssaqms.xiamen.handler.month.supper;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.commons.utils.tool.DateUtils;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.supper.GradeData;
import more.ssaqms.xiamen.model.supper.PollutantData;
import more.ssaqms.xiamen.model.supper.SiteAqiData;
import more.ssaqms.xiamen.util.CalculationUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class SupperChapter1 {

    public static final Map<String, String> FACTOR_NAME_MAPPING = new HashMap() {{
        put("SO2", "SO₂");
        put("NOX", "NO᙮");
        put("NO2", "NO₂");
        put("O3", "O₃");
        put("PM10", "PM₁₀");
        put("PM2.5", "PM₂.₅");
        put("H8O3", "O₃");
    }};

    public static final Map<String, String> FACTOR_NAME_MA = new HashMap() {{
        put("SO₂", "二氧化流");
        put("CO", "一氧化碳");
        put("NO₂", "二氧化氮");
        put("O₃", "臭氧");
        put("PM₁₀", "细颗粒物");
        put("PM₂.₅", "可吸入颗粒物");
    }};

    public String html(TimeRange monthTimeRange, List<SiteAqiData> siteAqiDataList) {
        Date beginTime = monthTimeRange.getBeginTime();
        if (CollectionUtils.isEmpty(siteAqiDataList)) {
            String sb = headline1("一、常规污染物监测") +
                    textPartTwo("超级站空气质量综合指数--，监测数据有效天数--天，空气质量指数（AQI）优的天数--天；；空气质量优良率--%，优级率--%。",
                            "空气中主要污染物XXX,平均浓度分别为XXX。") +
                    textPart("与去年同期相比，" + DateUtils.formatDate(beginTime, "yyyy年MM月") + "超级站空气质量优良率--%，与上月相比超级站空气质量优良率--%。");
            return sb;
        }
        SiteAqiData siteAqiData = siteAqiDataList.get(0);
        String value = Objects.isNull(siteAqiData.getValue()) ? "--" : siteAqiData.getValue().toString();
        String day = Objects.isNull(siteAqiData.getDay()) ? "--" : siteAqiData.getDay().toString();
        List<GradeData> gradeDatas = siteAqiData.getGradeDatas();
        StringBuilder gradeStr = new StringBuilder();
        String pollutantStr = "";
        if (CollectionUtils.isNotEmpty(gradeDatas)) {
            gradeStr.append("，空气质量指数（AQI）");
            ArrayList<String> list = new ArrayList<>();
            Map<String, Integer> pollutantDataMap = new LinkedHashMap<>();
            for (GradeData gradeData : gradeDatas) {
                list.add(gradeData.getGradeName() + "的天数" + gradeData.getDay() + "天");
                List<PollutantData> pollutantDatas = gradeData.getPollutantDatas();
                if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                    for (PollutantData pollutantData : pollutantDatas) {
                        String factorName = FACTOR_NAME_MA.getOrDefault(pollutantData.getFactorName(), pollutantData.getFactorName());
                        Integer day1 = pollutantData.getDay();
                        pollutantDataMap.put(factorName, pollutantDataMap.getOrDefault(factorName, 0) + day1);
                    }
                }
            }
            gradeStr.append(String.join("，", list));
            List<String> pollutantList = new ArrayList<>();
            for (Map.Entry<String, Integer> entry : pollutantDataMap.entrySet()) {
                String factorName = entry.getKey();
                Integer dayNumb = entry.getValue();
                pollutantList.add(factorName + dayNumb + "天");
            }
            if (CollectionUtils.isNotEmpty(pollutantList)) {
                pollutantStr = String.join("、", pollutantList);
            }
        }
        if (StringUtils.isNotBlank(pollutantStr)) {
            pollutantStr = "（首要污染物为" + pollutantStr + "）";
        }

        String excellentRate = Objects.isNull(siteAqiData.getExcellentRate()) ? "--" : BigDecimal.valueOf(siteAqiData.getExcellentRate()).setScale(2, RoundingMode.HALF_UP).toString();
        String superiorRate = Objects.isNull(siteAqiData.getSuperiorRate()) ? "--" : BigDecimal.valueOf(siteAqiData.getSuperiorRate()).setScale(2, RoundingMode.HALF_UP).toString();

        List<String> pollutantNameList = new ArrayList<>();
        List<String> pollutantValueList = new ArrayList<>();
        List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
        if (CollectionUtils.isNotEmpty(pollutantDatas)) {
            for (PollutantData pollutantData : pollutantDatas) {
                String factorName = pollutantData.getFactorName();
                pollutantNameList.add(FACTOR_NAME_MAPPING.getOrDefault(factorName, factorName));
                String unit = "微克/立方米";
                if (factorName.equalsIgnoreCase("co")) unit = "毫克/立方米";
                pollutantValueList.add(pollutantData.getValue().toString() + " " + unit);
            }
        }
        Float momExcellentRate1 = siteAqiData.getMomExcellentRate();
        Float yoyExcellentRate1 = siteAqiData.getYoyExcellentRate();
        String momExcellentRate = Objects.isNull(momExcellentRate1) ? "（上升/下降）--%" : momExcellentRate1 >= 0 ? momExcellentRate1 == 0f ? "持平" : "上升" + CalculationUtils.digitHalfEven(momExcellentRate1, 2) + "%" : "下降" + (momExcellentRate1 * -1f) + "%";
        String yoyExcellentRate = Objects.isNull(yoyExcellentRate1) ? "（上升/下降）--%" : yoyExcellentRate1 >= 0 ? yoyExcellentRate1 == 0f ? "持平" : "上升" + CalculationUtils.digitHalfEven(yoyExcellentRate1, 2) + "%" : "下降" + (yoyExcellentRate1 * -1f) + "%";

        StringBuilder sb = new StringBuilder();
        sb.append(headline1("一、常规污染物监测"));
        sb.append(textPartTwo("超级站空气质量综合指数" + value + "，监测数据有效天数" + day + "天" + gradeStr + pollutantStr + "；空气质量优良率" + excellentRate + "%，优级率" + superiorRate + "%。",
                "空气中主要污染物" + String.join("、", pollutantNameList) + "平均浓度分别为" + String.join("、", pollutantValueList) + "。"));
        sb.append(textPart("与去年同期相比，" + DateUtils.formatDate(beginTime, "yyyy年MM月") + "超级站空气质量优良率" + yoyExcellentRate + "，与上月相比超级站空气质量优良率" + momExcellentRate + "。"));
        return sb.toString();
    }


    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String textPartTwo(String text1, String text2) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text1 + "</span>" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text2 + "</span>\n" +
                "</p>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

}
