package more.ssaqms.xiamen.handler.month.supper;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.ssaqms.xiamen.model.report.PieData;
import more.ssaqms.xiamen.model.statistics.AvgData;
import more.ssaqms.xiamen.model.supper.*;
import more.ssaqms.xiamen.util.CalculationUtils;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.RestTemplateUtils;
import org.springframework.stereotype.Component;

import java.util.*;

import static more.ssaqms.xiamen.util.MonthReportUtils.image;
import static more.ssaqms.xiamen.util.MonthReportUtils.textCenter;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class SupperChapter2 {

    public String html(List<OcEcData> ocEcDataList, List<HeavyMetalData> heavyMetalDataList,
                       List<BlackCarbonData> blackCarbonDataList, List<IonData> ionDataList, List<LidarData> lidarDataList, List<SpectrometryData> spectrometryDataList) {
        StringBuilder sb = new StringBuilder();
        sb.append(headline1("二、颗粒物组分网监测"));
        sb.append(headline2("（一）元素碳/有机碳"));
        OcEcData ocEcData = new OcEcData();
        if (CollectionUtils.isNotEmpty(ocEcDataList)){
            ocEcData = ocEcDataList.get(0);
        }

        Float pocAvg = ocEcData.getPocAvg();
        Float socAvg = ocEcData.getSocAvg();
        Float tcAvg = ocEcData.getTcAvg();
        String ocMinStr = Objects.isNull(ocEcData.getOcMin()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getOcMin(), 2);
        String ocMaxStr = Objects.isNull(ocEcData.getOcMax()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getOcMax(), 2);
        String ocAvgStr = Objects.isNull(ocEcData.getOcAvg()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getOcAvg(), 2);
        String ecMinStr = Objects.isNull(ocEcData.getEcMin()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getEcMin(), 2);
        String ecMaxStr = Objects.isNull(ocEcData.getEcMax()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getEcMax(), 2);
        String ecAvgStr = Objects.isNull(ocEcData.getEcAvg()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getEcAvg(), 2);
        String ocEcAvgStr = Objects.isNull(ocEcData.getOcEcAvg()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getOcEcAvg(), 2);
        String tcAvgStr = Objects.isNull(ocEcData.getTcAvg()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getTcAvg(), 2);
        String pocAvgStr = Objects.isNull(pocAvg) ? "--" : CalculationUtils.digitHalfEvenStr(pocAvg, 2);
        String socAvgStr = Objects.isNull(ocEcData.getSocAvg()) ? "--" : CalculationUtils.digitHalfEvenStr(ocEcData.getSocAvg(), 2);
        String pocTc = "--";
        String socTc = "--";
        if (!"--".equals(tcAvgStr)) {
            if (!"--".equals(pocAvgStr)) {
                pocTc = CalculationUtils.digitHalfEvenStr(pocAvg * 100 / tcAvg, 1);
            }
            if (!"--".equals(socAvgStr)) {
                socTc = CalculationUtils.digitHalfEvenStr(socAvg * 100 / tcAvg, 1);
            }
        }

        sb.append(textPart("超级站点位有机碳（OC）的日均浓度范围为" + ocMinStr + "~" + ocMaxStr + " μg/m³，月均浓度为" + ocAvgStr + " μg/m³；" +
                "元素碳（EC）的日均浓度范围为" + ecMinStr + "~" + ecMaxStr + " μg/m³，月均浓度为" + ecAvgStr + " μg/m³；" +
                "总碳（TC）的月均浓度为" + tcAvgStr + "μg/m³，其中POC的平均浓度为" + pocAvgStr + " μg/m³，" +
                "占TC的" + pocTc + "%；SOC的平均浓度为" + socAvgStr + " μg/m³，占TC的" + socTc + "%。OC/EC的值为" + ocEcAvgStr + "，表明颗粒物主要受XXX的影响。"));

        sb.append(image(ocEcData.getChartUrl(), null, null));

        sb.append(textCenter("图2-1 OC、EC日均浓度变化特征"));

//        HeavyMetalData heavyMetalData = heavyMetalDataList.get(0);
        HeavyMetalData heavyMetalData = new HeavyMetalData();
        if (CollectionUtils.isNotEmpty(heavyMetalDataList)){
            heavyMetalData = heavyMetalDataList.get(0);
        }

        List<String> factorNames = heavyMetalData.getFactorNames();
        String factorNameStr = null;
        int numb = 5;
        if (CollectionUtils.isNotEmpty(factorNames)) {
            factorNameStr = String.join("、", factorNames);
            numb = factorNames.size();
        } else {
            factorNameStr = "XX、XX、XX、XX、XX";
        }
        String valueStr = null;
        List<String> values = heavyMetalData.getValues();
        if (CollectionUtils.isNotEmpty(factorNames)) {
            valueStr = String.join("、", values);
        } else {
            valueStr = "--、--、--、--、--";
        }

        sb.append(headline2("（二）无机元素"));

        sb.append(textPart("超级站月均浓度排名前" + numb + "的无机元素为" + factorNameStr + "，" +
                "浓度分别为" + valueStr + "。 "));

        sb.append(image(heavyMetalData.getChartUrl(), null, null));
        sb.append(textCenter("图2-2 PM2.5中无机元素前五月均浓度"));

        BlackCarbonData blackCarbonData = new BlackCarbonData();
        if (CollectionUtils.isNotEmpty(blackCarbonDataList)){
            blackCarbonData = blackCarbonDataList.get(0);
        }

        Float min = blackCarbonData.getMin();
        Float max = blackCarbonData.getMax();
        Float avg = blackCarbonData.getAvg();
        Float mom = blackCarbonData.getMom();
        Float yoy = blackCarbonData.getYoy();

        String yoyRateStr = "--%";
        String momRateStr = "--%";
        if (!Objects.isNull(avg)) {
            if (!Objects.isNull(mom)) {
                float momRate = ((avg - mom) * 100) / avg;
                String s = momRate > 0 ? "增加了" : "減少了";
                momRateStr = s + CalculationUtils.digitHalfEvenStr(Math.abs(momRate), 2) + "%";
                if (momRate == 0f) momRateStr = "持平";

            }
            if (!Objects.isNull(yoy)) {
                float yoyRate = ((avg - yoy) * 100) / avg;
                String s = yoyRate > 0 ? "增加" : "減少";
                yoyRateStr = s + CalculationUtils.digitHalfEvenStr(Math.abs(yoyRate), 2) + "%";
                if (yoyRate == 0f) yoyRateStr = "持平";
            }
        }

        sb.append(headline2("（三）黑碳"));
        sb.append(textPartTwo("以超站有效数据进行评价，厦门市黑碳浓度日均浓度跨度范围为" +
                        (Objects.isNull(min) ? "--" : CalculationUtils.digitHalfEvenStr(min / 1000, 2)) + "~" +
                        (Objects.isNull(max) ? "--" : CalculationUtils.digitHalfEvenStr(max / 1000, 2)) +
                        " μg/m³，月均浓度为" + (Objects.isNull(avg) ? "--" : CalculationUtils.digitHalfEvenStr(avg / 1000, 2)) + " μg/m³。",
                "与上月相比，BC浓度" + momRateStr + "，与去年同期相比，BC浓度" + yoyRateStr + "。 "));

        sb.append(image(blackCarbonData.getChartUrl(), null, null));
        sb.append(textCenter("图2-4 厦门市超站黑碳日均浓度变化特征"));

//        IonData ionData = ionDataList.get(0);
        IonData ionData = new IonData();
        if (CollectionUtils.isNotEmpty(ionDataList)){
            ionData = ionDataList.get(0);
        }

        List<AvgData> top3DataList = ionData.getTop3DataList();
        if (CollectionUtils.isEmpty(top3DataList)){
            top3DataList = new ArrayList<>();
        }
        String top3FactorNames = "XX、XX和XX";
        String top3FactorValues = "--μg/m³、--μg/m³和--μg/m³";
        float top3Sum = 0F;
        float top1 = 0F;
        float top2 = 0F;
        float top3 = 0F;
        String top1Name = "XX";
        String top2Name = "XX";
        String top3Name = "XX";

        for (int i = 0; i < top3DataList.size(); i++) {
            AvgData avgData = top3DataList.get(i);
            String factorName = avgData.getFactorName();
            String[] split = factorName.split("\\(");
            if (split.length==2){
                factorName = split[1].replaceAll("\\)","");
            }
            Float value = avgData.getValue();
            top3Sum += value;
            if (0 == i) {
                top3FactorNames = factorName;
                top3FactorValues = CalculationUtils.digitHalfEvenStr(value, 2) + "μg/m³";
                top1 = value;
                top1Name = factorName;
            } else if (1 == i) {
                top3FactorNames += "、" + factorName;
                top3FactorValues += "、" + CalculationUtils.digitHalfEvenStr(value, 2) + "μg/m³";
                top2 = value;
                top2Name = factorName;
            } else if (2 == i) {
                top3FactorNames += "和" + factorName;
                top3FactorValues += "和" + CalculationUtils.digitHalfEvenStr(value, 2) + "μg/m³";
                top3 = value;
                top3Name = factorName;
            }
        }
        List<AvgData> cationDataList = ionData.getCationDataList();
        Map<String, String> map = getStringStringMap(cationDataList);
        String cationMax = map.get("max");
        String cationOther = map.get("other");

        List<AvgData> anionDataList = ionData.getAnionDataList();
        Map<String, String> anionMap = getStringStringMap(anionDataList);
        String anionMax = anionMap.get("max");
        String anionOther = anionMap.get("other");

        sb.append(headline2("（四）水溶性离子"));
        sb.append(textPart("超级站月均浓度排名前三的水溶性离子为" + top3FactorNames + "，浓度分别为" + top3FactorValues + "。"));
        sb.append(textPart("水溶性离子中" + top1Name + "浓度占比最大，占比为" + (top3Sum == 0F ? "--%" : CalculationUtils.digitHalfEvenStr((top1 * 100) / top3Sum, 2) + "%") + "；其次是" + top2Name + "和" + top3Name + "，占比分别为" + (top3Sum == 0F ? "--%" : CalculationUtils.digitHalfEvenStr((top2 * 100) / top3Sum, 2) + "%") + "和" + (top3Sum == 0F ? "--%" : CalculationUtils.digitHalfEvenStr((top3 * 100) / top3Sum, 2) + "%") + "。" +
                "其中阳离子中占比最大的是" + cationMax + cationOther + "；" +
                "阴离子中占比最大的是" + anionMax + anionOther + "。"));

        sb.append(image(ionData.getChartUrl(), null, null));

        sb.append(textCenter("图2-3 水溶性离子日均浓度变化特"));

        sb.append(headline2("（五）单颗粒质谱"));
//        SpectrometryData spectrometryData = spectrometryDataList.get(0);
        SpectrometryData spectrometryData = new SpectrometryData();
        if (CollectionUtils.isNotEmpty(spectrometryDataList)){
            spectrometryData = spectrometryDataList.get(0);
        }

        List<String> list = spectrometryData.getList();
        String text = "二次无机源--%、船舶污染源--%、燃煤污染源--%、生物质燃烧源--%、扬尘污染源--%、移动源--%、其他源--%、工业污染源--%、海盐源--%";
        if (CollectionUtils.isNotEmpty(list)){
            text =  String.join("、",list);
        }
        sb.append(textPart("超级站监测到的污染源占比从高到低依次为"+text+"。"));
        sb.append(image(spectrometryData.getChartUrl(), null, null));
        sb.append(textCenter("图2-5 颗粒物质谱来源时序分布特征"));

        sb.append(image(spectrometryData.getPieChartUrl(), null, null));
        sb.append(textCenter("图2-6 单颗粒质谱来源占比"));

        sb.append(headline2("（六）颗粒物激光雷达"));
        sb.append(textPart("颗粒物激光雷达图显示：X月X日-X日 XXkm以下高空消光系数显著增大，表明本地受细颗粒物影响；" +
                "X月X日-X日 XXkm以下消光系数处于低值，退偏值升高，表明近地面粗颗粒物浓度增大；" +
                "XX月XX日-XX日 XXkm以下退偏值显著增大，对应沙尘传输时段，" +
                "XX日凌晨高空 XXkm以上可见沙尘的高空传输；XX月XX日 XXkm以下消光系数突增，对应地面细颗粒物浓度的突增。"));
//        LidarData lidarData = lidarDataList.get(0);
        LidarData lidarData = new LidarData();
        if (CollectionUtils.isNotEmpty(lidarDataList)){
            lidarData = lidarDataList.get(0);
        }


        sb.append(image(lidarData.getChartDataExtinctionUrl(), null, null));
        sb.append(image(lidarData.getChartDataDepolarizationUrl(), null, null));
        sb.append(textCenter("图2-7 颗粒物激光雷达时间序列图"));
        return sb.toString();
    }

    public Map<String, String> getStringStringMap(List<AvgData> cationDataList) {
        Map<String, String> map = new HashMap<>();
        String cationMax = "XX，占比为--%";
        String cationOther = "，其次是XX、XX、XX和XX，占比分别为--%、--%、--%和--%";

        if (CollectionUtils.isNotEmpty(cationDataList)) {
            float maxValue = 0f;
            float lastValue = 0f;
            float sumValue = 0f;
            String lastName = null;
            List<String> otherNames = new ArrayList<>();
            List<Float> otherValues = new ArrayList<>();
            for (int i = 0; i < cationDataList.size(); i++) {
                AvgData avgData = cationDataList.get(i);
                String factorName = avgData.getFactorName();
                String[] split = factorName.split("\\(");
                if (split.length==2){
                    factorName = split[1].replaceAll("\\)","");
                }
                Float value = avgData.getValue();
                sumValue += value;
                if (0 == i) {
                    cationMax = factorName + "，占比为";
                    maxValue = value;
                } else if (i == (cationDataList.size() - 1)) {
                    lastValue = value;
                    lastName = factorName;
                } else {
                    otherNames.add(factorName);
                    otherValues.add(value);
                }
            }
            if (sumValue > 0f) {
                cationMax += CalculationUtils.digitHalfEvenStr(maxValue * 100 / sumValue, 2) + "%";
            }
            if (StringUtils.isNotBlank(lastName)) {
                if (CollectionUtils.isNotEmpty(otherNames)) {
                    List<String> names = new ArrayList<>();
                    List<String> valueList = new ArrayList<>();
                    for (int i = 0; i < otherNames.size(); i++) {
                        String name = otherNames.get(i);
                        Float value = otherValues.get(i);
                        names.add(name);
                        valueList.add(CalculationUtils.digitHalfEvenStr(value * 100 / sumValue, 2) + "%");
                    }


                    cationOther = "，其次是" + String.join("、", names) + "和" + lastName + "占比分别为" + String.join("、", valueList) + "和" + CalculationUtils.digitHalfEvenStr(lastValue * 100 / sumValue, 2) + "%";
                } else {
                    cationOther = "，其次是" + lastName + "占比为" + CalculationUtils.digitHalfEvenStr(lastValue * 100 / sumValue, 2) + "%";
                }
            }
        }
        map.put("max", cationMax);
        map.put("other", cationOther);
        return map;
    }


    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }

    public String textPartTwo(String text1, String text2) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text1 + "</span>" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text2 + "</span>\n" +
                "</p>";
    }

}
