package more.ssaqms.xiamen.handler.month.supper;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import more.ssaqms.xiamen.model.report.PieData;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.statistics.AvgData;
import more.ssaqms.xiamen.model.statistics.PollutantData;
import more.ssaqms.xiamen.model.supper.HtcData;
import more.ssaqms.xiamen.model.supper.OzoneRadarData;
import more.ssaqms.xiamen.model.supper.VocsData;
import more.ssaqms.xiamen.model.supper.WeatherData;
import more.ssaqms.xiamen.util.CalculationUtils;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.RestTemplateUtils;
import org.springframework.stereotype.Component;

import java.util.*;

import static more.ssaqms.xiamen.util.MonthReportUtils.image;
import static more.ssaqms.xiamen.util.MonthReportUtils.textCenter;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class SupperChapter3 {

    public String html(List<HtcData> htcDataList, List<VocsData> vocsDataList,
                       List<OzoneRadarData> ozoneRadarDataList, List<WeatherData> weatherDataList) {
        StringBuilder sb = new StringBuilder();
        sb.append(headline1("三、光化学组分网监测"));

        HtcData htcData = new HtcData();
        if (CollectionUtils.isNotEmpty(htcDataList)){
            htcData = htcDataList.get(0);
        }

        String maxCh4Str = Objects.isNull(htcData.getMaxCh4()) ? "--" : htcData.getMaxCh4().toString();
        String minCh4Str = Objects.isNull(htcData.getMinCh4()) ? "--" : htcData.getMinCh4().toString();
        String avgCh4Str = Objects.isNull(htcData.getAvgCh4()) ? "--" : CalculationUtils.digitHalfEvenStr(htcData.getAvgCh4(),2);
        String maxNmhcStr = Objects.isNull(htcData.getMaxNmhc()) ? "--" : htcData.getMaxNmhc().toString();
        String minNmhcStr = Objects.isNull(htcData.getMinNmhc()) ? "--" : htcData.getMinNmhc().toString();
        String avgNmhcStr = Objects.isNull(htcData.getAvgNmhc()) ? "--" : CalculationUtils.digitHalfEvenStr(htcData.getAvgNmhc(),2);


        sb.append(headline2("（一）甲烷非甲烷总烃 "));
        sb.append(textPart("以超站有效数据进行评价，甲烷日均浓度跨度范围为" + minCh4Str + "~" + maxCh4Str + " ppb，月均浓度为" + avgCh4Str + "ppb；" +
                "非甲烷总烃日均浓度跨度范围为" + minNmhcStr + "~" + maxNmhcStr + " ppb，月均浓度为" + avgNmhcStr + " ppb。"));
        sb.append(image(htcData.getChartUrl(), null, null));
        sb.append(textCenter("图3-1 甲烷、非甲烷总烃 浓度时序图"));

        VocsData vocsData = new VocsData();
        if (CollectionUtils.isNotEmpty(vocsDataList)){
            vocsData = vocsDataList.get(0);
        }
        Float avg = vocsData.getAvg();
        String maxVocStr = Objects.isNull(vocsData.getMax()) ? "--" : CalculationUtils.digitHalfEvenStr(vocsData.getMax(),2);
        String minVocStr = Objects.isNull(vocsData.getMin()) ? "--" : CalculationUtils.digitHalfEvenStr(vocsData.getMin(),2);
        String avgVocStr = Objects.isNull(avg) ? "--" : CalculationUtils.digit(avg,2);

        List<AvgData> monthDataList = vocsData.getMonthDataList();

        String zhanbi = "烷烃占比为--%、OVOCs占比--%、卤代烃占比--%、芳香烃占比--%、烯烃占比--%、炔烃占比--%";

        String a99000 = "";
        if (CollectionUtils.isNotEmpty(monthDataList)){
            List<String> list = new ArrayList<>();
            for (AvgData avgData : monthDataList) {
                String factorId = avgData.getFactorId();
                String factorName = avgData.getFactorName();
                Float value = avgData.getValue();
                String valueStr = "--%";
                if (!Objects.isNull(avg) && 0f != avg){
                    valueStr = CalculationUtils.digit(value * 100 / avg,2) + "%";
                }
                if ("a99000".equals(factorId)){
                    a99000 = factorName + "占比为" + valueStr;
                }else {
                    list.add(factorName + "占比为" + valueStr);
                }

            }
            list.add(a99000);
            zhanbi = String.join("、",list);
        }


        sb.append(headline2("（二）挥发性有机物"));
        sb.append(textPart("以超级站VOCs有效数据进行评价，厦门市VOCs跨度范围为："+minVocStr+"~"+maxVocStr+" μg/m³，月均浓度为"+avgVocStr+" μg/m³。" +
                "VOCs检出浓度占比最高的是"+zhanbi+"。"));

        sb.append(image(vocsData.getChartUrl(), null, null));

        sb.append(image(vocsData.getPicChartUrl(), null, null));
        sb.append(textCenter("图3-2 厦门市超站VOCs浓度时序图和组分占比"));

        List<PollutantData> ofpList = vocsData.getOfpList();
        String ofpStr = "XX和XX";
        List<String> strings = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(ofpList)){
            ofpList.sort(Comparator.comparing(PollutantData::getIOfp).reversed());
            for (int i = 0; i < Math.min(ofpList.size(),2); i++) {
                PollutantData pollutantData = ofpList.get(i);
                strings.add(pollutantData.getFactorName());
            }
            ofpStr = String.join("和",strings);
        }


        List<PollutantData> soaList = vocsData.getSoaList();
        String soaStr = "XX和XX";
        strings = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(ofpList)){
            soaList.sort(Comparator.comparing(PollutantData::getISoa).reversed());
            for (int i = 0; i < Math.min(soaList.size(),2); i++) {
                PollutantData pollutantData = soaList.get(i);
                strings.add(pollutantData.getFactorName());
            }
            soaStr = String.join("和",strings);
        }

        sb.append(textPart("以超级站VOCs有效数据进行评价，厦门市OFP排名前十中，"+ofpStr+"占主导；" +
                "SOA排名前十中，"+soaStr+"排名最前列。XX和XX主要来自溶剂使用。"));

        sb.append(image(vocsData.getOfpChartUrl(), null, null));
        sb.append(textCenter("图3-3 厦门市超站OFP浓度前十"));


        sb.append(image(vocsData.getSoaChartUrl(), null, null));
        sb.append(textCenter("图3-4 厦门市超站SOA浓度前十"));

        sb.append(headline2("（三）臭氧激光雷达"));

        sb.append(textPart("臭氧激光雷达在X月X日前后，大气边界层高度较低，不利于臭氧的垂直扩散，近地面臭氧浓度相对较高。" +
                "X日后边界层告诉升高，XXkm以下可见臭氧浓度日变化特征增强，但臭氧浓度整体较低。"));
        OzoneRadarData ozoneRadarData = new OzoneRadarData();
        if (CollectionUtils.isNotEmpty(ozoneRadarDataList)){
            ozoneRadarData = ozoneRadarDataList.get(0);
        }
        sb.append(image(ozoneRadarData.getChartUrl(), null, null));
        sb.append(textCenter("图3-5 臭氧激光雷达时间序列图"));

        sb.append(headline2("（四）气象分析"));
        WeatherData weatherData = new WeatherData();
        if (CollectionUtils.isNotEmpty(weatherDataList)){
            weatherData = weatherDataList.get(0);
        }
        String wdStr = weatherData.getWdStr();
        sb.append(textPart("超级站常规气象参数时间序列见图3-6。"));
        sb.append(textPart("大气超级站小时相对湿度分布在"+weatherData.getSdMinStr()+"-"+weatherData.getSdMaxStr()+"%之间，平均相对湿度为"+weatherData.getSdAvgStr()+"%；" +
                "小时风速值分布在"+weatherData.getWsMinStr()+"-"+weatherData.getWsMaxStr()+" m/s之间，平均风速为"+weatherData.getWsAvgStr()+"m/s，主导风向为"+wdStr+"；" +
                "温度分布在"+weatherData.getWdMinStr()+"-"+weatherData.getWdMaxStr()+"℃之间，平均温度为"+weatherData.getWdAvgStr()+"℃；" +
                "气压值分布在"+weatherData.getQyMinStr()+"-"+weatherData.getQyMaxStr()+"kPa之间。"));

        sb.append(image(weatherData.getChartUrl(), null, null));
        sb.append(textCenter("图3-6 超级站气象参数时间序列"));

        return sb.toString();
    }

    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }

    public String textPartTwo(String text1, String text2) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text1 + "</span>" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text2 + "</span>\n" +
                "</p>";
    }

}
