package more.ssaqms.xiamen.handler.month.supper;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.statistics.AvgData;
import more.ssaqms.xiamen.model.supper.*;
import more.ssaqms.xiamen.util.CalculationUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * @author zuo
 * @date 2023/6/16 9:53
 */
@Component
public class SupperChapter4 {

    public static final Map<String, String> FACTOR_NAME_MAPPING = new HashMap() {{
        put("SO2", "SO₂");
        put("NOX", "NO᙮");
        put("NO2", "NO₂");
        put("O3", "O₃");
        put("PM10", "PM₁₀");
        put("PM2.5", "PM₂.₅");
        put("H8O3", "O₃");
    }};

    public String html(List<SiteAqiData> siteAqiDataList, List<OcEcData> ocEcDataList, List<HeavyMetalData> heavyMetalDataList, List<BlackCarbonData> blackCarbonDataList,
                       List<SpectrometryData> spectrometryDataList, List<HtcData> htcDataList, List<VocsData> vocsDataList, List<WeatherData> weatherDataList) {

        SiteAqiData siteAqiData = new SiteAqiData();
        if (CollectionUtils.isNotEmpty(siteAqiDataList)){
            siteAqiData = siteAqiDataList.get(0);
        }
        String value = Objects.isNull(siteAqiData.getValue()) ? "--" : siteAqiData.getValue().toString();
        String day = Objects.isNull(siteAqiData.getDay()) ? "--" : siteAqiData.getDay().toString();
        List<GradeData> gradeDatas = siteAqiData.gradeDatas;
        StringBuilder gradeStr = new StringBuilder();
        String pollutantStr = "";
        if (CollectionUtils.isNotEmpty(gradeDatas)) {
            gradeStr.append("，空气质量指数（AQI）");
            ArrayList<String> list = new ArrayList<>();
            Map<String, Integer> pollutantDataMap = new LinkedHashMap<>();
            for (GradeData gradeData : gradeDatas) {
                list.add(gradeData.getGradeName() + "的天数" + gradeData.getDay() + "天");
                List<PollutantData> pollutantDatas = gradeData.getPollutantDatas();
                if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                    for (PollutantData pollutantData : pollutantDatas) {
                        String factorName = pollutantData.getFactorName();
                        Integer day1 = pollutantData.getDay();
                        pollutantDataMap.put(factorName, pollutantDataMap.getOrDefault(factorName, 0) + day1);
                    }
                }
            }
            gradeStr.append(String.join("，", list));
            List<String> pollutantList = new ArrayList<>();
            for (Map.Entry<String, Integer> entry : pollutantDataMap.entrySet()) {
                String factorName = entry.getKey();
                Integer dayNumb = entry.getValue();
                pollutantList.add(factorName + dayNumb + "天");
            }
            if (CollectionUtils.isNotEmpty(pollutantList)) {
                pollutantStr = String.join("、", pollutantList);
            }
        }
        if (StringUtils.isNotBlank(pollutantStr)) {
            pollutantStr = "（首要污染物为" + pollutantStr + "）";
        }

        String excellentRate = Objects.isNull(siteAqiData.getExcellentRate()) ? "--" : BigDecimal.valueOf(siteAqiData.getExcellentRate()).setScale(2, RoundingMode.HALF_UP).toString();
        String superiorRate = Objects.isNull(siteAqiData.getSuperiorRate()) ? "--" : BigDecimal.valueOf(siteAqiData.getSuperiorRate()).setScale(2, RoundingMode.HALF_UP).toString();

        List<String> pollutantNameList = new ArrayList<>();
        List<String> pollutantValueList = new ArrayList<>();
        List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
        if (CollectionUtils.isNotEmpty(pollutantDatas)) {
            for (PollutantData pollutantData : pollutantDatas) {
                String factorName = pollutantData.getFactorName();
                pollutantNameList.add(FACTOR_NAME_MAPPING.getOrDefault(factorName, factorName));
                String unit = "微克/立方米";
                if (factorName.equalsIgnoreCase("co")) unit = "毫克/立方米";
                pollutantValueList.add(pollutantData.getValue().toString() + " " + unit);
            }
        }
        StringBuilder sb = new StringBuilder();
        sb.append(headline1("四、结论"));
        sb.append(headline2("（一）常规指标结论"));
        sb.append(textPartTwo("超级站空气质量综合指数" + value + gradeStr + pollutantStr + "；空气质量优良率" + excellentRate + "%，优级率" + superiorRate + "%。",
                "空气中主要污染物" + String.join("、", pollutantNameList) + "平均浓度分别为" + String.join("、", pollutantValueList) + "。"));
        sb.append(headline2("（二）颗粒物组分结论"));


        String ocAvgStr = "--";
        String ecAvgStr = "--";
        if (CollectionUtils.isNotEmpty(ocEcDataList)) {
            OcEcData ocEcData = ocEcDataList.get(0);
            ocAvgStr = Objects.isNull(ocEcData.getOcAvg()) ? "--" : CalculationUtils.digit(ocEcData.getOcAvg(), 3);
            ecAvgStr = Objects.isNull(ocEcData.getEcAvg()) ? "--" : CalculationUtils.digit(ocEcData.getEcAvg(), 3);
        }

        sb.append(textPart("1. 超级站有机碳（OC）的月均浓度为" + ocAvgStr + " μg/m3；元素碳（EC）的月均浓度为" + ecAvgStr + " μg/m3。"));


        HeavyMetalData heavyMetalData = new HeavyMetalData();
        if (CollectionUtils.isNotEmpty(heavyMetalDataList)){
            heavyMetalData = heavyMetalDataList.get(0);
        }
        List<String> factorNames = heavyMetalData.getFactorNames();
        String factorNameStr = null;
        int numb = 5;
        if (CollectionUtils.isNotEmpty(factorNames)) {
            factorNameStr = String.join("、", factorNames);
            numb = factorNames.size();
        } else {
            factorNameStr = "XX、XX、XX、XX、XX";
        }
        String valueStr = null;
        List<String> values = heavyMetalData.getValues();
        if (CollectionUtils.isNotEmpty(factorNames)) {
            valueStr = String.join("、", values);
        } else {
            valueStr = "--、--、--、--、--";
        }
        sb.append(textPart("2. 超级站月均浓度排名前" + numb + "的无机元素为" + factorNameStr + "，" +
                "浓度分别为" + valueStr + "。 "));

        BlackCarbonData blackCarbonData = new BlackCarbonData();
        if (CollectionUtils.isNotEmpty(blackCarbonDataList)){
            blackCarbonData = blackCarbonDataList.get(0);
        }
        Float min = blackCarbonData.getMin();
        Float max = blackCarbonData.getMax();
        Float avg = blackCarbonData.getAvg();
        Float mom = blackCarbonData.getMom();
        Float yoy = blackCarbonData.getYoy();

        String yoyRateStr = "--%";
        String momRateStr = "--%";
        if (!Objects.isNull(avg)){
            if (!Objects.isNull(mom)){
                float momRate = ((avg - mom) * 100) / avg;
                String s = momRate > 0 ? "增加" : "減少";
                momRateStr = s + CalculationUtils.digit(momRate, 2) + "%";
                if (momRate == 0){
                    momRateStr = "持平";
                }

            }
            if (!Objects.isNull(yoy)){
                float yoyRate = ((avg - yoy) * 100) / avg;
                String s = yoyRate > 0 ? "增加" : "減少";
                yoyRateStr = s + CalculationUtils.digit(yoyRate, 2) + "%";
                if (yoyRate == 0){
                    yoyRateStr = "持平";
                }
            }
        }
        sb.append(textPartTwo("3. 黑碳浓度跨度范围为" + (Objects.isNull(min) ? "--" : CalculationUtils.digit(min/1000,2)) + "~" + (Objects.isNull(max) ? "--" : CalculationUtils.digit(max/1000,2)) + " μg/m3，月均浓度为" + (Objects.isNull(avg) ? "--" : CalculationUtils.digit(avg/1000,2)) + " μg/m3。",
                "与上月相比，BC浓度"+momRateStr+"，与去年同期相比，BC浓度"+yoyRateStr+"。 "));

        SpectrometryData spectrometryData = new SpectrometryData();
        if (CollectionUtils.isNotEmpty(spectrometryDataList)){
            spectrometryData = spectrometryDataList.get(0);
        }
        List<String> list = spectrometryData.getList();
        String text = "二次无机源--%、船舶污染源--%、燃煤污染源--%、生物质燃烧源--%、扬尘污染源--%、移动源--%、其他源--%、工业污染源--%、海盐源--%";
        if (CollectionUtils.isNotEmpty(list)){
            text =  String.join("、",list);
        }
        sb.append(textPart("4.厦门市主要污染源占比为"+text+"。"));


        sb.append(textPart("5. 颗粒物激光雷达在X月X日-X日、X月X日-X日、X月X日-X日、X月X日可见近地面颗粒物浓度较高。"));
        sb.append(headline2("（三）光化学组分结论"));
        HtcData htcData = new HtcData();
        if (CollectionUtils.isNotEmpty(htcDataList)){
            htcData = htcDataList.get(0);
        }
        String maxCh4Str = Objects.isNull(htcData.getMaxCh4()) ? "--" : htcData.getMaxCh4().toString();
        String minCh4Str = Objects.isNull(htcData.getMinCh4()) ? "--" : htcData.getMinCh4().toString();
        String avgCh4Str = Objects.isNull(htcData.getAvgCh4()) ? "--" : CalculationUtils.digitHalfEvenStr(htcData.getAvgCh4(),2);
        String maxNmhcStr = Objects.isNull(htcData.getMaxNmhc()) ? "--" : htcData.getMaxNmhc().toString();
        String minNmhcStr = Objects.isNull(htcData.getMinNmhc()) ? "--" : htcData.getMinNmhc().toString();
        String avgNmhcStr = Objects.isNull(htcData.getAvgNmhc()) ? "--" : CalculationUtils.digitHalfEvenStr(htcData.getAvgNmhc(),2);
        sb.append(textPart("1.厦门市甲烷日均浓度跨度范围为" + minCh4Str + "~" + maxCh4Str + " ppb，月均浓度为" + avgCh4Str + "ppb；" +
                "非甲烷总烃日均浓度跨度范围为" + minNmhcStr + "~" + maxNmhcStr + " ppb，月均浓度为" + avgNmhcStr + " ppb。"));


        VocsData vocsData = new VocsData();
        if (CollectionUtils.isNotEmpty(vocsDataList)){
            vocsData = vocsDataList.get(0);
        }
        Float avgVocs = vocsData.getAvg();
        String maxVocStr = Objects.isNull(vocsData.getMax()) ? "--" : CalculationUtils.digit(vocsData.getMax(),2);
        String minVocStr = Objects.isNull(vocsData.getMin()) ? "--" : CalculationUtils.digit(vocsData.getMin(),2);
        String avgVocStr = Objects.isNull(avgVocs) ? "--" : CalculationUtils.digit(avgVocs,2);

        List<AvgData> monthDataList = vocsData.getMonthDataList();

        String zhanbi = "烷烃占比为--%、OVOCs占比--%、卤代烃占比--%、芳香烃占比--%、烯烃占比--%、炔烃占比--%";
        String a99000 = "";
        if (CollectionUtils.isNotEmpty(monthDataList)){
            List<String> list1 = new ArrayList<>();
            for (AvgData avgData : monthDataList) {
                String factorId = avgData.getFactorId();
                String factorName = avgData.getFactorName();
                Float valueVoc = avgData.getValue();
                String valueVocStr = "--%";
                if (!Objects.isNull(avgVocs) && 0f != avgVocs){
                    valueVocStr = CalculationUtils.digit(valueVoc * 100 / avgVocs,2) + "%";
                }
                if ("a99000".equals(factorId)){
                    a99000 = factorName + "占比为" + valueVocStr;
                }else {
                    list1.add(factorName + "占比为" + valueVocStr);
                }

            }
            list1.add(a99000);
            zhanbi = String.join("、", list1);
        }


        sb.append(textPart("2.厦门市VOCs跨度范围为："+minVocStr+"~"+maxVocStr+" μg/m³，月均浓度为"+avgVocStr+" μg/m³。VOCs检出浓度占比最高的是"+zhanbi+"。"));

        sb.append(textPart("3.臭氧激光雷达在X月X日前后，大气边界层高度较低，不利于臭氧的垂直扩散，近地面臭氧浓度较高。其余时段臭氧浓度相对较低。"));

        sb.append(headline2("（四）气象分析结论"));
        WeatherData weatherData = new WeatherData();
        if (CollectionUtils.isNotEmpty(weatherDataList)){
            weatherData = weatherDataList.get(0);
        }
        String wdStr = getString(weatherData.getWdStr());
        sb.append(textPart("大气超级站小时相对湿度分布在"+getString(weatherData.getSdMinStr())+"-"+getString(weatherData.getSdMaxStr())+"%之间，平均相对湿度为"+getString(weatherData.getSdAvgStr())+"%；" +
                "小时风速值分布在"+getString(weatherData.getWsMinStr())+"-"+getString(weatherData.getWsMaxStr())+" m/s之间，平均风速为"+getString(weatherData.getWsAvgStr())+"m/s，主导风向为"+wdStr+"；" +
                "温度分布在"+getString(weatherData.getWdMinStr())+"-"+getString(weatherData.getWdMaxStr())+"℃之间，平均温度为"+getString(weatherData.getWdAvgStr())+"℃；" +
                "气压值分布在"+getString(weatherData.getQyMinStr())+"-"+getString(weatherData.getQyMaxStr())+"kPa之间。"));
        return sb.toString();
    }

    public String getString(String text){
        return Objects.isNull(text)? "XX" : text;
    }


    public String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public String textPartTwo(String text1, String text2) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text1 + "</span>" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text2 + "</span>\n" +
                "</p>";
    }

    public String headline1(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.5;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }

}
