package more.ssaqms.xiamen.handler.month.supper;

import more.ssaqms.xiamen.handler.ReportDataUtil;
import more.ssaqms.xiamen.model.report.ReportOutput;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.supper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author zuo
 * @date 2023/5/4 12:21
 */
@Component
public class SupperMonthReport {

    @Value("${cloud.report.chart.url}")
    private String chartUrl;

    @Autowired
    private SupperHomePage supperHomePage;

    @Autowired
    private SupperChapter1 chapter1;

    @Autowired
    private SupperChapter2 chapter2;

    @Autowired
    private SupperChapter3 chapter3;

    @Autowired
    private SupperChapter4 chapter4;


    @Transactional
    public ReportOutput creatMonthHtmlReport(SupperInput input) {
        Date date = input.getDate();
        if (Objects.isNull(date)) {
            ReportOutput output = new ReportOutput();
            output.setHtml("");
            return output;
        }
        TimeRange monthTimeRange = ReportDataUtil.getMonthTimeRange(date);
        //常规污染物监测
        List<SiteAqiData> siteAqiDataList = input.getSiteAqiDataList();
        String home = supperHomePage.html(monthTimeRange);

        String chapter1 = this.chapter1.html(monthTimeRange, siteAqiDataList);

        List<OcEcData> ocEcDataList = input.getOcEcDataList();
        List<HeavyMetalData> heavyMetalDataList = input.getHeavyMetalDataList();
        List<BlackCarbonData> blackCarbonDataList = input.getBlackCarbonDataList();
        List<IonData> ionDataList = input.getIonDataList();
        List<SpectrometryData> spectrometryDataList = input.getSpectrometryDataList();
        List<LidarData> lidarDataList = input.getLidarDataList();

        String chapter2 = this.chapter2.html(ocEcDataList, heavyMetalDataList, blackCarbonDataList, ionDataList, lidarDataList, spectrometryDataList);

        List<HtcData> htcDataList = input.getHtcDataList();
        List<VocsData> vocsDataList = input.getVocsDataList();
        List<OzoneRadarData> ozoneRadarDataList = input.getOzoneRadarDataList();
        List<WeatherData> weatherDataList = input.getWeatherDataList();

        String chapter3 = this.chapter3.html(htcDataList, vocsDataList, ozoneRadarDataList, weatherDataList);

        String chapter4 = this.chapter4.html(siteAqiDataList, ocEcDataList, heavyMetalDataList, blackCarbonDataList, spectrometryDataList, htcDataList, vocsDataList, weatherDataList);
        ReportOutput reportOutput = new ReportOutput();
        reportOutput.setDate(input.getDate());
        reportOutput.setHtml(home + chapter1 + chapter2 + chapter3 + chapter4);
        return reportOutput;
    }


}
