package more.ssaqms.xiamen.handler.month.traffic;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import more.commons.utils.tool.DateUtils;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.traffic.GradeData;
import more.ssaqms.xiamen.model.traffic.PollutantData;
import more.ssaqms.xiamen.model.traffic.SiteAqiData;
import more.ssaqms.xiamen.util.CalculationUtils;
import more.ssaqms.xiamen.util.NumberConvert;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

import static more.ssaqms.xiamen.util.MonthReportUtils.table;
import static more.ssaqms.xiamen.util.MonthReportUtils.tr;

@Component
public class TrafficChapter1 {
    public static Map<String, String> factorNameMap = new HashMap<>();

    static {
        factorNameMap.put("O₃", "臭氧");
        factorNameMap.put("PM₂.₅", "细颗粒物");
        factorNameMap.put("PM₁₀", "可吸入颗粒物");
        factorNameMap.put("NO₂", "二氧化氮");
        factorNameMap.put("SO₂", "二氧化硫");
        factorNameMap.put("CO", "一氧化碳");
    }

    public static final Map<String, String> FACTOR_NAME_MAPPING = new HashMap() {{
        put("1001", "SO₂");
        put("1003", "NO₂");
        put("1006", "O₃");
        put("1007", "PM₁₀");
        put("1008", "PM₂.₅");
        put("1005", "CO");
    }};

    public String html(TimeRange monthTimeRange, List<SiteAqiData> siteAqiDatas) {
        StringBuilder sb = new StringBuilder();
        sb.append(headline("一、常规污染物监测"));
        if (CollectionUtils.isEmpty(siteAqiDatas)) {
            return sb.toString();
        }
        siteAqiDatas.forEach(siteAqiData -> {
            StringBuilder stringBuilder = new StringBuilder();
            //公路站只显示因子的均值浓度
            if (siteAqiData.getSiteId().equals("1721488148709826600")) {
                String strPollutant = "XXX";
                String datas = "XXX";
                List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
                if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                    List<String> names = new ArrayList<>();
                    List<String> factorValues = new ArrayList<>();
                    pollutantDatas.forEach(pollutantData -> {
                        String id = pollutantData.getFactorId();
                        if (FACTOR_NAME_MAPPING.containsKey(id)) {
                            names.add(FACTOR_NAME_MAPPING.get(id));
                        } else {
                            names.add(pollutantData.getFactorName());
                        }
                        factorValues.add(MessageFormat.format("{0}{1}", pollutantData.getValue(), pollutantData.getFactorName().equals("CO") ? "毫克/立方米" : "微克/立方米"));
                    });

                    strPollutant = String.join("、", names);

                    datas = String.join("、", factorValues);
                }
                stringBuilder.append(MessageFormat.format("{0}空气中主要污染物{1}平均浓度分别为{2}。", siteAqiData.getSiteName(), strPollutant, datas));
            } else {
                stringBuilder.append(MessageFormat.format("{0}空气质量综合指数为{1}。监测数据有效天数{2}天，", siteAqiData.getSiteName(), siteAqiData.getValue() == null ? "XXX" : siteAqiData.getValue(), siteAqiData.getDay()));
                List<GradeData> gradeDatas = siteAqiData.getGradeDatas();
                if (CollectionUtils.isNotEmpty(gradeDatas)) {
                    stringBuilder.append("其中AQI");
                    List<String> list = new ArrayList<>();
                    Optional<GradeData> gradeDataOptional = gradeDatas.stream().filter(a -> a.getGrade().equals("1")).findFirst();
                    if (gradeDataOptional.isPresent()) {
                        GradeData gradeData = gradeDataOptional.get();
                        list.add(MessageFormat.format("{0}的天数{1}天", gradeData.getGradeName(), gradeData.getDay()));
                    }

                    //良和轻度污染合并显示
                    List<GradeData> gradeDataList = gradeDatas.stream().filter(a -> a.getGrade().equals("2") || a.getGrade().equals("3")).collect(Collectors.toList());
                    List<String> names = new ArrayList<>();
                    StringBuilder strBuilder = new StringBuilder();
                    Map<String, Integer> factorDayMap = new HashMap<>();
                    if (CollectionUtils.isNotEmpty(gradeDataList)) {
                        gradeDataList.forEach(a -> {
                            names.add(MessageFormat.format("{0}的天数{1}天", a.getGradeName(), a.getDay()));
                            List<PollutantData> pollutantDatas = a.getPollutantDatas();
                            if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                                pollutantDatas.forEach(p -> {
                                    if (!factorDayMap.containsKey(p.getFactorName())) {
                                        factorDayMap.put(p.getFactorName(), 0);
                                    }
                                    Integer value = factorDayMap.get(p.getFactorName());
                                    value += p.getDay();
                                    factorDayMap.put(p.getFactorName(), value);
                                });
                            }
                        });
                        strBuilder.append(String.join(",", names));
                    }
                    if (CollectionUtils.isNotEmpty(factorDayMap)) {
                        List<String> factorDays = new ArrayList<>();
                        factorDayMap.forEach((factor, day) -> {
                            factorDays.add(MessageFormat.format("{0}{1}天", factorNameMap.containsKey(factor) ? factorNameMap.get(factor) : factor, day));
                        });
                        strBuilder.append(MessageFormat.format("（首要污染物为{0}）", String.join("、", factorDays)));
                    }
                    list.add(strBuilder.toString());

                    for (int i = 4; i <= 6; i++) {
                        int finalI = i;
                        gradeDataOptional = gradeDatas.stream().filter(a -> a.getGrade().equals(String.valueOf(finalI))).findFirst();
                        if (gradeDataOptional.isPresent()) {
                            StringBuilder stringBuilder3 = new StringBuilder();
                            GradeData gradeData = gradeDataOptional.get();
                            stringBuilder3.append(MessageFormat.format("{0}的天数{1}天", gradeData.getGradeName(), gradeData.getDay()));
                            List<PollutantData> pollutantDatas = gradeData.getPollutantDatas();
                            if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                                StringBuilder stringBuilder2 = new StringBuilder();
                                List<String> msg = new ArrayList<>();
                                pollutantDatas.forEach(pollutantData -> {
                                    msg.add(MessageFormat.format("{0}{1}天", pollutantData.getFactorName(), pollutantData.getDay()));
                                });
                                if (CollectionUtils.isNotEmpty(msg)) {
                                    stringBuilder2.append(String.join("、", msg));
                                }
                                stringBuilder3.append(MessageFormat.format("（首要污染物为{0}）", stringBuilder2.toString()));
                            }
                            list.add(stringBuilder3.toString());
                        }
                    }
                    if (CollectionUtils.isNotEmpty(list)) {
                        stringBuilder.append(String.join("，", list));
                        stringBuilder.append("；");
                    }
                }
                stringBuilder.append(MessageFormat.format("空气质量优良率{0}%，优级率{1}%。", siteAqiData.getExcellentRate(), siteAqiData.getSuperiorRate() != null ? siteAqiData.getSuperiorRate() : "XXX"));
                List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
                if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                    stringBuilder.append("空气中主要污染物");
                    List<String> factorValues = new ArrayList<>();
                    List<String> names = new ArrayList<>();
                    pollutantDatas.forEach(pollutantData -> {
                        String id = pollutantData.getFactorId();
                        if (FACTOR_NAME_MAPPING.containsKey(id)) {
                            names.add(FACTOR_NAME_MAPPING.get(id));
                        } else {
                            names.add(pollutantData.getFactorName());
                        }
                        factorValues.add(MessageFormat.format("{0}{1}", pollutantData.getValue(), pollutantData.getFactorName().equals("CO") ? "毫克/立方米" : "微克/立方米"));
                    });
                    stringBuilder.append(String.join("、", names));
                    stringBuilder.append("平均浓度分别为");
                    stringBuilder.append(String.join("、", factorValues));
                    stringBuilder.append("。");
                }
            }
            sb.append(textPart(stringBuilder.toString()));
        });

        String aqiTable = createAqiTable(monthTimeRange, siteAqiDatas);
        if (!aqiTable.equals("")) {
            sb.append(aqiTable);
        }

        String aqi = createtAqiChain(siteAqiDatas);
        if (!aqi.equals("")) {
            sb.append(aqi);
        }

        String stateAqi = createtStateAqi(siteAqiDatas);
        if (!stateAqi.equals("")) {
            sb.append(stateAqi);
        }
        return sb.toString();
    }

    private String createAqiTable(TimeRange monthTimeRange, List<SiteAqiData> siteAqiDatas) {
        if (CollectionUtils.isEmpty(siteAqiDatas)) {
            return "";
        }

        StringBuilder trStr = new StringBuilder();
        trStr.append(textCenter(MessageFormat.format("表1-1 {0}各监测点位空气质量数据表", DateUtils.formatDate(monthTimeRange.getBeginTime(), "yyyy年MM月"))));
        StringBuilder tdBuffer = new StringBuilder();
        tdBuffer.append(overstrikingTd(1, 2, "站点名称"));
        tdBuffer.append(overstrikingTd(6, 1, "主要污染物名称及浓度（浓度单位：CO为mg/m³，其余为µg/m³）"));
        tdBuffer.append(overstrikingTd(1, 2, "空气质量优良率（%）"));
        tdBuffer.append(overstrikingTd(1, 2, "综合指数"));
        trStr.append(tr(tdBuffer.toString()));
        tdBuffer = new StringBuilder();
        tdBuffer.append(overstrikingTd(1, 1, "SO₂"));
        tdBuffer.append(overstrikingTd(1, 1, "NO₂"));
        tdBuffer.append(overstrikingTd(1, 1, "CO"));
        tdBuffer.append(overstrikingTd(1, 1, "O₃"));
        tdBuffer.append(overstrikingTd(1, 1, "PM₁₀"));
        tdBuffer.append(overstrikingTd(1, 1, "PM₂.₅"));
        trStr.append(tr(tdBuffer.toString()));

        for (int i = 0; i < siteAqiDatas.size(); i++) {
            SiteAqiData siteAqiData = siteAqiDatas.get(i);
            List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
            Map<String, PollutantData> pollutantDataMapMap = new HashMap<>();

            for (PollutantData pollutantData : pollutantDatas) {
                pollutantDataMapMap.put(pollutantData.getFactorId(), pollutantData);
            }

            tdBuffer = new StringBuilder();
            tdBuffer.append(overstrikingTd(1, 1, siteAqiData.getSiteName()));
            tdBuffer.append(overstrikingTd(1, 1, pollutantDataMapMap.containsKey("1001") ? CalculationUtils.digit(pollutantDataMapMap.get("1001").getValue(), 0) : "-"));
            tdBuffer.append(overstrikingTd(1, 1, pollutantDataMapMap.containsKey("1003") ? CalculationUtils.digit(pollutantDataMapMap.get("1003").getValue(), 0) : "-"));
            tdBuffer.append(overstrikingTd(1, 1, pollutantDataMapMap.containsKey("1005") ? CalculationUtils.digit(pollutantDataMapMap.get("1005").getValue(), 1) : "-"));
            tdBuffer.append(overstrikingTd(1, 1, pollutantDataMapMap.containsKey("1006") ? CalculationUtils.digit(pollutantDataMapMap.get("1006").getValue(), 0) : "-"));
            tdBuffer.append(overstrikingTd(1, 1, pollutantDataMapMap.containsKey("1007") ? CalculationUtils.digit(pollutantDataMapMap.get("1007").getValue(), 0) : "-"));
            tdBuffer.append(overstrikingTd(1, 1, pollutantDataMapMap.containsKey("1008") ? CalculationUtils.digit(pollutantDataMapMap.get("1008").getValue(), 0) : "-"));
            tdBuffer.append(overstrikingTd(1, 1, null == siteAqiData.getExcellentRate() ? "-" : siteAqiData.getExcellentRate().toString()));
            tdBuffer.append(overstrikingTd(1, 1, null == siteAqiData.getValue() ? "-" : siteAqiData.getValue().toString()));
            trStr.append(tr(tdBuffer.toString()));
        }
        return table(trStr.toString());
    }

    private String createtAqiChain(List<SiteAqiData> siteAqiDatas) {
        if (CollectionUtils.isEmpty(siteAqiDatas)) {
            return "";
        }
        //公路站不纳入统计
        List<SiteAqiData> siteAqiData = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && null != a.getLastValuePercent() && a.getLastValuePercent() > 0).collect(Collectors.toList());
        List<SiteAqiData> siteAqiData1 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && null != a.getLastValuePercent() && a.getLastValuePercent() < 0).collect(Collectors.toList());
        List<SiteAqiData> siteAqiData2 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && null != a.getLastValuePercent() && a.getLastValuePercent() == 0).collect(Collectors.toList());

        StringBuilder str = new StringBuilder();
        str.append("与上月相比，除公路站外的各站点");

        List<String> total = new ArrayList<>();
        StringBuilder stringBuilder = new StringBuilder();

        List<String> title = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(siteAqiData)) {
            title.add(MessageFormat.format("{0}升", NumberConvert.arabicNumToChineseNum(siteAqiData.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData1)) {
            title.add(MessageFormat.format("{0}降", NumberConvert.arabicNumToChineseNum(siteAqiData1.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData2)) {
            title.add(MessageFormat.format("{0}平", NumberConvert.arabicNumToChineseNum(siteAqiData2.size())));
        }
        if (CollectionUtils.isNotEmpty(title)) {
            stringBuilder.append(MessageFormat.format("空气质量综合指数值 \"{0}\"", String.join("", title)));
        }

        List<String> msg = new ArrayList<>();
        siteAqiDatas.forEach(a -> {
            if (a.getSiteId().equals("1721488148709826600")) {
                return;
            }
            Float percent = a.getLastValuePercent();
            if (null == percent) {
                return;
            }
            msg.add(MessageFormat.format("{0}{1}{2}", a.getSiteName(), (percent > 0 ? "上升" : (percent < 0 ? "下降" : "持平")), percent == 0 ? "" : MessageFormat.format("{0}%", Math.abs(percent.floatValue()))));
        });

        if (CollectionUtils.isNotEmpty(msg)) {
            stringBuilder.append(MessageFormat.format("（{0}）", String.join("、", msg)));
        }
        if (null != stringBuilder.toString() && !stringBuilder.toString().equals("")) {
            total.add(stringBuilder.toString());
        }

        stringBuilder = new StringBuilder();
        siteAqiData = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && a.getLastAqiValuePercent() != null && a.getLastAqiValuePercent() > 0).collect(Collectors.toList());
        siteAqiData1 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && a.getLastAqiValuePercent() != null && a.getLastAqiValuePercent() < 0).collect(Collectors.toList());
        siteAqiData2 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && a.getLastAqiValuePercent() != null && a.getLastAqiValuePercent() == 0).collect(Collectors.toList());

        title.clear();
        if (CollectionUtils.isNotEmpty(siteAqiData)) {
            title.add(MessageFormat.format("{0}升", NumberConvert.arabicNumToChineseNum(siteAqiData.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData1)) {
            title.add(MessageFormat.format("{0}降", NumberConvert.arabicNumToChineseNum(siteAqiData.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData2)) {
            title.add(MessageFormat.format("{0}平", NumberConvert.arabicNumToChineseNum(siteAqiData2.size())));
        }
        if (CollectionUtils.isNotEmpty(title)) {
            stringBuilder.append(MessageFormat.format("空气质量优良率 \"{0}\"", String.join("", title)));
        }

        msg.clear();
        siteAqiDatas.forEach(a -> {
            if (a.getSiteId().equals("1721488148709826600")) {
                return;
            }
            Float percent = a.getLastAqiValuePercent();
            if (null == percent) {
                return;
            }
            msg.add(MessageFormat.format("{0}{1}{2}", a.getSiteName(), (percent > 0 ? "上升" : (percent < 0 ? "下降" : "持平")), percent == 0 ? "" : MessageFormat.format("{0}%", Math.abs(percent.floatValue()))));
        });

        if (CollectionUtils.isNotEmpty(msg)) {
            stringBuilder.append(MessageFormat.format("（{0}）", String.join("、", msg)));
        }
        if (null != stringBuilder.toString() && !stringBuilder.toString().equals("")) {
            total.add(stringBuilder.toString());
        }

        if (CollectionUtils.isEmpty(total)) {
            return "";
        }
        str.append(String.join("；", total));
        str.append("。");
        return textChainPart(str.toString());
    }

    private String createtStateAqi(List<SiteAqiData> siteAqiDatas) {
        if (CollectionUtils.isEmpty(siteAqiDatas)) {
            return "";
        }
        List<SiteAqiData> siteAqiData = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && null != a.getLastStateValuePercent() && a.getLastStateValuePercent() > 0).collect(Collectors.toList());
        List<SiteAqiData> siteAqiData1 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && null != a.getLastStateValuePercent() && a.getLastStateValuePercent() < 0).collect(Collectors.toList());
        List<SiteAqiData> siteAqiData2 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && null != a.getLastStateValuePercent() && a.getLastStateValuePercent() == 0).collect(Collectors.toList());
        Float value = siteAqiDatas.get(0).getLastStateValue();
        StringBuilder str = new StringBuilder();
        str.append(MessageFormat.format("与同期全市国控数据（{0}）相比，除公路站外的各站点", value == null ? "-" : value));

        List<String> total = new ArrayList<>();
        StringBuilder stringBuilder = new StringBuilder();

        List<String> title = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(siteAqiData)) {
            title.add(MessageFormat.format("{0}高", NumberConvert.arabicNumToChineseNum(siteAqiData.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData1)) {
            title.add(MessageFormat.format("{0}低", NumberConvert.arabicNumToChineseNum(siteAqiData1.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData2)) {
            title.add(MessageFormat.format("{0}平", NumberConvert.arabicNumToChineseNum(siteAqiData2.size())));
        }
        if (CollectionUtils.isNotEmpty(title)) {
            stringBuilder.append(MessageFormat.format("空气质量综合指数值 \"{0}\"", String.join("", title)));
        }

        List<String> msg = new ArrayList<>();
        siteAqiDatas.forEach(a -> {
            if (a.getSiteId().equals("1721488148709826600")) {
                return;
            }
            Float percent = a.getLastStateValuePercent();
            if (null == percent) {
                return;
            }
            msg.add(MessageFormat.format("{0}{1}{2}", a.getSiteName(), (percent > 0 ? "高出" : (percent < 0 ? "低出" : "持平")), percent == 0 ? "" : MessageFormat.format("{0}%", Math.abs(percent.floatValue()))));
        });

        if (CollectionUtils.isNotEmpty(msg)) {
            stringBuilder.append(MessageFormat.format("（{0}）", String.join("、", msg)));
        }
        if (null != stringBuilder.toString() && !stringBuilder.toString().equals("")) {
            total.add(stringBuilder.toString());
        }

        stringBuilder = new StringBuilder();
        siteAqiData = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && a.getLastStateAqiValuePercent() != null && a.getLastStateAqiValuePercent() > 0).collect(Collectors.toList());
        siteAqiData1 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && a.getLastStateAqiValuePercent() != null && a.getLastStateAqiValuePercent() < 0).collect(Collectors.toList());
        siteAqiData2 = siteAqiDatas.stream().filter(a -> !a.getSiteId().equals("1721488148709826600") && a.getLastStateAqiValuePercent() != null && a.getLastStateAqiValuePercent() == 0).collect(Collectors.toList());

        title.clear();
        if (CollectionUtils.isNotEmpty(siteAqiData)) {
            title.add(MessageFormat.format("{0}高", NumberConvert.arabicNumToChineseNum(siteAqiData.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData1)) {
            title.add(MessageFormat.format("{0}低", NumberConvert.arabicNumToChineseNum(siteAqiData1.size())));
        }
        if (CollectionUtils.isNotEmpty(siteAqiData2)) {
            title.add(MessageFormat.format("{0}平", NumberConvert.arabicNumToChineseNum(siteAqiData2.size())));
        }
        if (CollectionUtils.isNotEmpty(title)) {
            stringBuilder.append(MessageFormat.format("空气质量优良率\"{0}\"", String.join("", title)));
        }

        msg.clear();
        siteAqiDatas.forEach(a -> {
            if (a.getSiteId().equals("1721488148709826600")) {
                return;
            }
            Float percent = a.getLastStateAqiValuePercent();
            if (null == percent) {
                return;
            }
            msg.add(MessageFormat.format("{0}{1}{2}", a.getSiteName(), (percent > 0 ? "高出" : (percent < 0 ? "低出" : "持平")), percent == 0 ? "" : MessageFormat.format("{0}%", Math.abs(percent.floatValue()))));
        });

        if (CollectionUtils.isNotEmpty(msg)) {
            stringBuilder.append(MessageFormat.format("（{0}）", String.join("、", msg)));
        }
        if (null != stringBuilder.toString() && !stringBuilder.toString().equals("")) {
            total.add(stringBuilder.toString());
        }

        if (CollectionUtils.isEmpty(total)) {
            return "";
        }
        str.append(String.join("；", total));
        str.append("。");
        return textChainPart(str.toString());
    }

    private String overstrikingTd(Integer colspan, Integer rowspan, String text) {
        return "<td colspan=\"" + colspan + "\" rowspan=\"" + rowspan + "\" style=\"text-align:center\">" +
                "<p style=\"text-align:center;\">" +
                "    <span style=\"font-size: 10.5pt; font-family: 仿宋, SimSun;\">" + text + "</span>" +
                "</p>" +
                "</td>";
    }

    private String textCenter(String text) {
        return "<p style=\"margin: 0px; text-align: center; line-height: 1.1;\">" +
                "<span style=\"font-size: 12pt; font-weight:bold; font-family: 宋体, SimSun;\">" + text + "</span>" +
                "</p>";
    }

    private String textPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    private String textChainPart(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    private String headline(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }
}
