package more.ssaqms.xiamen.handler.month.traffic;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.ssaqms.xiamen.model.report.BasicInformation;
import more.ssaqms.xiamen.model.site.Site;
import more.ssaqms.xiamen.model.traffic.*;
import more.ssaqms.xiamen.util.CalculationUtils;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.NumberConvert;
import more.ssaqms.xiamen.util.RestTemplateUtils;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

import static more.ssaqms.xiamen.util.MonthReportUtils.image;
import static more.ssaqms.xiamen.util.MonthReportUtils.textCenter;

@Component
public class TrafficChapter2 {
    public String html(List<CH4AvgData> ch4AvgDataList, MageeAvgData mageeAvgData, VOCsData voCsData,BasicInformation monthBasicInformation){
        StringBuilder sb = new StringBuilder();
        sb.append(headline("二、其他污染物监测"));
        String mageeHtml = createMageeHtml(mageeAvgData,monthBasicInformation);
        if(null!=mageeHtml&&!mageeHtml.equals("")){
            sb.append(mageeHtml);
        }
        String ch4Html = createCH4Html(ch4AvgDataList,monthBasicInformation);
        if(null!=ch4Html&&!ch4Html.equals("")){
            sb.append(ch4Html);
        }

        String vocsHtml = createVocsHtml(voCsData,monthBasicInformation);
        if(null!=vocsHtml&&!vocsHtml.equals("")){
            sb.append(vocsHtml);
        }
        return sb.toString();
    }

    private String createVocsHtml(VOCsData voCsData,BasicInformation monthBasicInformation){
        StringBuilder sb = new StringBuilder();
        sb.append(headline2("（三）挥发性有机物"));
        if(null == voCsData){
            sb.append(partText("以全市5个监测点位（超级站、铁路、公路、港口、机场）VOCs有效数据进行评价，厦门市VOCs日平均值跨度范围为：XXX~XXX ppb，月均浓度为XXX ppb。VOCs检出浓度占比最高的是烷烃占比为XXX%、烯烃占比XXX%、芳香烃占比XXX%、炔烃占比XXX%。"));
            return sb.toString();
        }
        List<String> siteNames = monthBasicInformation.getSiteMap().values().stream().map(Site::getName).collect(Collectors.toList());
        StringBuilder str = new StringBuilder();
        if(CollectionUtils.isNotEmpty(siteNames)){
            String name = String.join("、",siteNames);
            str.append(MessageFormat.format("以全市{0}个监测点位（{1}）VOCs（56种，PAMs）有效数据进行评价，",siteNames.size(),name));
        }
        if(null != voCsData.getDayAvgMax() && null != voCsData.getDayAvgMin()){
            str.append(MessageFormat.format("厦门市VOCs日平均值跨度范围为：{0}~{1}ppb，月均浓度为{2}ppb。",voCsData.getDayAvgMin(),voCsData.getDayAvgMax(),voCsData.getMonthAvg()));
        }

        List<VOCsMonthDataDetail> voCsMonthDataDetails = voCsData.getVoCsMonthDataDetails();
        if(CollectionUtils.isNotEmpty(voCsMonthDataDetails)){
            str.append("VOCs检出浓度占比最高的是");
            List<String>msgs=new ArrayList<>();
            voCsMonthDataDetails.forEach(voCsMonthDataDetail -> {
                if(null != voCsMonthDataDetail.getPercent()){
                    msgs.add(MessageFormat.format("{0}占比为{1}%",voCsMonthDataDetail.getFactorName(),voCsMonthDataDetail.getPercent()));
                }
            });

            if(CollectionUtils.isNotEmpty(msgs)){
                str.append(String.join("、",msgs));
                str.append("。");
            }
        }
        sb.append(partText(str.toString()));
        List<Map<String,String>> chartDatas = voCsData.getChartDatas();
        if(CollectionUtils.isNotEmpty(chartDatas)){
            chartDatas.forEach(charDataMap->{
                if(charDataMap.containsKey("siteName")){
                    sb.append(image(charDataMap.get("chartUrl"), null, null));
                    sb.append(textCenter(charDataMap.get("siteName")));
                }
            });
        }

        str = new StringBuilder();
        str.append("按照");
        List<SiteVOCsData> siteVOCsDataList = voCsData.getSiteVOCsDataList();
        List<String> totalMsg = new ArrayList<>();
        List<String> chainMsg = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(siteVOCsDataList)){
            StringBuilder stringBuilder = new StringBuilder();
            List<String>msgs = new ArrayList<>();
            siteVOCsDataList.sort((o1, o2) -> {
                if (o1.getValue() == null && o2.getValue() == null) {
                    return 0;
                } else if (o1.getValue() == null) {
                    return 1;
                } else if (o2.getValue() == null) {
                    return -1;
                } else {
                    return o2.getValue().compareTo(o1.getValue());
                }
            });
            siteVOCsDataList.forEach(a->{
                if(null == a.getValue()){
                    return;
                }
                msgs.add(MessageFormat.format("{0}（{1}ppb）",a.getSiteName(),a.getValue()));
            });
            if(CollectionUtils.isNotEmpty(msgs)){
                stringBuilder.append("VOCs月均浓度由高到低排名：");
                stringBuilder.append(String.join(">",msgs));
                totalMsg.add(stringBuilder.toString());
            }

            String msg = buildChainMsg(siteVOCsDataList,"VOCs");
            if(!msg.equals("") && null !=msg){
                chainMsg.add(msg);
            }
        }

        List<SiteVOCsData> siteOFPDataList = voCsData.getSiteOFPDataList();
        if(CollectionUtils.isNotEmpty(siteOFPDataList)){
            StringBuilder stringBuilder = new StringBuilder();
            List<String>msgs = new ArrayList<>();
            siteOFPDataList.sort((o1, o2) -> {
                if (o1.getValue() == null && o2.getValue() == null) {
                    return 0;
                } else if (o1.getValue() == null) {
                    return 1;
                } else if (o2.getValue() == null) {
                    return -1;
                } else {
                    return o2.getValue().compareTo(o1.getValue());
                }
            });
            siteOFPDataList.forEach(a->{
                if(null == a.getValue()){
                    return;
                }
                msgs.add(MessageFormat.format("{0}（{1}μg/m³）",a.getSiteName(),a.getValue()));
            });
            if(CollectionUtils.isNotEmpty(msgs)){
                stringBuilder.append("OFP月均浓度由高到低排名：");
                stringBuilder.append(String.join(">",msgs));
                totalMsg.add(stringBuilder.toString());
            }

            String msg = buildChainMsg(siteOFPDataList,"OFP");
            if(!msg.equals("") && null !=msg){
                chainMsg.add(msg);
            }
        }

        List<SiteVOCsData> siteSoaDataList = voCsData.getSiteSoaDataList();
        if(CollectionUtils.isNotEmpty(siteSoaDataList)){
            StringBuilder stringBuilder = new StringBuilder();
            List<String>msgs = new ArrayList<>();
            siteSoaDataList.sort((o1, o2) -> {
                if (o1.getValue() == null && o2.getValue() == null) {
                    return 0;
                } else if (o1.getValue() == null) {
                    return 1;
                } else if (o2.getValue() == null) {
                    return -1;
                } else {
                    return o2.getValue().compareTo(o1.getValue());
                }
            });
            siteSoaDataList.forEach(a->{
                if(null == a.getValue()){
                    return;
                }
                msgs.add(MessageFormat.format("{0}（{1}μg/m³）",a.getSiteName(),a.getValue()));
            });
            if(CollectionUtils.isNotEmpty(msgs)){
                stringBuilder.append("SOAp月均浓度由高到低排名：");
                stringBuilder.append(String.join(">",msgs));
                totalMsg.add(stringBuilder.toString());
            }

            String msg = buildChainMsg(siteSoaDataList,"SOAp");
            if(!msg.equals("") && null !=msg){
                chainMsg.add(msg);
            }
        }
        if(CollectionUtils.isNotEmpty(totalMsg)){
            str.append(String.join(";",totalMsg));
        }
        str.append("。");
        sb.append(partText(str.toString()));


        //每个站点的soa图
        if(CollectionUtils.isNotEmpty(siteSoaDataList)){
            siteSoaDataList.forEach(a->{
                String chartUrl = a.getChartUrl();
                if(StringUtils.isBlank(chartUrl)){
                    return;
                }
                sb.append(image(chartUrl, null, null));
                sb.append(textCenter(a.getSiteName()));
            });
        }
        sb.append(textCenter("图2-5 各站点SOAp浓度前十"));

        //每个站点的ofp图
        if(CollectionUtils.isNotEmpty(siteOFPDataList)){
            siteOFPDataList.forEach(a->{
                String chartUrl = a.getChartUrl();
                sb.append(image(chartUrl, null, null));
                sb.append(textCenter(a.getSiteName()));
            });
        }
        sb.append(textCenter("图2-5 各站点OFP浓度前十"));

        str = new StringBuilder();
        str.append("与上月同期相比，");
        if(null != voCsData.getPercent()){
            str.append(MessageFormat.format("厦门市VOCs浓度{0}{1}。",(voCsData.getPercent()>0?"上升":(voCsData.getPercent()<0?"下降":"持平")),(voCsData.getPercent()!=0?Math.abs(voCsData.getPercent())+"%":"")));
        }
        else{
            str.append("与上月同期相比，厦门市VOCs浓度上升 -- %。");
        }

        if(CollectionUtils.isNotEmpty(chainMsg)){
            str.append(String.join("。",chainMsg));
            str.append("。");
        }
        sb.append(partText(str.toString()));
        return sb.toString();
    }

    private String buildChainMsg(List<SiteVOCsData> siteDataList, String title){
        if(CollectionUtils.isEmpty(siteDataList)){
            return "";
        }
        StringBuilder str = new StringBuilder();
        str.append(MessageFormat.format("各站点{0}浓度",title));

        List<SiteVOCsData> siteVOCsDataList1 = siteDataList.stream().filter(a->null!=a.getPercent()&&a.getPercent()>0).collect(Collectors.toList());
        List<SiteVOCsData> siteVOCsDataList2 = siteDataList.stream().filter(a->null!=a.getPercent()&&a.getPercent()<0).collect(Collectors.toList());
        List<SiteVOCsData> siteVOCsDataList3 = siteDataList.stream().filter(a->null!=a.getPercent()&&a.getPercent()==0).collect(Collectors.toList());

        List<String> datas = new ArrayList<>();
        StringBuilder sb1;
        if(siteVOCsDataList1.size()>0){
            sb1=new StringBuilder();
            sb1.append(MessageFormat.format("\"{0}升\"",NumberConvert.arabicNumToChineseNum(siteVOCsDataList1.size())));
            List<String> strMsgs = new ArrayList<>();
            for(SiteVOCsData siteVOCsData : siteVOCsDataList1){
                String s = MessageFormat.format("{0}{1}{2}",siteVOCsData.getSiteName(),siteVOCsData.getPercent()>0?"上升":(siteVOCsData.getPercent()<0?"下降":"持平"),siteVOCsData.getPercent()==0?"":Math.abs(siteVOCsData.getPercent())+"%");
                strMsgs.add(s);
            }
            sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
            datas.add(sb1.toString());
        }
        if(siteVOCsDataList2.size()>0){
            sb1=new StringBuilder();
            sb1.append(MessageFormat.format("\"{0}降\"",NumberConvert.arabicNumToChineseNum(siteVOCsDataList2.size())));
            List<String> strMsgs = new ArrayList<>();
            for(SiteVOCsData siteVOCsData : siteVOCsDataList2){
                String s = MessageFormat.format("{0}{1}{2}",siteVOCsData.getSiteName(),siteVOCsData.getPercent()>0?"上升":(siteVOCsData.getPercent()<0?"下降":"持平"),siteVOCsData.getPercent()==0?"":Math.abs(siteVOCsData.getPercent())+"%");
                strMsgs.add(s);
            }
            sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
            datas.add(sb1.toString());
        }
        if(siteVOCsDataList3.size()>0){
            sb1=new StringBuilder();
            sb1.append(MessageFormat.format("\"{0}平\"",NumberConvert.arabicNumToChineseNum(siteVOCsDataList3.size())));
            List<String> strMsgs = new ArrayList<>();
            for(SiteVOCsData siteVOCsData : siteVOCsDataList3){
                String s = MessageFormat.format("{0}{1}{2}",siteVOCsData.getSiteName(),siteVOCsData.getPercent()>0?"上升":(siteVOCsData.getPercent()<0?"下降":"持平"),siteVOCsData.getPercent()==0?"":Math.abs(siteVOCsData.getPercent())+"%");
                strMsgs.add(s);
            }
            sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
            datas.add(sb1.toString());
        }
        if(CollectionUtils.isNotEmpty(datas)){
            str.append(String.join("、",datas));
        }
        return str.toString();
    }

    private String createMageeHtml(MageeAvgData mageeAvgData,BasicInformation monthBasicInformation){
        StringBuilder sb = new StringBuilder();
        sb.append(headline2("（一）黒碳"));
        if(null == mageeAvgData){
            sb.append(partText("以全市4个监测点位（机场、铁路、公路、港口）有效数据进行评价，厦门市黑碳浓度日均浓度跨度范围为XXX~XXX ng/m³，月均浓度为XXX ng/m³。BC主要来自移动源，月均浓度为XXX ng/m³，占总BC的XXX%；生物质燃烧产生的BC的月均浓度为XXX ng/m³，占总BC的XXX%。"));
            return sb.toString();
        }
        List<String> siteNames = monthBasicInformation.getSiteMap().values().stream().filter(a->!a.getId().equals("1721488148709826562")).map(a->a.getName()).collect(Collectors.toList());
        StringBuilder str = new StringBuilder();
        if(CollectionUtils.isNotEmpty(siteNames)){
            String name = String.join("、",siteNames);
            str.append(MessageFormat.format("以全市{0}个监测点位（{1}）有效数据进行评价，",siteNames.size(),name));
        }
        if(null !=mageeAvgData.getDayAvgMax()&&null !=mageeAvgData.getDayAvgMin()){
            str.append(MessageFormat.format("厦门市黑碳浓度日均浓度跨度范围为{0}~{1}ng/m³，月均浓度为{2}ng/m³。",mageeAvgData.getDayAvgMin(),mageeAvgData.getDayAvgMax(),mageeAvgData.getMonthAvg()));
        }

        if(mageeAvgData.getTotalMonthAvgSpamsData()!=null && mageeAvgData.getTotalMonthAvgSpamsData().size()>0){
            List<SpamsData> totalMonthAvgSpamsData = mageeAvgData.getTotalMonthAvgSpamsData();
            Collections.sort(totalMonthAvgSpamsData, new Comparator<SpamsData>() {
                @Override
                public int compare(SpamsData p1, SpamsData p2) {
                    return Float.compare(p2.getPercent(), p1.getPercent());
                }
            });
            for(int i=0;i<totalMonthAvgSpamsData.size();i++){
                SpamsData spamsData = totalMonthAvgSpamsData.get(i);
                if(i==0){
                    str.append(MessageFormat.format("BC主要来自{0}，月均浓度为{1}ng/m³，占总BC的{2}%；", spamsData.getFactorName(), spamsData.getValue(), spamsData.getPercent()));
                }
                else{
                    str.append(MessageFormat.format("{0}产生的BC的月均浓度为{1}ng/m³，占总BC的{2}%。", spamsData.getFactorName(), spamsData.getValue(), spamsData.getPercent()));
                }
            }
        }
        else {
            str.append("BC主要来自移动源，月均浓度为--ng/m³，占总BC的--%；生物质燃烧产生的BC的月均浓度为-- ng/m³，占总BC的--%。");
        }
        sb.append(partText(str.toString()));

        str = new StringBuilder();
        List<String> strMsg = new ArrayList<>();
        List<SiteMageeAvgData> siteMageeAvgDataList = mageeAvgData.getSiteMageeAvgDataList();
        if(CollectionUtils.isNotEmpty(siteMageeAvgDataList)){
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("按照黑碳月均值浓度由高到低排名：");
            Collections.sort(siteMageeAvgDataList, new Comparator<SiteMageeAvgData>() {
                @Override
                public int compare(SiteMageeAvgData o1, SiteMageeAvgData o2) {
                    if (o1.getValue() == null && o2.getValue() == null) {
                        return 0;
                    } else if (o1.getValue() == null) {
                        return 1;
                    } else if (o2.getValue() == null) {
                        return -1;
                    } else {
                        return o2.getValue().compareTo(o1.getValue());
                    }
                }
            });
            List<String> strMsg1 = new ArrayList<>();
            siteMageeAvgDataList.forEach(a->{
                if(null!=a.getValue()){
                    strMsg1.add(MessageFormat.format("{0}（{1}ng/m³）",a.getSiteName(),a.getValue()));
                }
            });
            if(CollectionUtils.isNotEmpty(strMsg1)){
                stringBuilder.append(String.join(">",strMsg1));
            }
            strMsg.add(stringBuilder.toString());
        }

        //按移动源和生物质燃烧 排序每个站点的浓度
        if(CollectionUtils.isNotEmpty(mageeAvgData.getMonthAvgSpamsData())){
            List<SpamsData> monthAvgSpamsData = mageeAvgData.getMonthAvgSpamsData();
            Map<String,List<SpamsData>> avgMap = monthAvgSpamsData.stream().collect(Collectors.groupingBy(SpamsData::getFactorId));
            if(CollectionUtils.isNotEmpty(avgMap)){
                avgMap.forEach((factor,avgDataList)->{
                    if(CollectionUtils.isNotEmpty(avgDataList)){
                        String name = avgDataList.get(0).getFactorName();
                        StringBuilder stringBuilder = new StringBuilder();
                        stringBuilder.append(MessageFormat.format("按照{0}排放黒碳由高到低排名：",name));
                        Collections.sort(avgDataList, new Comparator<SpamsData>() {
                            @Override
                            public int compare(SpamsData p1, SpamsData p2) {
                                return Float.compare(p2.getValue(), p1.getValue());
                            }
                        });
                        List<String> strSort = new ArrayList<>();
                        avgDataList.forEach(avgData->{
                            strSort.add(MessageFormat.format("{0}({1})ng/m³",avgData.getSiteName(), CalculationUtils.digit2(avgData.getValue(),1)));
                        });
                        if(CollectionUtils.isNotEmpty(strSort)){
                            stringBuilder.append(String.join(">",strSort));
                        }
                        strMsg.add(stringBuilder.toString());
                    }
                });
            }
        }

        if(CollectionUtils.isNotEmpty(strMsg)){
            str.append(String.join("；",strMsg));
            str.append("。");
        }
        sb.append(partText(str.toString()));

        //各站点黑炭日均浓度变化特征
        String chartUrl = mageeAvgData.getChartUrl();
        if(StringUtils.isNotBlank(chartUrl)){
            sb.append(image(chartUrl, null, null));
            sb.append(textCenter("图2-1 各站点黑碳日均浓度变化特征"));
        }

        //环比
        str = new StringBuilder();
        str.append("与上月同期相比，");
        if(null!=mageeAvgData.getPercent()){
            str.append(MessageFormat.format("厦门市黑碳浓度{0}{1}。",(mageeAvgData.getPercent()>0?"上升":(mageeAvgData.getPercent()<0?"下降":"持平")),(mageeAvgData.getPercent()!=0?Math.abs(mageeAvgData.getPercent().floatValue())+"%":"")));
        }
        else{
            str.append("厦门市黑碳浓度 --%。");

        }
        List<SiteMageeAvgData> siteMageeAvgDatas = mageeAvgData.getSiteMageeAvgDataList();
        if(CollectionUtils.isNotEmpty(siteMageeAvgDatas)){
            str.append("各站点黑碳浓度");

            List<SiteMageeAvgData> siteMageeAvgDataList1 = siteMageeAvgDatas.stream().filter(a->null!=a.getPercent()&&a.getPercent()>0).collect(Collectors.toList());
            List<SiteMageeAvgData> siteMageeAvgDataList2 = siteMageeAvgDatas.stream().filter(a->null!=a.getPercent()&&a.getPercent()<0).collect(Collectors.toList());
            List<SiteMageeAvgData> siteMageeAvgDataList3 = siteMageeAvgDatas.stream().filter(a->null!=a.getPercent()&&a.getPercent()==0).collect(Collectors.toList());

            List<String> datas = new ArrayList<>();
            StringBuilder sb1;
            if(siteMageeAvgDataList1.size()>0){
                sb1=new StringBuilder();
                sb1.append(MessageFormat.format("\"{0}升\"",NumberConvert.arabicNumToChineseNum(siteMageeAvgDataList1.size())));
                List<String> strMsgs = new ArrayList<>();
                for(SiteMageeAvgData siteMageeAvgData : siteMageeAvgDataList1){
                    String s = MessageFormat.format("{0}{1}{2}",siteMageeAvgData.getSiteName(),siteMageeAvgData.getPercent()>0?"上升":(siteMageeAvgData.getPercent()<0?"下降":"持平"),siteMageeAvgData.getPercent()==0?"":Math.abs(siteMageeAvgData.getPercent().floatValue())+"%");
                    strMsgs.add(s);
                }
                sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
                datas.add(sb1.toString());
            }
            if(siteMageeAvgDataList2.size()>0){
                sb1=new StringBuilder();
                sb1.append(MessageFormat.format("\"{0}降\"",NumberConvert.arabicNumToChineseNum(siteMageeAvgDataList2.size())));
                List<String> strMsgs = new ArrayList<>();
                for(SiteMageeAvgData siteMageeAvgData : siteMageeAvgDataList2){
                    String s = MessageFormat.format("{0}{1}{2}",siteMageeAvgData.getSiteName(),siteMageeAvgData.getPercent()>0?"上升":(siteMageeAvgData.getPercent()<0?"下降":"持平"),siteMageeAvgData.getPercent()==0?"":Math.abs(siteMageeAvgData.getPercent().floatValue())+"%");
                    strMsgs.add(s);
                }
                sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
                datas.add(sb1.toString());
            }
            if(siteMageeAvgDataList3.size()>0){
                sb1=new StringBuilder();
                sb1.append(MessageFormat.format("\"{0}平\"",NumberConvert.arabicNumToChineseNum(siteMageeAvgDataList2.size())));
                List<String> strMsgs = new ArrayList<>();
                for(SiteMageeAvgData siteMageeAvgData : siteMageeAvgDataList3){
                    String s = MessageFormat.format("{0}{1}{2}",siteMageeAvgData.getSiteName(),siteMageeAvgData.getPercent()>0?"上升":(siteMageeAvgData.getPercent()<0?"下降":"持平"),siteMageeAvgData.getPercent()==0?"":Math.abs(siteMageeAvgData.getPercent().floatValue())+"%");
                    strMsgs.add(s);
                }
                sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
                datas.add(sb1.toString());
            }
            if(CollectionUtils.isNotEmpty(datas)){
                str.append(String.join("、",datas));
                str.append("。");
            }
        }
        sb.append(partText(str.toString()));
        return sb.toString();
    }

    private String createCH4Html(List<CH4AvgData> ch4AvgDataList, BasicInformation monthBasicInformation){
        StringBuilder sb = new StringBuilder();
        sb.append(headline2("（二）甲烷非甲烷总烃"));
        if(CollectionUtils.isEmpty(ch4AvgDataList)){
            sb.append(partText("以全市5个监测点位（超站、机场、铁路、公路、港口）有效数据进行评价，总烃日均浓度跨度范围为XXX~XXX ppb，月均浓度为XXX ppb；甲烷日均浓度跨度范围为XXX~XXX ppb，月均浓度为XXX ppb；非甲烷总烃日均浓度跨度范围为XXX~XXX ppb，月均浓度为XXX ppb。"));
            return sb.toString();
        }
        List<String> siteNames = monthBasicInformation.getSiteMap().values().stream().map(Site::getName).collect(Collectors.toList());
        StringBuilder str = new StringBuilder();
        if(CollectionUtils.isNotEmpty(siteNames)){
            String name = String.join("、",siteNames);
            str.append(MessageFormat.format("以全市{0}个监测点位（{1}）有效数据进行评价，",siteNames.size(),name));
        }

        List<String> strAvgS= new ArrayList<>();
        for(CH4AvgData ch4AvgData: ch4AvgDataList){
            strAvgS.add(MessageFormat.format("{0}日均浓度跨度范围为{1}~{2}ppb，月均浓度为{3}ppb",ch4AvgData.getFactorId().equals("1027")? "总烃":ch4AvgData.getFactorName(),null == ch4AvgData.getDayAvgMin() ? "--" : ch4AvgData.getDayAvgMin(),null == ch4AvgData.getDayAvgMax() ? "--" : ch4AvgData.getDayAvgMax(),null == ch4AvgData.getMonthAvg() ? "--" : ch4AvgData.getMonthAvg()));
        }

        if(CollectionUtils.isNotEmpty(strAvgS)){
            String st = String.join(";",strAvgS);
            str.append(st);
            str.append("。");
        }
        sb.append(partText(str.toString()));

        //每个因子折线图
        ch4AvgDataList.forEach(a->{
            String chartUrl = a.getChartUrl();
            if(StringUtils.isBlank(chartUrl)){
                return;
            }
            sb.append(image(chartUrl, null, null));
        });
        sb.append(textCenter("图2-2 总烃、甲烷、非甲烷总烃 浓度时序图"));

        strAvgS= new ArrayList<>();
        for(CH4AvgData ch4AvgData: ch4AvgDataList){
            String name =ch4AvgData.getFactorId().equals("1027") ? "总烃" : ch4AvgData.getFactorName();
            StringBuilder stringBuilder = new StringBuilder();
            List<SiteCH4MonthAvgData> ch4Datas = ch4AvgData.getCh4Datas();
            if(CollectionUtils.isNotEmpty(ch4Datas)){
                Collections.sort(ch4Datas, new Comparator<SiteCH4MonthAvgData>() {
                    @Override
                    public int compare(SiteCH4MonthAvgData o1, SiteCH4MonthAvgData o2) {
                        if (o1.getAvgData() == null && o2.getAvgData() == null) {
                            return 0;
                        } else if (o1.getAvgData() == null) {
                            return 1;
                        } else if (o2.getAvgData() == null) {
                            return -1;
                        } else {
                            return o2.getAvgData().compareTo(o1.getAvgData());
                        }
                    }
                });
                stringBuilder.append(MessageFormat.format("按照{0}月均值由高到低排名：",name));
                List<String>siteAvgs = new ArrayList<>();
                ch4Datas.forEach(ch4AvgData1->{
                    if(null!=ch4AvgData1.getAvgData()){
                        siteAvgs.add(MessageFormat.format("{0}（{1}ppb）",ch4AvgData1.getSiteName(),ch4AvgData1.getAvgData()));
                    }
                });
                if(CollectionUtils.isNotEmpty(siteAvgs)){
                    stringBuilder.append(String.join(">",siteAvgs));
                }
                strAvgS.add(stringBuilder.toString());
            }
        }
        if(CollectionUtils.isNotEmpty(strAvgS)){
            String st = String.join(";",strAvgS);
            sb.append(partText(st.toString() + "。"));
        }
        else {
            sb.append(partText("。"));
        }


        StringBuilder stringBuilderChain = new StringBuilder();
        stringBuilderChain.append("与上月同期相比，厦门市");
        List<String> stringList = new ArrayList<>();
        for(CH4AvgData ch4AvgData: ch4AvgDataList){
            //总烃不参与统计
            if(ch4AvgData.getFactorId().equals("1027")){
                continue;
            }
            StringBuilder stringBuilder = new StringBuilder();
            String name = ch4AvgData.getFactorName();
            if(null!=ch4AvgData.getPercent()){
                stringBuilder.append(MessageFormat.format("{0}浓度{1}{2}。",name,ch4AvgData.getPercent()>0?"上升":(ch4AvgData.getPercent()<0?"下降":"持平"),ch4AvgData.getPercent()==0?"":Math.abs(ch4AvgData.getPercent().floatValue())+"%"));
            }
            if(null == ch4AvgData.getCh4Datas()){
                continue;
            }
            List<SiteCH4MonthAvgData> siteCH4MonthAvgDataList = ch4AvgData.getCh4Datas().stream().filter(a->null!=a.getPercent()&&a.getPercent()>0).collect(Collectors.toList());
            List<SiteCH4MonthAvgData> siteCH4MonthAvgDataList1 = ch4AvgData.getCh4Datas().stream().filter(a->null!=a.getPercent()&&a.getPercent()<0).collect(Collectors.toList());
            List<SiteCH4MonthAvgData> siteCH4MonthAvgDataList2 = ch4AvgData.getCh4Datas().stream().filter(a->null!=a.getPercent()&&a.getPercent()==0).collect(Collectors.toList());

            stringBuilder.append(MessageFormat.format("各站点{0}浓度",name));
            List<String> datas = new ArrayList<>();
            StringBuilder sb1;
            if(siteCH4MonthAvgDataList.size()>0){
                sb1=new StringBuilder();
                sb1.append(MessageFormat.format("\"{0}升\"",NumberConvert.arabicNumToChineseNum(siteCH4MonthAvgDataList.size())));
                List<String> strMsgs = new ArrayList<>();
                for(SiteCH4MonthAvgData siteCH4MonthAvgData : siteCH4MonthAvgDataList){
                    String strMsg = MessageFormat.format("{0}{1}{2}",siteCH4MonthAvgData.getSiteName(),siteCH4MonthAvgData.getPercent()>0?"上升":(ch4AvgData.getPercent()<0?"下降":"持平"),siteCH4MonthAvgData.getPercent()==0?"":Math.abs(siteCH4MonthAvgData.getPercent().floatValue())+"%");
                    strMsgs.add(strMsg);
                }
                sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
                datas.add(sb1.toString());
            }
            if(siteCH4MonthAvgDataList1.size()>0){
                sb1=new StringBuilder();
                sb1.append(MessageFormat.format("\"{0}降\"",NumberConvert.arabicNumToChineseNum(siteCH4MonthAvgDataList1.size())));
                List<String> strMsgs = new ArrayList<>();
                for(SiteCH4MonthAvgData siteCH4MonthAvgData : siteCH4MonthAvgDataList1){
                    String strMsg = MessageFormat.format("{0}{1}{2}",siteCH4MonthAvgData.getSiteName(),siteCH4MonthAvgData.getPercent()>0?"上升":(ch4AvgData.getPercent()<0?"下降":"持平"),siteCH4MonthAvgData.getPercent()==0?"":Math.abs(siteCH4MonthAvgData.getPercent())+"%");
                    strMsgs.add(strMsg);
                }
                sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
                datas.add(sb1.toString());
            }
            if(siteCH4MonthAvgDataList2.size()>0){
                sb1=new StringBuilder();
                sb1.append(MessageFormat.format("\"{0}平\"",NumberConvert.arabicNumToChineseNum(siteCH4MonthAvgDataList2.size())));
                List<String> strMsgs = new ArrayList<>();
                for(SiteCH4MonthAvgData siteCH4MonthAvgData : siteCH4MonthAvgDataList1){
                    String strMsg = MessageFormat.format("{0}{1}{2}",siteCH4MonthAvgData.getSiteName(),siteCH4MonthAvgData.getPercent()>0?"上升":(ch4AvgData.getPercent()<0?"下降":"持平"),siteCH4MonthAvgData.getPercent()==0?"":Math.abs(siteCH4MonthAvgData.getPercent())+"%");
                    strMsgs.add(strMsg);
                }
                sb1.append(MessageFormat.format("（{0}）",String.join("、",strMsgs)));
                datas.add(sb1.toString());
            }
            if(CollectionUtils.isNotEmpty(datas)){
                stringBuilder.append(String.join("、",datas));
            }
            stringList.add(stringBuilder.toString());
        }

        if(CollectionUtils.isNotEmpty(stringList)){
            stringBuilderChain.append(String.join(";",stringList));
            stringBuilderChain.append("。");
            sb.append(partText(stringBuilderChain.toString()));
        }
        return sb.toString();
    }

    private String headline(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }

    private String partText(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    public static  String images(List<Map<String,String>> images , Double width,Double height){
        if(CollectionUtils.isEmpty(images)){
            return "";
        }

        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 1.5;
        String head = "<p style=\"text-align:center;\">";
        String end = "</p>";
        StringBuilder sb = new StringBuilder();
        StringBuilder stringBuilder = new StringBuilder();
        StringBuilder stringBuilder1 = new StringBuilder();
        stringBuilder.append(head);

        for(int i =0;i<images.size();i++){
            Map<String,String> image = images.get(i);
            String path = image.get("path");
            String title = image.get("siteName");
            sb.append("<img src=\"" + path + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />");
            if((i+1) % 2 == 0){
                stringBuilder.append(sb.toString());
                stringBuilder.append(end);
                stringBuilder1.append(stringBuilder.toString());

                sb = new StringBuilder();
                stringBuilder =new StringBuilder();
                stringBuilder.append(head);
            }
        }
        return stringBuilder1.toString();
    }
}
