package more.ssaqms.xiamen.handler.month.traffic;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import more.ssaqms.xiamen.model.traffic.*;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.RestTemplateUtils;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

import static more.ssaqms.xiamen.util.MonthReportUtils.table;
import static more.ssaqms.xiamen.util.MonthReportUtils.tr;

@Component
public class TrafficChapter3 {

    private Map<String,String> siteDescMap = new HashMap<String,String>(){{
        put("1721488148709826562","站点北边是居民区和公路，东边主要是商贸区和交通干道，南边有交通干道、建筑工地和机动车检测公司，西边主要为建筑工地。");
        put("1721488148709826599","站点西－西北－北面是机场跑道、东向和西南方位是物流分拨中心和交通干道，");
        put("1721488148709826602","站点北边为铁路货场和铁路支线，东-南边主要为海翔大道、正新橡胶工业有限公司，西边主要为居民区和物流中心。");
        put("1721488148709826600","公路点南－东南－东是海沧大道，西北方向主要是滨湖路和其他交通干道，");
        put("1721488148709826601","港口点西北－东北主要是仓储物流区域，东－东南－南为码头仓储区，");
    }};

    public String html(List<SiteWindModel>siteWindModels){
        StringBuilder sb = new StringBuilder();
        sb.append(headline("三、气象分析"));
        if(CollectionUtils.isEmpty(siteWindModels)){
            return sb.toString();
        }
        for(int i=0;i<siteWindModels.size();i++){
            SiteWindModel siteWindModel = siteWindModels.get(i);
            Integer index = i+1;
            sb.append(partText(MessageFormat.format("{0}.{1}",index,siteWindModel.getSiteName())));
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append(MessageFormat.format("{0}不同风频下污染物浓度的变化，见图3-{1}和表3-{2}。",siteWindModel.getSiteName(),index,index));
            List<WindstatisticsModel> windstatisticsModelList = siteWindModel.getWindstatisticsModelList();
            String windName="--";
            String desc = "--";
            Float percent = null;
            if(CollectionUtils.isNotEmpty(windstatisticsModelList)){
                List<WindstatisticsModel> windSortstatisticsModelList = windstatisticsModelList.stream().sorted(Comparator.comparing(WindstatisticsModel::getPercent).reversed()).collect(Collectors.toList());
                WindstatisticsModel windstatisticsModel = windSortstatisticsModelList.get(0);
                windName =windstatisticsModel.getName();
                desc = windstatisticsModel.getWindName();
                percent = windstatisticsModel.getPercent();
            }
            stringBuilder.append(MessageFormat.format("{0}以{1}（{2}°）为主，占总风频的{3}%。",siteWindModel.getSiteName(),windName,desc,percent));
            String siteDesc = "--";
            if(siteDescMap.containsKey(siteWindModel.getSiteId())){
                siteDesc = siteDescMap.get(siteWindModel.getSiteId());
            }
            stringBuilder.append(siteDesc);
            stringBuilder.append("污染高值可能受到XXX影响。");
            String factorNames = "--";
            List<WindstatisticsModel> windstatisticsModels = siteWindModel.getWindstatisticsModelList();
            if(CollectionUtils.isNotEmpty(windstatisticsModels)){
                List<SiteFactorModel> siteFactorModelList = windstatisticsModels.get(0).getSiteFactorModelList();
                if(CollectionUtils.isNotEmpty(siteFactorModelList)){
                    List<String> names = siteFactorModelList.stream().map(a->a.getName()).collect(Collectors.toList());
                    if(CollectionUtils.isNotEmpty(names)){
                        factorNames = String.join("、",names);
                    }
                }
            }
            stringBuilder.append(MessageFormat.format("其中{0}高值可能受到XXX的影响。",factorNames));

            sb.append(partText(stringBuilder.toString()));

            Map<String,String> factorChartMap = siteWindModel.getChartPollutantRoseData();
            Map<String,String> factorChart = new HashMap<>();
            factorChartMap.forEach((factor,chartUrl)->{
                if(!factorChart.containsKey(factor)){
                    factorChart.put(factor,chartUrl);
                }
            });

            sb.append(images(new ArrayList<>(factorChart.values()), null,null));
            sb.append(textCenter(MessageFormat.format("图3-{0} {1}主要污染物风玫瑰图", index,siteWindModel.getSiteName())));

            //创建表格
            String table = createAqiTable(siteWindModel,index);
            if(!table.equals("")){
                sb.append(table);
            }
        }
        return sb.toString();
    }

    private String createAqiTable(SiteWindModel siteWindModel,Integer index){
        if(null == siteWindModel){
            return "";
        }

        StringBuilder trStr = new StringBuilder();
        trStr.append(textCenter(MessageFormat.format("表3-{0} {1}不同风向下风频及污染高值占比", index,siteWindModel.getSiteName())));
        StringBuilder tdBuffer = new StringBuilder();
        tdBuffer.append(overstrikingTd(1,1,"风向°"));
        tdBuffer.append(overstrikingTd(1,1,"风频%"));
        List<SiteFactorModel> siteFactorModelList = siteWindModel.getWindstatisticsModelList().get(0).getSiteFactorModelList();
        for(SiteFactorModel siteFactorModel:siteFactorModelList){
            tdBuffer.append(overstrikingTd(1,1,MessageFormat.format("{0}，% <br/>（>{1}{2}）",siteFactorModel.getName(),siteFactorModel.getThreshold(),siteFactorModel.getUnit())));
        }
        trStr.append(tr(tdBuffer.toString()));
        List<WindstatisticsModel> windstatisticsModelList = siteWindModel.getWindstatisticsModelList();
        for(WindstatisticsModel windstatisticsModel:windstatisticsModelList){
            tdBuffer = new StringBuilder();
            tdBuffer.append(overstrikingTd(1,1,windstatisticsModel.getWindName()));
            tdBuffer.append(overstrikingTd(1,1,windstatisticsModel.getPercent().toString()));
            List<SiteFactorModel> siteFactorModels = windstatisticsModel.getSiteFactorModelList();
            for(SiteFactorModel siteFactorModel:siteFactorModels){
                tdBuffer.append(overstrikingTd(1,1,siteFactorModel.getPercent()==null?"--":siteFactorModel.getPercent().toString()));
            }
            trStr.append(tr(tdBuffer.toString()));
        }
        return table(trStr.toString());
    }

    private String headline(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    private String partText(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    private String overstrikingTd(Integer colspan, Integer rowspan, String text) {
        return "<td colspan=\"" + colspan + "\" rowspan=\"" + rowspan + "\" style=\"text-align:center\">" +
                "<p style=\"text-align:center;\">" +
                "    <span style=\"font-size: 10.5pt; font-family: 仿宋, SimSun;\">" + text + "</span>" +
                "</p>" +
                "</td>";
    }

    private String textCenter(String text) {
        return "<p style=\"margin: 0px; text-align: center; line-height: 1.1;\">" +
                "<span style=\"font-size: 12pt; font-weight:bold; font-family: 宋体, SimSun;\">" + text + "</span>" +
                "</p>";
    }

    /**
     * 图片
     * @param path 路径
     * @param width 宽
     * @param height 高
     * @return
     */
    public static String image(String path, Double width, Double height) {
        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 2.34375;
        return "<p style=\"text-align:center;\">" +
                "<img src=\"" + path + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />" +
                "</p>";
    }

    public static  String images(List<String> paths , Double width,Double height){
        if(CollectionUtils.isEmpty(paths)){
            return "";
        }

        String widthStr = "";
        if (!Objects.isNull(width)) widthStr = "width:" + width + "in;";
        if (Objects.isNull(height)) height = 1.5;
        String head = "<p style=\"text-align:center;\">";
        String end = "</p>";
        StringBuilder sb = new StringBuilder();
        StringBuilder stringBuilder = new StringBuilder();
        StringBuilder stringBuilder1 = new StringBuilder();
        stringBuilder.append(head);
        for(int i =0;i<paths.size();i++){
            String path = paths.get(i);
            sb.append("<img src=\"" + path + "\" style=\"" + widthStr + "height:" + height + "in;vertical-align:text-align：center;\" />");
            if((i+1) % 3 == 0){
                stringBuilder.append(sb.toString());
                stringBuilder.append(end);
                stringBuilder1.append(stringBuilder.toString());

                sb = new StringBuilder();
                stringBuilder =new StringBuilder();
                stringBuilder.append(head);
            }
        }
        return stringBuilder1.toString();
    }
}
