package more.ssaqms.xiamen.handler.month.traffic;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import more.ssaqms.xiamen.model.traffic.*;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

@Component
public class TrafficChapter4 {

    public static final Map<String, String> FACTOR_NAME_MAPPING = new HashMap() {{
        put("1001", "SO₂");
        put("1003", "NO₂");
        put("1006", "O₃");
        put("1007", "PM₁₀");
        put("1008", "PM₂.₅");
        put("1005", "CO");
    }};
    public String html(List<SiteAqiData> siteAqiDatas,List<CH4AvgData> ch4AvgDataList,MageeAvgData mageeAvgData,VOCsData voCsData,List<SiteWindModel> siteWindModels){
        StringBuilder sb = new StringBuilder();
        sb.append(headline("四、结论"));
        sb.append(headline2("（一）常规指标结论"));
        Integer index = 1;

        for(SiteAqiData siteAqiData:siteAqiDatas){
            StringBuilder str = new StringBuilder();
            if(siteAqiData.getSiteId().equals("1721488148709826600")){
                List<PollutantData> pollutantDatas = siteAqiData.getPollutantDatas();
                if(CollectionUtils.isNotEmpty(pollutantDatas)){
                    List<String>names = new ArrayList<>();
                    List<String> factorValues = new ArrayList<>();
                    pollutantDatas.forEach(pollutantData -> {
                        String id =pollutantData.getFactorId();
                        if(FACTOR_NAME_MAPPING.containsKey(id)){
                            names.add(FACTOR_NAME_MAPPING.get(id));
                        }
                        else{
                            names.add(pollutantData.getFactorName());
                        }
                        factorValues.add(MessageFormat.format("{0}{1}", pollutantData.getValue(), pollutantData.getFactorName().equals("CO") ? "毫克/立方米" : "微克/立方米"));
                    });

                    String datas = String.join("、",factorValues);
                    str.append(MessageFormat.format("{0}.{1}空气中主要污染物 {2} 平均浓度分别为{3}",index,siteAqiData.getSiteName(),String.join("、",names),datas));
                }
            }
            else {
                str.append(MessageFormat.format("{0}.{1}空气质量综合指数为{2}，",index,siteAqiData.getSiteName(),siteAqiData.getValue()));
                List<PollutantData> pollutantDatas = siteAqiData.getComposites();
                if (CollectionUtils.isNotEmpty(pollutantDatas)) {
                    str.append("其中");
                    List<String> msg = new ArrayList<>();
                    pollutantDatas.forEach(a -> {
                        String name = a.getFactorName();
                        if(FACTOR_NAME_MAPPING.containsKey(a.getFactorId())){
                            name = FACTOR_NAME_MAPPING.get(a.getFactorId());
                        }
                        if (null != a.getPercent()) {
                            msg.add(MessageFormat.format("{0}贡献率{1}%", name, a.getPercent()));
                        }
                    });
                    if (CollectionUtils.isNotEmpty(msg)) {
                        str.append(String.join("、", msg));
                    }
                }
            }
            index++;
            str.append("。");
            sb.append(partText(str.toString()));
        }

        String vocsHtml = createVocsResult(ch4AvgDataList,mageeAvgData,voCsData);
        if(!vocsHtml.equals("")){
            sb.append(vocsHtml);
        }

        String strWind = createWindResult(siteWindModels);
        if(!strWind.equals("")){
            sb.append(strWind);
        }

        return sb.toString();
    }

    public String createVocsResult(List<CH4AvgData> ch4AvgDataList,MageeAvgData mageeAvgData,VOCsData voCsData){
        StringBuilder sb = new StringBuilder();
        sb.append(headline2("（二）其他污染物结论"));
        StringBuilder stringBuilder = new StringBuilder();
        String value = "--";
        if(null!=mageeAvgData&&mageeAvgData.getMonthAvg()!=null){
            value = mageeAvgData.getMonthAvg().toString();
        }
        stringBuilder.append(MessageFormat.format("1.厦门市黑碳月均浓度为{0}ng/m³。",value));
        if(mageeAvgData !=null && mageeAvgData.getTotalMonthAvgSpamsData()!=null && mageeAvgData.getTotalMonthAvgSpamsData().size()>0){
            List<SpamsData> totalMonthAvgSpamsData = mageeAvgData.getTotalMonthAvgSpamsData();
            Collections.sort(totalMonthAvgSpamsData, new Comparator<SpamsData>() {
                @Override
                public int compare(SpamsData p1, SpamsData p2) {
                    return Float.compare(p2.getPercent(), p1.getPercent());
                }
            });
            for(int i=0;i<totalMonthAvgSpamsData.size();i++){
                SpamsData spamsData = totalMonthAvgSpamsData.get(i);
                if(i==0){
                    stringBuilder.append(MessageFormat.format("BC主要来自{0}，月均浓度为{1}ng/m³，占总BC的{2}%；", spamsData.getFactorName(), spamsData.getValue(), spamsData.getPercent()));
                }
                else{
                    stringBuilder.append(MessageFormat.format("{0}产生的BC的月均浓度为{1}ng/m³，占总BC的{2}%。", spamsData.getFactorName(), spamsData.getValue(), spamsData.getPercent()));
                }
            }
        }
        else {
            stringBuilder.append("BC主要来自移动源，月均浓度为--ng/m³，占总BC的--%；生物质燃烧产生的BC的月均浓度为-- ng/m³，占总BC的--%。");
        }
        sb.append(partText(stringBuilder.toString()));

        stringBuilder = new StringBuilder();
        List<String> msg = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(ch4AvgDataList)) {
            for (CH4AvgData ch4AvgData : ch4AvgDataList) {
                msg.add(MessageFormat.format("{0}月均浓度为{1}ppb", ch4AvgData.getFactorId().equals("1027")?"总烃":ch4AvgData.getFactorName(), null == ch4AvgData.getMonthAvg() ? "--" : ch4AvgData.getMonthAvg()));
            }
        }
        if(CollectionUtils.isNotEmpty(msg)){
            stringBuilder.append("2.厦门市");
            stringBuilder.append(String.join("、",msg));
            stringBuilder.append("。");
            sb.append(partText(stringBuilder.toString()));
        }

        stringBuilder = new StringBuilder();
        if(null!=voCsData) {
            stringBuilder.append(MessageFormat.format("3.厦门市VOCs月均浓度为{0}ppb。", voCsData.getMonthAvg() == null ? "--" : voCsData.getMonthAvg()));

            List<VOCsMonthDataDetail> voCsMonthDataDetails = voCsData.getVoCsMonthDataDetails();
            if (CollectionUtils.isNotEmpty(voCsMonthDataDetails)) {
                stringBuilder.append("VOCs检出浓度占比最高的是");
                List<String> msgs = new ArrayList<>();
                voCsMonthDataDetails.forEach(voCsMonthDataDetail -> {
                    if (null != voCsMonthDataDetail.getPercent()) {
                        msgs.add(MessageFormat.format("{0}占比为{1}%", voCsMonthDataDetail.getFactorName(), voCsMonthDataDetail.getPercent()));
                    }
                });

                if (CollectionUtils.isNotEmpty(msgs)) {
                    stringBuilder.append(String.join("、", msgs));
                    stringBuilder.append("。");
                }
            }
            sb.append(partText(stringBuilder.toString()));
        }

        return sb.toString();
    }

    private String createWindResult(List<SiteWindModel> siteWindModels){
        StringBuilder sb = new StringBuilder();
        sb.append(headline2("（三）气象分析结论"));

        if(CollectionUtils.isEmpty(siteWindModels)){
            return sb.toString();
        }
        int index = 1;
        for(SiteWindModel siteWindModel:siteWindModels){
            StringBuilder stringBuilder = new StringBuilder();
            List<WindstatisticsModel> windstatisticsModelList = siteWindModel.getWindstatisticsModelList();
            if(CollectionUtils.isNotEmpty(windstatisticsModelList)){
                List<SiteFactorModel> siteFactorModelList = windstatisticsModelList.get(0).getSiteFactorModelList();
                if(CollectionUtils.isNotEmpty(siteFactorModelList)){
                    stringBuilder.append(MessageFormat.format("{0}.{1}",index,siteWindModel.getSiteName()));
                    List<String> factorNames = siteFactorModelList.stream().map(a->a.getName()).collect(Collectors.toList());
                    stringBuilder.append(MessageFormat.format("{0}高值可能受到XXX影响。",String.join("、",factorNames)));
                    sb.append(partText(stringBuilder.toString()));
                    index++;
                }
            }
        }
        return sb.toString();
    }

    private String partText(String text) {
        return "<p style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span style=\"font-size: 16pt; font-family: 仿宋_GB2312, SimSun;\">" + text + "</span>\n" +
                "</p>";
    }

    private String headline(String text) {
        return "<h1 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h1>";
    }

    public String headline2(String text) {
        return "<h2 style=\"margin: 0px; text-indent: 2em; line-height: 1.1;\">" +
                "<span>" +
                "<strong style=\"font-size: 16pt; font-family: 黑体, SimSun;\">" + text + "</strong>" +
                "</span>" +
                "</h2>";
    }
}
