package more.ssaqms.xiamen.model.data;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.pmf.QueryInput;

import javax.validation.constraints.NotNull;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/7/15 18:52
 */
@ApiModel(value = "数据查询接收参数模型")
@Data
@ToString
@Accessors(chain = true)
public class DataReceive {

    @NotNull
    @ApiModelProperty(value = "查询开始时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date beginTime;

    @NotNull
    @ApiModelProperty(value = "查询结束时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date endTime;

    @ApiModelProperty(value = "查询时间LIST", required = true)
    private List<String> queryTimes;

    //    @NotBlank
    @ApiModelProperty(value = "站点设备Id", required = true)
    private String siteDeviceId;

    @ApiModelProperty(value = "dataType", required = true)
    private Integer dataType;

    @ApiModelProperty(value = "因子Id", required = true)
    private String factorId;

    @ApiModelProperty(value = "时候查询审核后数据,不传或false为查全部", required = true)
    private Boolean isAuditCompleted;

    @ApiModelProperty(value = "数据查询使用,获取查询设备和查询因子", required = true)
    private List<SiteDeviceReceive> siteDevices;

    @ApiModelProperty(value = "站点ids", required = true)
    private List<String> siteIds;

    @ApiModelProperty(value = "因子ids", required = true)
    private List<String> factorIds;

    @ApiModelProperty(value = "unit单位", required = true)
    private String unit;

    @ApiModelProperty(value = "是否是显示单位", required = true)
    private Boolean isDisplayUnit;

    @ApiModelProperty(value = "Aqi等级", required = true)
    private Integer grade;

    @ApiModelProperty(value = "时间list,用于查询不同时段的数据", required = true)
    private List<Date> dates;

    @ApiModelProperty(value = "VOC分类查询使用", required = true)
    private Boolean isAvg;

    @ApiModelProperty(value = "查询时间粒度", required = true)
    private String granularity;

    @ApiModelProperty(value = "是否需要无效数据", required = true)
    private Boolean isGetInvalidData;

    @ApiModelProperty(value = "是否AuditFlag", required = true)
    private Boolean isAuditFlag;

    @ApiModelProperty(value = "LIMIT的个数", required = true)
    private Integer num;

    @ApiModelProperty(value = "是否重积分数据，不传为false")
    private Boolean isMultipleIntegral;

    @ApiModelProperty(value = "是否校验权限", hidden = true)
    private Boolean isCheckPermission;

    @ApiModelProperty(value = "是否融合数据（有审核后查询审核后去除无效数据的，繁殖查询审核前全部数据），不传为false")
    private Boolean isFuse;

    @ApiModelProperty(value = "是否需要tvoc计算，为分组查询添加（赵大师要求）")
    private Boolean tvoc;

    public DataReceive() {
        this.isCheckPermission = true;
    }

    public DataReceive(DataReceive dataReceive) {
        this.beginTime = dataReceive.getBeginTime();
        this.endTime = dataReceive.getEndTime();
        this.queryTimes = dataReceive.getQueryTimes();
        this.siteDeviceId = dataReceive.getSiteDeviceId();
        this.dataType = dataReceive.getDataType();
        this.factorId = dataReceive.getFactorId();
        this.isAuditCompleted = dataReceive.getIsAuditCompleted();
        this.siteDevices = dataReceive.getSiteDevices();
        this.siteIds = dataReceive.getSiteIds();
        this.factorIds = dataReceive.getFactorIds();
        this.unit = dataReceive.getUnit();
        this.isDisplayUnit = dataReceive.getIsDisplayUnit();
        this.grade = dataReceive.getGrade();
        this.dates = dataReceive.getDates();
        this.isAvg = dataReceive.getIsAvg();
        this.granularity = dataReceive.getGranularity();
        this.isGetInvalidData = dataReceive.getIsGetInvalidData();
        this.isAuditFlag = dataReceive.getIsAuditFlag();
        this.num = dataReceive.getNum();
        this.isMultipleIntegral = dataReceive.getIsMultipleIntegral();
        this.isCheckPermission = dataReceive.getIsCheckPermission();
        this.isFuse = dataReceive.getIsFuse();
        this.tvoc = dataReceive.getTvoc();
    }

    public DataReceive(QueryInput queryInput) {
        this.isCheckPermission = true;
        this.beginTime = queryInput.getBeginTime();
        this.endTime = queryInput.getEndTime();
        this.dataType = 60;
        Integer dataType = queryInput.getDataType();
        if (1440 == dataType) {
            this.granularity = "day";
        }
        this.siteIds = Collections.singletonList(queryInput.getSiteId());
        this.factorIds = queryInput.getFactorIds();
        Integer auditType = queryInput.getAuditType();
        //原始数据
        if (1 == auditType) {
            this.isGetInvalidData = true;
        } else if (2 == auditType) {
            this.isAuditCompleted = true;
            this.isGetInvalidData = false;
            this.isMultipleIntegral = true;
        } else if (3 == auditType) {
            this.isFuse = true;
        }
    }
}
