package more.ssaqms.xiamen.model.data;

import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.experimental.Accessors;

import java.util.Date;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/7/13 19:12
 */
@Getter
@Setter
@ToString
@Accessors(chain = true)
public class WindProfileRadarData {

    /**
     * 风速id
     */
    @ApiModelProperty(value = "风速id", required = true)
    private String windSpeedId;

    /**
     * 风向id
     */
    @ApiModelProperty(value = "风向id", required = true)
    private String windDirectionId;

    /**
     * 数据时间
     */
    @ApiModelProperty(value = "数据时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date dataTime;

    /**
     * 数采因子值(有从新生成的显示从新生成值,没有就显示原始值)
     */
    @ApiModelProperty(value = "数采因子值(有从新生成的显示从新生成值,没有就显示原始值)", required = true)
    @TableField
    private String dasWindSpeedValue;

    /**
     * 数采因子值(有从新生成的显示从新生成值,没有就显示原始值)
     */
    @ApiModelProperty(value = "数采因子值(有从新生成的显示从新生成值,没有就显示原始值)", required = true)
    @TableField
    private String dasWindDirectionValue;

    /**
     * 数采标签
     */
    @ApiModelProperty(value = "数采标签", required = true)
    @TableField
    private String dasWindSpeedFlag;

    /**
     * 数采标签
     */
    @ApiModelProperty(value = "数采标签", required = true)
    @TableField
    private String dasWindDirectionFlag;

    /**
     * 审核标签
     */
    @ApiModelProperty(value = "风速审核标签", required = true)
    @TableField
    private String auditFlag;

    /**
     * 审核意见
     */
    @ApiModelProperty(value = "风向审核标签", required = true)
    @TableField
    private String auditComment;

    /**
     * 是否审核完成
     */
    @ApiModelProperty(value = "是否审核完成", required = true)
    @TableField
    private Boolean isAuditCompleted;

    public WindProfileRadarData() {
    }

    public WindProfileRadarData(RadarData radarData) {
        String factorId = radarData.getFactorId();
        if ("7100".equals(factorId)){
            this.windDirectionId = radarData.getId();
            this.dasWindDirectionValue = radarData.getDasValue();
            this.dasWindDirectionFlag = radarData.getDasFlag();
        }else if ("7101".equals(factorId)){
            this.windSpeedId = radarData.getId();
            this.dasWindSpeedValue = radarData.getDasValue();
            this.dasWindSpeedFlag = radarData.getDasFlag();
        }
        this.auditFlag = radarData.getAuditFlag();
        this.auditComment = radarData.getAuditComment();
        this.dataTime = radarData.getDataTime();
        this.isAuditCompleted = radarData.getIsAuditCompleted();
    }
}
