package more.ssaqms.xiamen.model.device;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Map;
import java.util.Objects;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/6/8 15:11
 */
@Getter
@Setter
@ToString
@Accessors(chain = true)
@TableName(value = "t_device")
public class Device {
    /**
     * 主键
     */
    @ApiModelProperty(value = "主键", required = true)
    @TableId(type = IdType.ASSIGN_ID)
    private String id;

    /**
     * 仪器品牌
     */
    @ApiModelProperty(value = "仪器品牌", required = true)
    @TableField
    @NotBlank
    private String brand;

    /**
     * 设备品牌
     */
    @ApiModelProperty(value = "仪器型号", required = true)
    @TableField
    @NotBlank
    private String model;

    /**
     * 平台仪器名称
     */
    @ApiModelProperty(value = "平台仪器名称", required = true)
    @TableField
    @NotBlank
    private String name;

    /**
     * 数采仪器名称
     */
    @ApiModelProperty(value = "数采仪器名称", required = true)
    @TableField
    @NotBlank
    private String stdName;

    /**
     * 仪器类型Id(二进制)
     */
    @ApiModelProperty(value = "仪器类型Id(二进制)", required = true)
    @TableField
    @NotNull
    private Long deviceCategoryId;

    /**
     * 仪器传输编码
     */
    @ApiModelProperty(value = "仪器传输编码", required = true)
    @TableField
    @NotBlank
    private String code;

    /**
     * 数据频率(单位：分钟)
     */
    @ApiModelProperty(value = "数据频率(单位：分钟)", required = true)
    @TableField
    @NotNull
    private Integer dataFrequency;

    /**
     * 数据最大延迟时间(单位：分钟)
     */
    @ApiModelProperty(value = "数据最大延迟时间(单位：分钟)", required = true)
    @TableField
    @NotNull
    private Integer maxDelayTime;

    /**
     * 仪器量程下限
     */
    @ApiModelProperty(value = "仪器量程下限", required = true)
    @TableField
    @NotNull
    private Float lowRange;

    /**
     * 量程上限
     */
    @ApiModelProperty(value = "量程上限", required = true)
    @TableField
    @NotNull
    private Float highRange;

    /**
     * 类型：0：服务器下载；1：自定
     */
    @ApiModelProperty(value = "类型：0：服务器下载；1：自定", required = true)
    @TableField
    @NotNull
    private Integer type;

    /**
     * 高度(雷达数据使用)
     */
    @ApiModelProperty(value = "高度(雷达数据使用)")
    @TableField
    private String height;

    /**
     * 检测因子类别
     */
    @ApiModelProperty(value = "检测因子类别")
    @TableField
    private String factorCategory;

    public Device() {
    }

    public Device(Map<String, Object> dataDevice, Map<String, Long> map) {
        this.id = dataDevice.get("id").toString();
        this.brand = dataDevice.get("brandName").toString();
        this.model = dataDevice.get("modelName").toString();
        this.name = Objects.isNull(dataDevice.get("displayName")) ? "" : dataDevice.get("displayName").toString();
        this.stdName = Objects.isNull(dataDevice.get("deviceName")) ? "" : dataDevice.get("deviceName").toString();
        this.deviceCategoryId = map.get(dataDevice.get("typeId").toString());
        this.code = dataDevice.get("code").toString();
        this.dataFrequency = Integer.valueOf(dataDevice.get("dataFrequency").toString());
        this.maxDelayTime = 0;
        this.lowRange = Float.valueOf(dataDevice.get("lowRange").toString());
        this.highRange = Float.valueOf(dataDevice.get("highRange").toString());
        this.type = 0;
    }
}
