package more.ssaqms.xiamen.model.device;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Map;
import java.util.Objects;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/6/10 10:27
 */
@Getter
@Setter
@ToString
@Accessors(chain = true)
@TableName(value = "t_device_status")
public class DeviceStatus {

    /**
     * 主键
     */
    @ApiModelProperty(value = "主键", required = true)
    @TableId(type = IdType.ASSIGN_ID)
    private String id;

    /**
     * 状态名
     */
    @ApiModelProperty(value = "状态名", required = true)
    @TableField
    @NotBlank
    private String name;

    /**
     * 状态码
     */
    @ApiModelProperty(value = "状态码", required = true)
    @TableField
    @NotBlank
    private String code;

    /**
     * 设备ID
     */
    @ApiModelProperty(value = "设备ID", required = true)
    @TableField
    @NotBlank
    private String deviceId;

    /**
     * 度量单位
     */
    @ApiModelProperty(value = "度量单位", required = true)
    @TableField
    private String unit;

    /**
     * 仪器状态值类型
     */
    @ApiModelProperty(value = "仪器状态值类型id", required = true)
    @TableField
    private String valueTypeId;

    /**
     * 状态
     */
    @ApiModelProperty(value = "状态", required = true)
    @TableField
    @NotNull
    private Boolean status;

    /**
     * 类型：0：服务器下载；1：自定
     */
    @ApiModelProperty(value = "类型：0：服务器下载；1：自定", required = true)
    @TableField
    @NotNull
    private Integer type;

    @ApiModelProperty(value = "仪器状态保留位数")
    @TableField
    private Integer digitalCount;

    @ApiModelProperty(value = "是否关键参数")
    @TableField
    private Boolean isImportance;

    @ApiModelProperty(value = "告警上限")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMax;

    @ApiModelProperty(value = "告警下限")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMin;

    public DeviceStatus() {
    }

    public DeviceStatus(Map<String, Object> deviceStatus) {
        this.id = deviceStatus.get("id").toString();
        this.code = deviceStatus.get("code").toString();
        this.deviceId = deviceStatus.get("deviceId").toString();
        this.name = deviceStatus.get("cnName").toString();
        this.unit = Objects.isNull(deviceStatus.get("unit")) ? null : deviceStatus.get("unit").toString();
        this.digitalCount = Objects.isNull(deviceStatus.get("digitalCount")) ? null : Integer.valueOf(deviceStatus.get("digitalCount").toString());
        this.isImportance = !Objects.isNull(deviceStatus.get("isImportance")) ? null : Boolean.valueOf(deviceStatus.get("isImportance").toString());
        this.alarmMax = Objects.isNull(deviceStatus.get("alarmMax")) ? null : Float.valueOf(deviceStatus.get("alarmMax").toString());
        this.alarmMin = Objects.isNull(deviceStatus.get("alarmMin")) ? null : Float.valueOf(deviceStatus.get("alarmMin").toString());
        this.status = true;
        this.type = 0;
    }
}
