package more.ssaqms.xiamen.model.factor;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Map;
import java.util.Objects;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/6/10 9:39
 */
@Getter
@Setter
@ToString
@Accessors(chain = true)
@TableName(value = "t_factor")
public class Factor {
    /**
     * 主键
     */
    @ApiModelProperty(value = "主键", required = true)
    @TableId(type = IdType.ASSIGN_ID)
    private String id;

    /**
     * 扩展编码
     */
    @ApiModelProperty(value = "因子编码", required = true)
    @TableField
    @NotBlank
    private String factorCode;

    /**
     * 中文名
     */
    @ApiModelProperty(value = "中文名", required = true)
    @TableField
    @NotBlank
    private String cnName;

    /**
     * 因子分组id
     */
    @ApiModelProperty(value = "因子分组id", required = true)
    @TableField
    private String groupId;

    /**
     * 英文名
     */
    @ApiModelProperty(value = "英文名", required = true)
    @TableField
    private String enName;

    /**
     * 别名
     */
    @ApiModelProperty(value = "别名", required = true)
    @TableField
    private String alias;

    /**
     * 展示名
     */
    @ApiModelProperty(value = "展示名", required = true)
    @TableField
    private String displayName;

    /**
     * 分子式
     */
    @ApiModelProperty(value = "分子式", required = true)
    @TableField
    private String formula;

    /**
     * 分子量
     */
    @ApiModelProperty(value = "分子量", required = true)
    @TableField
    private Float molecular;

    /**
     * 根因子Id
     */
    @ApiModelProperty(value = "根因子Id", required = true)
    @TableField
    private String rootFactorId;

    /**
     * 所有单位
     */
    @ApiModelProperty(value = "所有单位", required = true)
    @TableField
    private String allUnits;

    /**
     * 显示单位
     */
    @ApiModelProperty(value = "显示单位", required = true)
    @TableField
    private String displayUnit;

    /**
     * 显示单位1
     */
    @ApiModelProperty(value = "显示单位1")
    @TableField
    private String displayUnit1;

    /**
     * 标准单位
     */
    @ApiModelProperty(value = "标准单位", required = true)
    @TableField
    private String stdUnit;

    /**
     * 因子统一编码，国家编码
     */
    @ApiModelProperty(value = "因子统一编码，国家编码", required = true)
    @TableField
    private String cas;

    /**
     * 监测数据值的小数位数，即精度
     */
    @ApiModelProperty(value = "监测数据值的小数位数，即精度", required = true)
    @TableField
    private Integer decimalNum;

    /**
     * 0：删除，1：使用
     */
    @ApiModelProperty(value = "状态,0：删除，1：使用", required = true)
    @TableField
    @NotNull
    private Boolean status;

    /**
     * 扩展编码
     */
    @ApiModelProperty(value = "分子式", required = true)
    @TableField
    private String mf;

    /**
     * 扩展编码
     */
    @ApiModelProperty(value = "编码类型", required = true)
    @TableField
    private String codeType;

    /**
     * 显示数据类型(1.表格,2.图形,3.表格和图形)
     */
    @ApiModelProperty(value = "显示数据类型(1.表格,2.图形,3.表格和图形)", required = true)
    @TableField
    @NotNull
    private Integer displayType;

    /**
     * 类型：0：服务器下载；1：自定
     */
    @ApiModelProperty(value = "类型：0：服务器下载；1：自定", required = true)
    @TableField
    @NotNull
    private Integer type;

    @ApiModelProperty(value = "是否报警（0：不报警，1：报警）")
    @TableField
    private Boolean isAlarm;

    @ApiModelProperty(value = "超标报警上限", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMax;

    @ApiModelProperty(value = "超标报警下限", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMin;

    @ApiModelProperty(value = "修正下限值")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float amendLowRange;

    @ApiModelProperty(value = "修正上限值", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float amendHighRange;

    @ApiModelProperty(value = "修正值", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float amendValue;

    @ApiModelProperty(value = "超标下限", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float limitLow;

    @ApiModelProperty(value = "超标上限", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float limitHigh;

    @ApiModelProperty(value = "mir")
    @TableField
    private Float mir;

    @ApiModelProperty(value = "KOH是物种i 与 OH 自由基反应速率常数")
    @TableField
    private Float koh;

    @ApiModelProperty(value = "SOA生成系数")
    @TableField
    private Float fac;

    @ApiModelProperty(value = "VOCs物种中参与反应的分数")
    @TableField
    private Float fvocr;

    @ApiModelProperty(value = "方法检出限")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float detectionLimit;

    @ApiModelProperty(value = "顔色")
    @TableField(value = "`color`")
    private String color;

    public Factor() {
    }

    public Factor(Map<String, Object> factor) {
        this.id = factor.get("id").toString();
        this.factorCode = factor.get("code").toString();
        this.cnName = factor.get("cnName").toString();
        this.groupId = Objects.isNull(factor.get("categoryId")) ? null : factor.get("categoryId").toString();
        this.enName = Objects.isNull(factor.get("enName")) ? null : factor.get("enName").toString();
        this.alias = Objects.isNull(factor.get("alias")) ? null : factor.get("alias").toString();
        this.displayName = factor.get("cnName").toString();
        this.molecular = Objects.isNull(factor.get("weight")) || factor.get("weight").toString().equals("0") ? null : Float.valueOf(factor.get("weight").toString());
        this.rootFactorId = Objects.isNull(factor.get("rootFactorId")) ? null : factor.get("rootFactorId").toString();
        this.allUnits = Objects.isNull(factor.get("units")) ? null : factor.get("units").toString();
        this.displayUnit = Objects.isNull(factor.get("dasUnit")) ? null : factor.get("dasUnit").toString();
        this.displayUnit1 = Objects.isNull(factor.get("dasUnit")) ? null : factor.get("dasUnit").toString();
        this.stdUnit = Objects.isNull(factor.get("unit")) ? null : factor.get("unit").toString();
        this.cas = Objects.isNull(factor.get("cas")) ? null : factor.get("cas").toString();
        this.decimalNum = 3;
        this.status = true;
        this.mf = Objects.isNull(factor.get("mf")) ? null : factor.get("mf").toString();
        this.codeType = Objects.isNull(factor.get("codeType")) ? null : factor.get("codeType").toString();
        this.displayType = Integer.valueOf(factor.get("displayType").toString());
        this.type = 0;
        this.mir = Objects.isNull(factor.get("mir")) ? null : Float.valueOf(factor.get("mir").toString());
        this.koh = Objects.isNull(factor.get("koh")) ? null : Float.valueOf(factor.get("koh").toString());
        this.fac = Objects.isNull(factor.get("fac")) ? null : Float.valueOf(factor.get("fac").toString());
        this.fvocr = Objects.isNull(factor.get("fvocr")) ? null : Float.valueOf(factor.get("fvocr").toString());
        this.detectionLimit = Objects.isNull(factor.get("detectionLimit")) ? null : Float.valueOf(factor.get("detectionLimit").toString());
        this.color = Objects.isNull(factor.get("color")) ? null : factor.get("color").toString();
    }
}
