package more.ssaqms.xiamen.model.operation.integral;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.data.DataReceive;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/7/15 18:52
 */
@ApiModel(value = "重积分接收参数模型")
@Data
@ToString
@Accessors(chain = true)
public class IntegralReceive {

    @NotNull
    @ApiModelProperty(value = "查询开始时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    @NotNull(message = "开始时间不能为空")
    private Date beginTime;

    @NotNull
    @ApiModelProperty(value = "查询结束时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    @NotNull(message = "结束时间不能为空")
    private Date endTime;

    @ApiModelProperty(value = "站点ids", required = true)
    @Size(min = 1 ,message = "站点ids最少要有1个")
    private List<String> siteIds;

    @ApiModelProperty(value = "因子ids", required = true)
    @Size(min = 1 ,message = "因子ids最少要有1个")
    private List<String> factorIds;

    public IntegralReceive() {
    }

    public IntegralReceive(DataReceive dataReceive) {
        this.beginTime = dataReceive.getBeginTime();
        this.endTime = dataReceive.getEndTime();
        this.siteIds = dataReceive.getSiteIds();
        this.factorIds = dataReceive.getFactorIds();
    }
}
