package more.ssaqms.xiamen.model.reachGood;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import more.ssaqms.xiamen.model.report.TimeSeriesChart;

import java.util.List;
import java.util.Map;

/**
 * @author zuo
 * @date 2024/7/29 11:48
 */
@Data
public class ReachGoodAqiData {

    /**
     * 厦门市当日AQI（日AQI）值
     */
    @ApiModelProperty(value = "厦门市当日AQI（日AQI）值", required = true)
    private Float dayAqiValue;

    /**
     * 污染等级（中文）
     */
    @ApiModelProperty(value = "污染等级（中文）", required = true)
    private String gradeStr;

    /**
     * 厦门市当日AQI（日AQI）对应的首要污染物，无首要污染物改字段为空或者null
     */
    @ApiModelProperty(value = "厦门市当日AQI（日AQI）对应的首要污染物，无首要污染物改字段为空或者null", required = true)
    private String dayPrimaryPollutant;

    /**
     * 厦门市当日AQI（日AQI）对应的常规污染物浓度情况
     */
    @ApiModelProperty(value = "厦门市当日AQI（日AQI）对应的常规污染物浓度情况", required = true)
    private List<ReachGoodPollutantData> dayPollutantDataList;

    /**
     * 图1 厦门市常规监测因子时间序列图chart数据
     */
    @ApiModelProperty(value = "图1 厦门市常规监测因子时间序列图chart数据", required = true)
    private Map<String, Object> hourPollutantChartMap;

    /**
     * 国控点AQI各物质浓度小时的站点均值的最大值，时间，物质浓度
     * key 因子id
     * value 最大值因子对应的时间，值，浓度
     */
    @ApiModelProperty(value = "国控点AQI各物质浓度小时的站点均值的最大值，时间，物质浓度（key 因子id，value 最大值因子对应的时间，值，浓度）", required = true)
    private Map<String, ReachGoodPollutantData> maxMap;

    /**
     * 国控点AQI各物质浓度小时的站点均值的最小值，时间，物质浓度
     * key 因子id
     * value 最小值因子对应的时间，值，浓度
     */
    @ApiModelProperty(value = "国控点AQI各物质浓度小时的站点均值的最小值，时间，物质浓度（key 因子id，value 最小值因子对应的时间，值，浓度）", required = true)
    private Map<String, ReachGoodPollutantData> minMap;

    /**
     * 站点对比
     * key factorId
     * value Map<String, String>
     * ("factorName",factorName)
     * ("maxSiteName",maxSiteName)
     * ("minSiteName",minSiteName)
     */
    @ApiModelProperty(value = "站点对比（key factorId ， value Map<String, String>{ {'factorName',factorName} ，{'maxSiteName',maxSiteName} ，{'minSiteName',minSiteName}}", required = true)
    private Map<String,Map<String, String>> siteComparison;

    /**
     * chart数据（O3、pm2.5、pm10）小时浓度序列图
     */
    @ApiModelProperty(value = "chart数据（O3、pm2.5、pm10）小时浓度序列图", required = true)
    private List<TimeSeriesChart> timeSeriesCharts;

}
