package more.ssaqms.xiamen.model.report;

import more.ssaqms.xiamen.model.data.SiteAuditDataExceptionCount;
import more.ssaqms.xiamen.model.site.ReportSiteDevice;
import more.ssaqms.xiamen.model.site.Site;

import java.util.*;

/**
 * @author purpose
 * @version 1.0
 * @date 2023/5/5 1:00 下午
 */
public class ReportFormatAuditData {

  public static final Object[] HEADER_COLUMNS = new Object[]{"设备名称", "无效小时数", "无效原因"};

  private String siteName;

  private List<SiteAuditDataExceptionCount> auditExceptionCountList = new ArrayList<>();

  private String mainDeleteReason = null;

  private List<ReportDeleteReason> otherDeleteReasons = new ArrayList<>();

  private List<ReportNoDataReason> noDataReasonList = new ArrayList<>();

  private Object[][] tableData = null;

  public ReportFormatAuditData(Site site) {
    this.siteName = site.getName();
  }

  public String getMainDeleteReason() {
    return this.mainDeleteReason;
  }

  public List<ReportDeleteReason> getOtherDeleteReasons() {
    return this.otherDeleteReasons;
  }

  public List<ReportNoDataReason> getNoDataReasonList() {
    return this.noDataReasonList;
  }

  public Object[][] getTableData() {
    return this.tableData;
  }

  public ReportFormatAuditData add(SiteAuditDataExceptionCount auditExceptionCount, Map<String, ReportSiteDevice> siteDeviceMap) {
    ReportSiteDevice siteDevice = siteDeviceMap.get(auditExceptionCount.getSiteDeviceId());

    if (siteDevice  == null) {
      return this;
    }

    auditExceptionCount.setDeviceName(siteDevice.getDeviceName());

    this.auditExceptionCountList.add(auditExceptionCount);

    return this;
  }

  public String getSiteName() {
    return this.siteName;
  }

  public ReportFormatAuditData setData() {
    this.setTableData();
    return this;
  }

  private void setTableData() {
    List<SiteAuditDataExceptionCount> auditExceptionCountList = this.auditExceptionCountList;
    int size = auditExceptionCountList.size();

    if (size == 0) {
      return;
    }

    Object[][] tableData = this.tableData = new Object[size + 2][HEADER_COLUMNS.length];

    tableData[0] = HEADER_COLUMNS;

    List<ReportNoDataReason> noDataReasonList = this.noDataReasonList;
    Map<String, Integer> deleteReasonCountMap = new HashMap<>();
    Map<String, List<String>> deleteReasonDeviceMap = new HashMap<>();
    Map<String, ReportNoDataReason> noDataReasonMap = new HashMap<>();
    int sum = 0;

    for (int i = 0; i < size;) {
      SiteAuditDataExceptionCount auditData = auditExceptionCountList.get(i);
      String deviceName = auditData.getDeviceName();
      Integer count = auditData.getCount();
      String auditComment = auditData.getAuditComment();

      Object[] columns = tableData[++i];
      columns[0] = deviceName;
      columns[1] = count;
      columns[2] = auditComment;

      sum += count;

      if (auditData.getType() == 1) {
        String siteDeviceId = auditData.getSiteDeviceId();
        ReportNoDataReason noDataReason = noDataReasonMap.get(siteDeviceId);

        if (noDataReason == null) {
          noDataReasonMap.put(siteDeviceId, noDataReason = new ReportNoDataReason(deviceName));
          noDataReasonList.add(noDataReason);
        }

        noDataReason.add(auditComment, count);
      } else {
        if (deleteReasonCountMap.containsKey(auditComment)) {
          deleteReasonCountMap.put(auditComment, count + deleteReasonCountMap.get(auditComment));
          deleteReasonDeviceMap.get(auditComment).add(deviceName);
        } else {
          deleteReasonCountMap.put(auditComment, count);
          deleteReasonDeviceMap.put(auditComment, new ArrayList<>());
        }
      }
    }

    Object[] lastRow = tableData[tableData.length - 1];
    lastRow[0] = "合计";
    lastRow[1] = sum;
    lastRow[2] = "/";

    class NameValue {
      public String name;
      public int value;

      NameValue(String name, int value) {
        this.name = name;
        this.value = value;
      }

      int getValue() {
        return this.value;
      }
    }

    Set<Map.Entry<String, Integer>> entries = deleteReasonCountMap.entrySet();
    List<NameValue> list = new ArrayList<>();
    int index = 0;

    for (Map.Entry<String, Integer> entry : entries) {
      list.add(new NameValue(entry.getKey(), entry.getValue()));
    }

    if (list.size() > 0) {
      list.sort(Comparator.comparing(NameValue::getValue));
      NameValue mainReason = list.remove(list.size() - 1);

      this.mainDeleteReason = mainReason.name;

      List<ReportDeleteReason> otherDeleteReasons = this.otherDeleteReasons;

      for (NameValue nameValue : list) {
        String reason = nameValue.name;
        otherDeleteReasons.add(new ReportDeleteReason(reason, deleteReasonDeviceMap.get(reason)));
      }
    }
  }
}
