package more.ssaqms.xiamen.model.report;

import com.alibaba.fastjson.JSONArray;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.site.Site;
import more.ssaqms.xiamen.model.statistics.AqiData;
import more.ssaqms.xiamen.model.statistics.PollutantData;
import more.ssaqms.xiamen.util.AqiStatUtil;
import more.ssaqms.xiamen.util.AqiStringPools;
import more.ssaqms.xiamen.util.CalculationUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author purpose
 * @version 1.0
 * @date 2023/5/6 10:45 上午
 */
@Data
@ToString
@Accessors(chain = true)
public class ReportMonthAQILinGangData {
  public static final String[][] AQI_FACTOR_LIST = new String[][] {
      new String[]{"1001", "SO2", "SO₂", AqiStringPools.SO2},
      new String[]{"1003", "NO2", "NO₂", AqiStringPools.NO2},
      new String[]{"1006", "O3", "O₃_8H", AqiStringPools.H8O3},
      new String[]{"1005", "CO", "CO", AqiStringPools.CO},
      new String[]{"1007", "PM10", "PM₁₀", AqiStringPools.PM10},
      new String[]{"1008", "PM2POINT5", "PM₂.₅", AqiStringPools.PM25}
  };

  public final static Map<String, Integer> getAQIFactorIdIndexMap() {
    Map<String, Integer> aqiFactorIdIndexMap = new HashMap<>();
    int length = AQI_FACTOR_LIST.length;

    for (int i = 0; i < length; i++) {
      aqiFactorIdIndexMap.put(AQI_FACTOR_LIST[i][0], i);
    }

    return aqiFactorIdIndexMap;
  }

  // 当前参与统计AQI的站房列表
  private List<Site> sites = null;

  // 月份数据, 如果为null, 则代表没有数据
  private ReportAQILinGangData monthData = null;

  // 年数据, 如果为null, 则代表没有数据
  private ReportAQILinGangData yearData = null;

  // 年度环境空气质量统计表
  private Object[][] aqiTableData = null;

  private List<ReportSiteAQIInfo> siteAQIInfoList = null;

  public int[] getYearMonth(TimeRange timeRange) {
    Calendar calendar = Calendar.getInstance();
    calendar.setTime(timeRange.getBeginTime());
    return new int[]{calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH) + 1};
  }

  public ReportMonthAQILinGangData(
      TimeRange timeRange,
      BasicInformation information,
      List<String> siteIds,
      List<AqiData> monthAqiDataList,
      List<AqiData> yearAqiDataList,
      List<AqiData> monthSiteAqiDataList,
      List<AqiData> lastYearAqiDataList,
      JSONArray cityAQIList
  ) {
    this.setSites(siteIds, information);

    if (monthAqiDataList.size() > 0) {
      this.monthData = new ReportAQILinGangData(monthAqiDataList);
    }

    if (yearAqiDataList.size() > 0) {
      this.yearData = new ReportAQILinGangData(yearAqiDataList);
    }

    this.setAqiTableData(
        monthAqiDataList,
        monthSiteAqiDataList,
        yearAqiDataList,
        lastYearAqiDataList,
        cityAQIList
    );
  }

  private void setSites(List<String> siteIds, BasicInformation information) {
    Map<String, Site> siteMap = information.getSiteMap();
    List<Site> sites = new ArrayList<>();

    for (String siteId : siteIds) {
      sites.add(siteMap.get(siteId));
    }

    this.sites = sites;
  }

  private void setAqiTableData(
      List<AqiData> monthAqiDataList,
      List<AqiData> monthSiteAqiDataList,
      List<AqiData> yearAqiDataList,
      List<AqiData> lastYearAqiDataList,
      JSONArray cityAQIList
  ) {
    int aqiFactorListLen = AQI_FACTOR_LIST.length;
    int totalColumnNum = 1 + aqiFactorListLen + 1;
    List<Site> sites = this.sites;
    int siteSize = sites == null ? 0 : sites.size();

    Object[][] aqiTableData = this.aqiTableData = new Object[1 + siteSize + 1 + 4][totalColumnNum];

    Map<String, Integer> factorIdIndexMap = new HashMap<>();
    Map<String, Integer> factorNameIndexMap = new HashMap<>();

    int rowIndex = 0;
    Object[] aqiFactorNames = aqiTableData[rowIndex++];

    for (int i = 0; i < aqiFactorListLen; i++) {
      int index = i + 1;
      String[] factorInfo = AQI_FACTOR_LIST[i];
      factorIdIndexMap.put(factorInfo[0], index);
      factorNameIndexMap.put(factorInfo[1], index);
      aqiFactorNames[index] = factorInfo[2];
    }

    int goodDayColumnIndex = totalColumnNum - 1;
    rowIndex = this.setSiteStatisticData(
        aqiTableData,
        rowIndex,
        monthSiteAqiDataList,
        factorIdIndexMap,
        aqiFactorListLen,
        goodDayColumnIndex
    );

    Object[] row = aqiTableData[rowIndex++];
    row[0] = "站点综合";
    this.setStatisticData(row, monthAqiDataList, factorIdIndexMap);

    row = aqiTableData[rowIndex++];
    row[0] = "上海市";
    this.setCityAQIData(row, cityAQIList, factorNameIndexMap);

    Object[] currentYearRow = aqiTableData[rowIndex++];
    currentYearRow[0] = "站点综合";
    this.setStatisticData(currentYearRow, yearAqiDataList, factorIdIndexMap);

    Object[] lastYearRow = aqiTableData[rowIndex++];
    lastYearRow[0] = "站点综合";
    this.setStatisticData(lastYearRow, lastYearAqiDataList, factorIdIndexMap);

    Object[] ratioRow = aqiTableData[rowIndex];

    for (int i = 1; i < totalColumnNum; i++) {
      if (i == totalColumnNum - 1) {
        Integer value1 = (Integer) currentYearRow[i];

        if (value1 == null) {
          continue;
        }

        Integer value2 = (Integer) lastYearRow[i];

        if (value2 == null || value2 == 0) {
          continue;
        }

        ratioRow[i] = CalculationUtils.digit2((value1.floatValue() - value2) / value2 * 100, 2);
      } else {
        Float value1 = (Float) currentYearRow[i];

        if (value1 == null) {
          continue;
        }

        Float value2 = (Float) lastYearRow[i];

        if (value2 == null || value2 == 0) {
          continue;
        }

        ratioRow[i] = CalculationUtils.digit2((value1 - value2) / value2 * 100, 2);
      }
    }
  }

  private int setSiteStatisticData(
      Object[][] aqiTableData,
      int rowIndex,
      List<AqiData> aqiDataList,
      Map<String, Integer> factorIdIndexMap,
      int aqiFactorListLen,
      int goodDayColumnIndex
  ) {
    List<Site> sites = this.sites;
    Map<String, Float[][]> siteStatisticDataMap = new HashMap<>();
    Map<String, Integer> siteIndexMap = new HashMap<>();
    Map<String, ReportSiteAQIInfo> siteAQIInfoMap = new HashMap<>();
    Map<String, Integer> siteStatisticDayMap = new HashMap<>();

    for (Site site: sites) {
      Float[][] statisticDatas = new Float[aqiFactorListLen][2];

      for (int i = 0; i < aqiFactorListLen; i++) {
        Float[] statisticData = statisticDatas[i];
        statisticData[0] = statisticData[1] = 0f;
      }

      String siteId = site.getId();
      String siteName = site.getName();
      siteStatisticDataMap.put(siteId, statisticDatas);
      siteStatisticDayMap.put(siteId, null);
      aqiTableData[rowIndex][0] = siteName;
      siteIndexMap.put(siteId, rowIndex++);
      siteAQIInfoMap.put(siteId, new ReportSiteAQIInfo(siteName));
    }

    for (AqiData aqiData : aqiDataList) {
      String siteId = aqiData.getSiteId();
      Float[][] statisticData = siteStatisticDataMap.get(siteId);

      if (statisticData == null) {
        continue;
      }

      List<PollutantData> pollutantDatas = aqiData.getPollutantData();

      if (pollutantDatas == null) {
        continue;
      }

      ReportSiteAQIInfo reportSiteAQIInfo = siteAQIInfoMap.get(siteId);
      reportSiteAQIInfo.setAQILevel(aqiData.getGrade());

      for (PollutantData pollutantData : pollutantDatas) {
        Float value = pollutantData.getValue();

        if (value == null) {
          continue;
        }

        String factorId = pollutantData.getFactorId();

        Integer factorIndex = factorIdIndexMap.get(factorId);

        if (factorIndex == null) {
          continue;
        }

        if (factorId.equals("1006")) {
          if (!pollutantData.getRootFactorId().equals("H8O3")) {
            continue;
          }

          reportSiteAQIInfo.addO3Value(value);
        }

        factorIndex -= 1;
        statisticData[factorIndex][0] += value;
        statisticData[factorIndex][1] += 1;
      }

      if (aqiData.getGrade() < 3) {
        Object[] row = aqiTableData[siteIndexMap.get(siteId)];
        Integer sum = (Integer) row[goodDayColumnIndex];

        if (sum == null) {
          sum = 0;
        }

        row[goodDayColumnIndex] = sum + 1;
      }

      Integer total = siteStatisticDayMap.get(siteId);

      if (total == null) {
        total = 0;
      }

      siteStatisticDayMap.put(siteId, total + 1);
    }

    List<ReportSiteAQIInfo> siteAQIInfoList = new ArrayList<>();

    for (Site site: sites) {
      String siteId = site.getId();
      Object[] row = aqiTableData[siteIndexMap.get(siteId)];
      Float[][] statisticDatas = siteStatisticDataMap.get(siteId);
      ReportSiteAQIInfo reportSiteAQIInfo = siteAQIInfoMap.get(siteId)
          .setGoodDays((Integer) row[goodDayColumnIndex])
          .setStatisticDays(siteStatisticDayMap.get(siteId));

      for (int i = 0; i < aqiFactorListLen; i++) {
        Float[] statisticData = statisticDatas[i];
        Float count = statisticData[1];

        if (count == 0) {
          continue;
        }

        Float value = CalculationUtils.digit2(statisticData[0] / count, 2);
        row[i + 1] = value;

        if (AQI_FACTOR_LIST[i][0] == "1008") {
          reportSiteAQIInfo.setPM25Value(value);
        }
      }

      siteAQIInfoList.add(reportSiteAQIInfo);
    }

    this.siteAQIInfoList = siteAQIInfoList;

    return rowIndex;
  }

  private void setStatisticData(
      Object[] row,
      List<AqiData> aqiDataList,
      Map<String, Integer> factorIdIndexMap
  ) {
    int totalColumnNum = row.length;
    int goodDayColumnIndex = totalColumnNum - 1;
    int aqiFactorLen = AQI_FACTOR_LIST.length;
    Float[][] statistics = new Float[aqiFactorLen][2];

    for (int i = 0; i < aqiFactorLen; i++) {
      Float[] statistic = statistics[i];
      statistic[0] = statistic[1] = 0f;
    }

    for (AqiData aqiData : aqiDataList) {
      List<PollutantData> pollutantDatas = aqiData.getPollutantData();

      if (pollutantDatas == null) {
        continue;
      }

      for (PollutantData pollutantData : pollutantDatas) {
        String factorId = pollutantData.getFactorId();

        if (factorId.equals("1006") && !pollutantData.getRootFactorId().equals("H8O3")) {
          continue;
        }

        Integer factorIndex = factorIdIndexMap.get(factorId);

        if (factorIndex == null) {
          continue;
        }

        Float value = pollutantData.getValue();

        if (value == null) {
          continue;
        }

        factorIndex -= 1;
        statistics[factorIndex][0] += value;
        statistics[factorIndex][1] += 1;
      }

      if (aqiData.getGrade() < 3) {
        Integer sum = (Integer) row[goodDayColumnIndex];

        if (sum == null) {
          sum = 0;
        }

        row[goodDayColumnIndex] = sum + 1;
      }
    }

    for (int i = 0; i < aqiFactorLen; i++) {
      Float[] statistic = statistics[i];
      Float count = statistic[1];

      if (count == 0) {
        continue;
      }

      row[i + 1] = CalculationUtils.digit2(statistic[0] / count, 2);
    }
  }

  private void setCityAQIData(
      Object[] row,
      JSONArray cityAQIList,
      Map<String, Integer> factorNameIndexMap
  ) {
    int totalColumnNum = row.length;
    int goodDayColumnIndex = totalColumnNum - 1;
    int aqiFactorLen = AQI_FACTOR_LIST.length;
    Float[][] statistics = new Float[aqiFactorLen][2];

    for (int i = 0; i < aqiFactorLen; i++) {
      Float[] statistic = statistics[i];
      statistic[0] = statistic[1] = 0f;
    }

    int goodDays = 0;

    for (Object cityAQI : cityAQIList) {
      Float[] aqi = new Float[]{0f};

      ((Map<String, Object>) cityAQI).forEach((key, value) -> {
        Integer factorIndex = factorNameIndexMap.get(key);

        if (factorIndex == null || value == null) {
          return;
        }

        factorIndex -= 1;
        Class<?> clazz = value.getClass();
        Float iaqi = 0f;
        String rootFactorId = AQI_FACTOR_LIST[factorIndex][3];

        if (Integer.class.equals(clazz)) {
          Integer castValue = (Integer) value;
          statistics[factorIndex][0] += castValue;
          iaqi = AqiStatUtil.calcIAQI(castValue.floatValue(), rootFactorId, 24);
        } else if (Float.class.equals(clazz)) {
          Float castValue = (Float) value;
          statistics[factorIndex][0] += castValue;
          iaqi = AqiStatUtil.calcIAQI(castValue, rootFactorId, 24);
        } else if (BigDecimal.class.equals(clazz)) {
          Float castValue = ((BigDecimal) value).floatValue();
          statistics[factorIndex][0] += castValue;
          iaqi = AqiStatUtil.calcIAQI(castValue, rootFactorId, 24);
        }

        statistics[factorIndex][1] += 1;

        if (iaqi > aqi[0]) {
          aqi[0] = iaqi;
        }
      });

      Integer level = AqiStatUtil.GetAQIInfo(aqi[0]);

      if (level < 3) {
        goodDays++;
      }
    }

    for (int i = 0; i < aqiFactorLen; i++) {
      Float[] statistic = statistics[i];
      Float count = statistic[1];

      if (count == 0) {
        continue;
      }

      row[i + 1] = CalculationUtils.digit2(statistic[0] / count, 2);
    }

    row[goodDayColumnIndex] = goodDays;
  }
}
