package more.ssaqms.xiamen.model.report;

/**
 * @author purpose
 * @version 1.0
 * @date 2023/6/18 11:37 AM
 */

import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.commons.utils.tool.DateUtils;
import more.ssaqms.xiamen.model.data.HistoricalData;
import more.ssaqms.xiamen.model.factor.Factor;
import more.ssaqms.xiamen.model.factor.FactorGroup;
import more.ssaqms.xiamen.model.site.ReportSiteDevice;
import more.ssaqms.xiamen.model.site.Site;
import more.ssaqms.xiamen.model.statistics.AvgData;
import more.ssaqms.xiamen.util.CalculationUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Data
@ToString
@Accessors(chain = true)
public class ReportMonthVOCsLinGangDataDetail {

  // 站房主键
  private String siteId;

  // 站房名称
  private String siteName;

  // 站房类型
  private Long siteType;

  // 站房VOCs仪器
  private final ReportSiteDevice siteDevice;

  // vocs日均值范围, 如果数据为Null, 代表没数据
  private Float[] vocsDailyRange;

  // 非甲烷日均值范围, 如果数据为Null, 代表没数据
  private Float[] nmhcDailyRange;

  // VOCs分类占比描述和分类下的top污染因子和第一污染因子的最高检出浓度
  private List<ReportVOCsFactorGroupValueInfo> factorGroupValueSortDescriptions;

  // VOCs分类的饼图数据列表
  private List<PieData> vocsGroupPieDatas;

  // VOCs分类趋势图用到的dimensions
  private List<Factor> vocsGroupDimensions = new ArrayList<>();

  // 非甲烷因子
  private Factor nmhcFactor;

  // 不会为空
  private Object[][] topTableData;

  // 月均浓度最高的VOCs因子信息
  private Factor vocsFactorTOp1Dimension;
  // 月均浓度最高的VOCs因子的小时数据列表，不会为空
  private List<HistoricalData> vocsFactorTop1DataList = new ArrayList();

  // 月均浓度第二高的VOCs因子信息
  private Factor vocsFactorTOp2Dimension;
  // 月均浓度第二高的VOCs因子的小时数据列表，不会为空
  private List<HistoricalData> vocsFactorTop2DataList = new ArrayList();

  // 内部用: 因子分组关系型数据
  private Map<String, FactorGroup> factorGroupMap;

  private Map<String, Float> ofpFactorMap;

  private Map<String, Float> soapFactorMap;

  public ReportMonthVOCsLinGangDataDetail(
    Site site,
    ReportSiteDevice siteDevice,
    Factor nmhcFactor,
    Map<String, FactorGroup> factorGroupMap,
    Map<String, Float> ofpFactorMap,
    Map<String, Float> soapFactorMap
  ) {
    this.siteId = site.getId();
    this.siteName = site.getName();
    this.siteType = site.getType();
    this.siteDevice = siteDevice;
    this.nmhcFactor = nmhcFactor;
    this.factorGroupMap = factorGroupMap;
    this.ofpFactorMap = ofpFactorMap;
    this.soapFactorMap = soapFactorMap;
    this.setVOCsFactorGroupData(siteDevice, factorGroupMap);
  }


  // VOCs分组日均值
  private List<AvgData> vocsGroupDailyAvgs = new ArrayList();
  // 内部用
  private Map<String, Float> totalDailyMap = new HashMap();
  private Map<String, Float[]> vocsGroupDailyValuesMap = new HashMap<>();

  public ReportMonthVOCsLinGangDataDetail addVOCsGroupDailyData(AvgData avgData) {
    Float value = avgData.getValue();

    if (value != null) {
      String dataTimeStr = avgData.getDataTimeStr();

      try {
        avgData.setDataTime(DateUtils.parseDate(dataTimeStr, "yyyy/MM/dd"));
      } catch (ParseException e) {
        e.printStackTrace();
      }
      this.vocsGroupDailyAvgs.add(avgData);

      Map<String, Float> totalDailyMap = this.totalDailyMap;
      Float total = totalDailyMap.get(dataTimeStr);
      if (total == null) {
        total = 0f;
      }
      totalDailyMap.put(dataTimeStr, total + value);

      String factorId = avgData.getFactorId();
      Map<String, Float[]> vocsGroupDailyValuesMap = this.vocsGroupDailyValuesMap;
      Float[] floats = vocsGroupDailyValuesMap.get(factorId);
      if (floats == null) {
        vocsGroupDailyValuesMap.put(factorId, floats = new Float[]{0f, 0f});
      }
      floats[0] += value;
      floats[1]++;
    }

    return this;
  }


  // 内部用
  private Map<String, Float[]> vocsFactorMonthlyAvgMap = new HashMap();

  public ReportMonthVOCsLinGangDataDetail addVOCsFactorDailyData(AvgData avgData) {
    Float value = avgData.getValue();
    if (value == null) {
      return this;
    }

    Map<String, Float[]> vocsFactorMonthlyAvgMap = this.vocsFactorMonthlyAvgMap;
    String factorId = avgData.getFactorId();
    Float[] floats = vocsFactorMonthlyAvgMap.get(factorId);
    if (floats == null) {
      vocsFactorMonthlyAvgMap.put(factorId, floats = new Float[]{0f, 0f});
    }
    floats[0] += value;
    floats[1]++;

    return this;
  }


  // 非甲烷日均值
  private List<AvgData> nmhcFactorDailyAvgs = new ArrayList<>();

  public ReportMonthVOCsLinGangDataDetail addNMHCFactorDailyData(AvgData avgData) {
    Factor nmhcFactor = this.nmhcFactor;
    avgData.setValue(CalculationUtils.UnitConverter(avgData.getValue(), nmhcFactor.getStdUnit(), nmhcFactor.getDisplayUnit(), nmhcFactor.getMolecular()));
    try {
      avgData.setDataTime(DateUtils.parseDate(avgData.getDataTimeStr(), "yyyy/MM/dd"));
    } catch (ParseException e) {
      e.printStackTrace();
    }
    this.nmhcFactorDailyAvgs.add(avgData);
    return this;
  }

  // 内部用
  private Map<String, List<HistoricalData>> factorDataMap = new HashMap();

  public ReportMonthVOCsLinGangDataDetail addVOCsFactorHourlyData(HistoricalData data) {
    Float value = data.getValue();
    if (value == null) {
      return this;
    }

    ReportSiteDevice siteDevice = this.siteDevice;
    Map<String, Factor> factorMap = siteDevice.getFactorMap();
    String factorId = data.getFactorId();
    Factor factor = factorMap.get(factorId);
    if (factor == null) {
      return this;
    }

    data.setValue(CalculationUtils.UnitConverter(value, factor.getStdUnit(), factor.getDisplayUnit(), factor.getMolecular()));

    Map<String, List<HistoricalData>> factorDataMap = this.factorDataMap;
    List<HistoricalData> historicalData = factorDataMap.get(factorId);
    if (historicalData == null) {
      factorDataMap.put(factorId, historicalData = new ArrayList());
    }
    historicalData.add(data);

    return this;
  }

  public ReportMonthVOCsLinGangDataDetail statisticData() {
    this.setVOCsDailyRange()
      .setNMHCDailyRange()
      .setVOCsGroupData();
    return this;
  }

  private ReportMonthVOCsLinGangDataDetail setVOCsDailyRange() {
    Float[] vocsDailyRange = new Float[2];

    totalDailyMap.forEach((dataTimeStr, total) -> {
      this.setRange(vocsDailyRange, total);
    });

    this.vocsDailyRange = vocsDailyRange;

    return this;
  }

  private ReportMonthVOCsLinGangDataDetail setNMHCDailyRange() {
    List<AvgData> nmhcFactorDailyAvgs = this.nmhcFactorDailyAvgs;

    if (nmhcFactorDailyAvgs == null || nmhcFactorDailyAvgs.size() == 0) {
      return this;
    }

    nmhcFactorDailyAvgs.sort((o1, o2) -> o1.getDataTimeStr().compareTo(o2.getDataTimeStr()));

    Float[] nmhcDailyRange = new Float[2];
    this.nmhcFactorDailyAvgs.forEach(avgData -> {
      this.setRange(nmhcDailyRange, avgData.getValue());
    });

    this.nmhcDailyRange = nmhcDailyRange;

    return this;
  }

  private ReportMonthVOCsLinGangDataDetail setVOCsGroupData() {
    this.createVOCsGroupPieChartData()
      .createVOCsFactorMonthlyData()
      .createVOCsGroupDescData()
      .createVOCsGroupDimensions()
      .setTopTableData();
    return this;
  }

  private ReportMonthVOCsLinGangDataDetail createVOCsGroupPieChartData() {
    Map<String, FactorGroup> factorGroupMap = this.factorGroupMap;

    List<PieData> pieCharts = new ArrayList();
    this.vocsGroupDailyValuesMap.forEach((groupId, statisticData) -> {
      FactorGroup factorGroup = factorGroupMap.get(groupId);
      if (factorGroup == null) {
        return;
      }

      float avg = statisticData[0] / statisticData[1];
      pieCharts.add(new PieData(groupId, factorGroup.getName(), avg));

      ReportVOCsFactorGroupValueInfo vocsFactorGroupValueInfo = this.getVOCsFactorGroupValueInfoByGroupId(groupId, factorGroup);
      vocsFactorGroupValueInfo.sumAvg(avg);
    });

    pieCharts.sort((o1, o2) -> o2.getValue() - o1.getValue() > 0 ? 1 : -1);
    this.vocsGroupPieDatas = pieCharts;

    return this;
  }


  private ReportMonthVOCsLinGangDataDetail createVOCsFactorMonthlyData() {
    Map<String, Factor> factorMap = this.siteDevice.getFactorMap();
    Map<String, FactorGroup> factorGroupMap = this.factorGroupMap;
    this.vocsFactorMonthlyAvgMap.forEach((factorId, floats) -> {
      Factor factor = factorMap.get(factorId);
      if (factor == null) {
        return;
      }

      String groupId = factor.getGroupId();
      FactorGroup factorGroup = factorGroupMap.get(groupId);
      if (factorGroup == null) {
        return;
      }

      Float value = CalculationUtils.UnitConverter(floats[0], factor.getStdUnit(), factor.getDisplayUnit(), factor.getMolecular());
      Float avg = value / floats[1];

      ReportVOCsFactorGroupValueInfo vocsFactorGroupValueInfo = getVOCsFactorGroupValueInfoByGroupId(groupId, factorGroup);
      vocsFactorGroupValueInfo.addData(new ReportNameValue(factorId, factor.getDisplayName(), avg));
    });

    return this;
  }

  private ReportMonthVOCsLinGangDataDetail createVOCsGroupDescData() {
    Map<String, List<HistoricalData>> factorDataMap = this.factorDataMap;
    if (factorDataMap == null || factorDataMap.size() == 0) {
      return this;
    }

    Map<String, ReportVOCsFactorGroupValueInfo> factorGroupValueInfoMap = this.factorGroupValueInfoMap;
    if (factorGroupValueInfoMap == null || factorGroupValueInfoMap.size() == 0) {
      return this;
    }

    factorGroupValueInfoMap.forEach((groupId, data) -> {
      data.sortListByDesc();
      List<ReportNameValue> avgDataList = data.getAvgDataList();
      int maxLen = Math.min(2, avgDataList.size());
      for (int i = 0; i < maxLen; i++) {
        ReportNameValue reportNameValue = avgDataList.get(i);
        String factorId = reportNameValue.getId();
        List<HistoricalData> historicalData = factorDataMap.get(factorId);
        historicalData.sort((o1, o2) -> {
          float diff = o2.getValue() - o1.getValue();
          return diff > 0 ? 1 : diff < 0 ? -1 : 0;
        });
        data.addHourMaxValues(
          new ReportNameValue(
            factorId,
            reportNameValue.getName(),
            CalculationUtils.digit2(historicalData.get(0).getValue(), 2)
          )
        );
      }
    });

    List<ReportVOCsFactorGroupValueInfo> factorGroupValueSortDescriptions = new ArrayList(factorGroupValueInfoMap.values());
    factorGroupValueSortDescriptions.sort((o1, o2) -> (o2.getAvgSum() - o1.getAvgSum()) > 0 ? 1 : -1);
    this.factorGroupValueSortDescriptions = factorGroupValueSortDescriptions;
    return this;
  }

  private ReportMonthVOCsLinGangDataDetail createVOCsGroupDimensions() {
    Map<String, FactorGroup> factorGroupMap = this.factorGroupMap;
    Map<String, Factor> siteDeviceFactorMap = this.siteDevice.getFactorMap();
    Map<String, Factor> vocsGroupDimensionMap = new HashMap();
    siteDeviceFactorMap.forEach((factorId, factor) -> {
      String groupId = factor.getGroupId();
      if (vocsGroupDimensionMap.containsKey(groupId)) {
        return;
      }

      FactorGroup factorGroup = factorGroupMap.get(groupId);
      if (factorGroup == null) {
        return;
      }
      Factor newFactor = new Factor();
      newFactor.setId(groupId).setDisplayName(factorGroup.getName()).setStdUnit("μg/m3").setDisplayUnit("μg/m3");
      vocsGroupDimensionMap.put(groupId, newFactor);
    });
    List<Factor> vocsGroupDimensions = new ArrayList(vocsGroupDimensionMap.values());
    vocsGroupDimensions.sort((o1, o2) -> o1.getId().compareTo(o2.getId()));
    this.vocsGroupDimensions = vocsGroupDimensions;
    return this;
  }


  private ReportMonthVOCsLinGangDataDetail setTopTableData() {
    Map<String, List<HistoricalData>> factorDataMap = this.factorDataMap;
    SimpleDateFormat dateFormat = new SimpleDateFormat("MM-dd HH");

    List<ReportNameValue> vocsFactorMonthlyDataList = new ArrayList();

    List<ReportVOCsFactorGroupValueInfo> factorGroupValueSortDescriptions = this.factorGroupValueSortDescriptions;
    if (factorGroupValueSortDescriptions == null) {
      factorGroupValueSortDescriptions = new ArrayList();
    }

    factorGroupValueSortDescriptions.forEach(factorGroupValueInfo -> {
      vocsFactorMonthlyDataList.addAll(factorGroupValueInfo.getAvgDataList());
    });

    vocsFactorMonthlyDataList.sort((o1, o2) -> (o2.getValue() - o1.getValue()) > 0 ? 1 : -1);
    int rowNum = Math.min(5, vocsFactorMonthlyDataList.size());
    Object[][] topTableData = new Object[rowNum + 1][5];
    topTableData[0] = new Object[]{"序号", "污染物名称", "月平均浓度<br />（μg/m³）", "最高检出浓度<br />（μg/m³）", "最高浓度检出时间"};
    for (int i = 0; i < rowNum;) {
      ReportNameValue reportNameValue = vocsFactorMonthlyDataList.get(i++);
      String factorName = reportNameValue.getName();
      String factorId = reportNameValue.getId();
      List<HistoricalData> historicalData = factorDataMap.get(factorId);
      historicalData.sort((o1, o2) -> (o2.getValue() - o1.getValue()) > 0 ? 1 : -1);
      HistoricalData topFactorData = historicalData.get(0);
      Object[] topTableDatum = topTableData[i];
      topTableDatum[0] = i;
      topTableDatum[1] = factorName;
      topTableDatum[2] = CalculationUtils.digit2(reportNameValue.getValue(), 2);
      topTableDatum[3] = CalculationUtils.digit2(topFactorData.getValue(), 2);
      topTableDatum[4] = dateFormat.format(topFactorData.getDataTime());

      switch(i) {
        case 1:
          this.vocsFactorTOp1Dimension = new Factor();
          this.vocsFactorTOp1Dimension.setId(factorId).setDisplayName(factorName).setStdUnit("μg/m3").setDisplayUnit("μg/m3");
          this.vocsFactorTop1DataList = historicalData;
          break;
        case 2:
          this.vocsFactorTOp2Dimension = new Factor();
          this.vocsFactorTOp2Dimension.setId(factorId).setDisplayName(factorName).setStdUnit("μg/m3").setDisplayUnit("μg/m3");
          this.vocsFactorTop2DataList = historicalData;
          break;
      }
    }
    this.topTableData = topTableData;

    return this;
  }

  private ReportMonthVOCsLinGangDataDetail setRange(Float[] range, Float value) {
    value = CalculationUtils.digit2(value, 2);

    if (range[0] == null || range[1] == null) {
      range[0] = range[1] = value;
    } else if (value < range[0]) {
      range[0] = value;
    } else if (value > range[1]) {
      range[1] = value;
    }

    return this;
  }


  private static final Map<String, Boolean> GROUP_ID_MAP = new HashMap() {{
    put("m21000", true);
    put("m25000", true);
  }};


  // 内部用: VOCs分类最高占比和最高检出浓度描述
  private Map<String, ReportVOCsFactorGroupValueInfo> factorGroupValueInfoMap = new HashMap();

  private String getGroupId(String groupId) {
    if (GROUP_ID_MAP.get(groupId) != null) {
      return "m24000";
    }
    return groupId;
  }

  private ReportVOCsFactorGroupValueInfo getVOCsFactorGroupValueInfoByGroupId(String groupId, FactorGroup factorGroup) {
    groupId = this.getGroupId(groupId);

    ReportVOCsFactorGroupValueInfo reportVOCsFactorGroupValueInfo = factorGroupValueInfoMap.get(groupId);
    if (reportVOCsFactorGroupValueInfo == null) {
      factorGroupValueInfoMap.put(
        groupId,
        reportVOCsFactorGroupValueInfo = new ReportVOCsFactorGroupValueInfo(
          groupId,
          groupId.equals("m24000") ? "烯、炔烃" : factorGroup.getName()
        )
      );
    }

    return reportVOCsFactorGroupValueInfo;
  }



  private List<FactorGroup> vocsfactorGroups;

  private ReportMonthVOCsLinGangDataDetail setVOCsFactorGroupData(
    ReportSiteDevice siteDevice,
    Map<String, FactorGroup> factorGroupMap
  ) {
    List<FactorGroup> vocsfactorGroups = new ArrayList<>();
    siteDevice.getFactorMap().forEach((factorId, factor) -> {
      String groupId = factor.getGroupId();
      FactorGroup factorGroup = factorGroupMap.get(groupId);
      if (factorGroup == null) {
        return;
      }
      vocsfactorGroups.add(factorGroup);
    });
    this.vocsfactorGroups = vocsfactorGroups;
    return this;
  }
}
