package more.ssaqms.xiamen.model.report;

import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.data.HistoricalData;
import more.ssaqms.xiamen.model.site.ReportSiteDevice;
import more.ssaqms.xiamen.model.site.Site;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author purpose
 * @version 1.0
 * @date 2023/6/16 9:49 AM
 */
@Data
@ToString
@Accessors(chain = true)
public class ReportMonthVOCsLinGangDataV2 {

  // 所有参与统计的站点VOCs均值
  private List<LinGangMonthlySiteVOCsNMHCReport> vocsNMHCDataList;

  public ReportMonthVOCsLinGangDataV2(
      List<String> siteIds,
      List<ReportSiteDevice> siteDevices,
      BasicInformation information,
      List<HistoricalData> dataList,
      ChartData nmhcChartData
  ) {
    this.vocsNMHCDataList = this.getVOCsNMHCDataList(siteIds, siteDevices, information, dataList, nmhcChartData);
  }


  private List<LinGangMonthlySiteVOCsNMHCReport> getVOCsNMHCDataList(
      List<String> siteIds,
      List<ReportSiteDevice> siteDevices,
      BasicInformation information,
      List<HistoricalData> vocsDataList,
      ChartData nmhcChartData
  ) {
    Map<String, Site> siteMap = information.getSiteMap();
    Map<String, Integer> siteIndexMap = new HashMap<>();
    siteMap.forEach((siteId, site) -> {
      siteIndexMap.put(siteId, siteIndexMap.size());
    });

    List<LinGangMonthlySiteVOCsNMHCReport> vocsNMHCDataList = new ArrayList();
    Map<String, LinGangMonthlySiteVOCsNMHCReport> vocsNMHCDataMap = new HashMap<>();
    for (String siteId : siteIds) {
      LinGangMonthlySiteVOCsNMHCReport linGangMonthlySiteVOCsNMHCReport = new LinGangMonthlySiteVOCsNMHCReport(information, siteMap.get(siteId));
      vocsNMHCDataList.add(linGangMonthlySiteVOCsNMHCReport);
      vocsNMHCDataMap.put(siteId, linGangMonthlySiteVOCsNMHCReport);
    }
    vocsNMHCDataList.sort((o1, o2) -> siteIndexMap.get(o1.getSiteId()) - siteIndexMap.get(o2.getSiteId()));

    vocsDataList.forEach(data -> {
      LinGangMonthlySiteVOCsNMHCReport linGangMonthlySiteVOCsNMHCReport = vocsNMHCDataMap.get(data.getSiteId());
      if (linGangMonthlySiteVOCsNMHCReport == null) {
        return;
      }
      linGangMonthlySiteVOCsNMHCReport.addVOCsData(data);
    });

    nmhcChartData.getDataList().forEach(data -> {
      LinGangMonthlySiteVOCsNMHCReport linGangMonthlySiteVOCsNMHCReport = vocsNMHCDataMap.get(data.getSiteId());
      if (linGangMonthlySiteVOCsNMHCReport == null) {
        return;
      }
      linGangMonthlySiteVOCsNMHCReport.addNMHCData(data);
    });

    vocsNMHCDataList.forEach(vocsNMHCData -> vocsNMHCData.statisticData());

    return  vocsNMHCDataList;
  }
}
