package more.ssaqms.xiamen.model.report;

import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.data.HistoricalData;
import more.ssaqms.xiamen.model.factor.Factor;
import more.ssaqms.xiamen.model.factor.FactorGroup;
import more.ssaqms.xiamen.model.site.ReportSiteDevice;
import more.ssaqms.xiamen.model.site.Site;
import more.ssaqms.xiamen.model.statistics.AvgData;

import java.util.*;

/**
 * @author purpose
 * @version 1.0
 * @date 2023/6/18 10:31 AM
 */
@Data
@ToString
@Accessors(chain = true)
public class ReportMonthVOCsLinGangDataWrapper {

  // VOCs数据详情列表
  List<ReportMonthVOCsLinGangDataDetail> siteVOCsDetails = new ArrayList();

  public ReportMonthVOCsLinGangDataWrapper(
      List<String> siteIds,
      List<ReportSiteDevice> siteDevices,
      BasicInformation information,
      List<AvgData> siteVOCsGroupDailies,
      List<AvgData> siteVOCsFactorDailies,
      List<AvgData> siteNMHCFactorDailies,
      List<HistoricalData> siteVOCsFactorHourlies
  ) {
    List<Site> sites = this.getSites(siteIds, information);
    this.siteVOCsDetails = this.buildSiteVOCsDetails(
      sites,
      siteDevices,
      information,
      siteVOCsGroupDailies,
      siteVOCsFactorDailies,
      siteNMHCFactorDailies,
      siteVOCsFactorHourlies
    );
  }

  private List<Site> getSites(
    List<String> siteIds,
    BasicInformation information
  ) {
    Map<String, Site> siteMap = information.getSiteMap();
    Map<String, Integer> siteIndexMap = new HashMap<>();
    siteMap.forEach((siteId, site) -> {
      siteIndexMap.put(siteId, siteIndexMap.size());
    });

    List<Site> sites = new ArrayList();
    for (String siteId : siteIds) {
      sites.add(siteMap.get(siteId));
    }
    sites.sort((o1, o2) -> siteIndexMap.get(o1.getId()) - siteIndexMap.get(o2.getId()));

    return sites;
  }

  private List<ReportMonthVOCsLinGangDataDetail> buildSiteVOCsDetails(
    List<Site> sites,
    List<ReportSiteDevice> siteDevices,
    BasicInformation information,
    List<AvgData> siteVOCsGroupDailies,
    List<AvgData> siteVOCsFactorDailies,
    List<AvgData> siteNMHCFactorDailies,
    List<HistoricalData> siteVOCsFactorHourlies
  ) {
    Map<String, ReportMonthVOCsLinGangDataDetail> siteVOCsDetailMap = this.createSiteVOCsDetailsMap(sites, siteDevices, information);

    this.setVOCsGroupDailies(siteVOCsDetailMap, siteVOCsGroupDailies)
      .setVOCsFactorDailies(siteVOCsDetailMap, siteVOCsFactorDailies)
      .setNMCHFactorDailies(siteVOCsDetailMap, siteNMHCFactorDailies)
      .setVOCsFactorHourlies(siteVOCsDetailMap, siteVOCsFactorHourlies);

    this.statisticData(siteVOCsDetailMap);

    return new ArrayList(siteVOCsDetailMap.values());
  }

  private ReportMonthVOCsLinGangDataWrapper statisticData(Map<String, ReportMonthVOCsLinGangDataDetail> siteVOCsDetailMap) {
    siteVOCsDetailMap.forEach((siteId, siteVOCsDetail) -> {
      siteVOCsDetail.statisticData();
    });
    return this;
  }

  private Map<String, ReportMonthVOCsLinGangDataDetail> createSiteVOCsDetailsMap(
    List<Site> sites,
    List<ReportSiteDevice> siteDevices,
    BasicInformation information
  ) {
    Map<String, ReportSiteDevice> siteDeviceMap = this.getSiteDeviceMap(siteDevices);
    Map<String, ReportMonthVOCsLinGangDataDetail> siteVOCsDetailMap = new LinkedHashMap<>();
    Factor nmhcFactor = information.getFactorMap().get("1026");
    Map<String, FactorGroup> factorCategoryMap = information.getFactorCategoryMap();
    Map<String, Factor> factorMap = information.getFactorMap();
    Map<String, Float> ofpFactorMap = this.getOfpFactorMap(factorMap);
    Map<String, Float> soapFactorMap = this.getSoapFactorMap(factorMap);

    sites.forEach(site -> {
      String siteId = site.getId();
      ReportSiteDevice reportSiteDevice = siteDeviceMap.get(siteId);
      if (reportSiteDevice == null) {
        return;
      }

      siteVOCsDetailMap.put(
        siteId,
        new ReportMonthVOCsLinGangDataDetail(
          site,
          reportSiteDevice,
          nmhcFactor,
          factorCategoryMap,
          ofpFactorMap,
          soapFactorMap
        ));
    });

    return siteVOCsDetailMap;
  }

  private ReportMonthVOCsLinGangDataWrapper setVOCsGroupDailies(
    Map<String, ReportMonthVOCsLinGangDataDetail> siteVOCsDetailMap,
    List<AvgData> avgDataList
  ) {
    avgDataList.forEach(avgData -> {
      String siteId = avgData.getSiteId();
      ReportMonthVOCsLinGangDataDetail reportMonthVOCsLinGangDataDetail = siteVOCsDetailMap.get(siteId);
      if (reportMonthVOCsLinGangDataDetail == null) {
        return;
      }
      reportMonthVOCsLinGangDataDetail.addVOCsGroupDailyData(avgData);
    });

    return this;
  }

  private ReportMonthVOCsLinGangDataWrapper setVOCsFactorDailies(
    Map<String, ReportMonthVOCsLinGangDataDetail> siteVOCsDetailMap,
    List<AvgData> avgDataList
  ) {
    avgDataList.forEach(avgData -> {
      String siteId = avgData.getSiteId();
      ReportMonthVOCsLinGangDataDetail reportMonthVOCsLinGangDataDetail = siteVOCsDetailMap.get(siteId);
      if (reportMonthVOCsLinGangDataDetail == null) {
        return;
      }
      reportMonthVOCsLinGangDataDetail.addVOCsFactorDailyData(avgData);
    });

    return this;
  }

  private ReportMonthVOCsLinGangDataWrapper setNMCHFactorDailies(
    Map<String, ReportMonthVOCsLinGangDataDetail> siteVOCsDetailMap,
    List<AvgData> avgDataList
  ) {
    avgDataList.forEach(avgData -> {
      String siteId = avgData.getSiteId();
      ReportMonthVOCsLinGangDataDetail reportMonthVOCsLinGangDataDetail = siteVOCsDetailMap.get(siteId);
      if (reportMonthVOCsLinGangDataDetail == null) {
        return;
      }
      reportMonthVOCsLinGangDataDetail.addNMHCFactorDailyData(avgData);
    });

    return this;
  }

  private ReportMonthVOCsLinGangDataWrapper setVOCsFactorHourlies(
    Map<String, ReportMonthVOCsLinGangDataDetail> siteVOCsDetailMap,
    List<HistoricalData> siteVOCsFactorHourlies
  ) {
    siteVOCsFactorHourlies.forEach(data -> {
      String siteId = data.getSiteId();
      ReportMonthVOCsLinGangDataDetail reportMonthVOCsLinGangDataDetail = siteVOCsDetailMap.get(siteId);
      if (reportMonthVOCsLinGangDataDetail == null) {
        return;
      }
      reportMonthVOCsLinGangDataDetail.addVOCsFactorHourlyData(data);
    });

    return this;
  }

  private Map<String, ReportSiteDevice> getSiteDeviceMap(List<ReportSiteDevice> siteDevices) {
    Map<String, ReportSiteDevice> siteDeviceMap = new HashMap();
    siteDevices.forEach(siteDevice -> {
      siteDeviceMap.put(siteDevice.getSiteId(), siteDevice);
    });

    return siteDeviceMap;
  }

  private Map<String, Float> getOfpFactorMap(Map<String, Factor> factorMap) {
    Map<String, Float> ofpFactorMap = new HashMap<>();

    factorMap.forEach((factorId, factor) -> {
      Float mir = factor.getMir();

      if (!Objects.isNull(mir)) {
        ofpFactorMap.put(factorId, mir);
      }
    });

    return ofpFactorMap;
  }

  private Map<String, Float> getSoapFactorMap(Map<String, Factor> factorMap) {
    Map<String, Float> soapFactorMap = new HashMap<>();

    factorMap.forEach((factorId, factor) -> {
      Float fvocr = factor.getFvocr();

      if (!Objects.isNull(fvocr)) {
        soapFactorMap.put(factorId, 1 / ((1 - fvocr) / factor.getFac()));
      }
    });

    return soapFactorMap;
  }
}
