package more.ssaqms.xiamen.model.report;

import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.util.CalculationUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author purpose
 * @version 1.0
 * @date 2023/5/9 10:41 上午
 */
@Data
@ToString
@Accessors(chain = true)
public class ReportSiteAQIInfo {

  // 站点名称
  private String siteName;

  // 优良天数
  private Integer goodDays = 0;

  // 统计天数
  private Integer statisticDays = 0;

  // PM2.5站点均值
  private Float PM25Value = null;

  // O3月每天的均值, 求90百分位数时用
  private List<Float> o3ValueList = new ArrayList<>();

  private Map<Integer, Integer> aqiLevelDayStatisticMap = new HashMap<>();

  // 站点AQI等级天数统计
  public List<ReportAQILinGangData.AQILevelDayStatistics> getAQILevelDayStatistics() {
    List<ReportAQILinGangData.AQILevelDayStatistics> aqiLevelDayStatistics = new ArrayList<>();

    this.aqiLevelDayStatisticMap.forEach((level, count) -> {
      int index = level - 1;
      aqiLevelDayStatistics.add(new ReportAQILinGangData.AQILevelDayStatistics(level, ReportAQILinGangData.AQI_LEVEL_NAMES[index], count, ReportAQILinGangData.AQI_LEVEL_COLORS[index]));
    });

    return aqiLevelDayStatistics;
  }

  // 获取污染天天数
  public Integer getBadDays() {
    Integer statisticDays = this.statisticDays;
    Integer goodDays = this.goodDays;
    return statisticDays == null ? null : goodDays == null ? statisticDays : (statisticDays - goodDays);
  }

  // 优良天占比
  public Float getGoodDayPercentage() {
    Integer statisticDays = this.statisticDays;
    Integer goodDays = this.goodDays;
    return statisticDays == null || statisticDays == 0 ? null : goodDays == null ? 0 : CalculationUtils.digit2((this.goodDays.floatValue() / statisticDays) * 100f, 2);
  }

  // 污染天占比
  public Float getBadDayPercentage() {
    Float goodDayPercentage = this.getGoodDayPercentage();
    return goodDayPercentage == null ? null : CalculationUtils.digit2(100 - goodDayPercentage, 2);
  }

  // 获取月O3百分90位数
  public Double getO390Value() {
    List<Float> o3ValueList = this.o3ValueList;
    return o3ValueList.size() == 0 ? null : CalculationUtils.digit2(CalculationUtils.getPercentile(o3ValueList, 0.9), 2);
  }

  public ReportSiteAQIInfo(String siteName) {
    this.siteName = siteName;
  }

  public ReportSiteAQIInfo addO3Value(Float value) {
    o3ValueList.add(value);
    return this;
  }

  public ReportSiteAQIInfo setAQILevel(Integer level) {
    Integer count = aqiLevelDayStatisticMap.get(level);

    if (count == null) {
      count = 0;
    }

    aqiLevelDayStatisticMap.put(level, count + 1);

    return this;
  }
}
