package more.ssaqms.xiamen.model.site;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author purpose
 * @version 1.0
 * @date 2021/6/3 12:55 下午
 */
@ApiModel(value = "站房数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_site")
public class Site {

    @ApiModelProperty(value = "站房主键")
    @TableId(type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty(value = "站房名称", required = true)
    @NotBlank
    @TableField
    private String name;

    @ApiModelProperty(value = "cmaqCode")
    @TableField
    private String cmaqCode;

    @ApiModelProperty(value = "站房mn号")
    @NotBlank
    @TableField
    private String mn;

    @ApiModelProperty(value = "站房密码")
    @NotBlank
    @TableField
    private String password;

    @ApiModelProperty(value = "站房类型, 1. 区域评价站; 2. 园区评价及预警站")
    @TableField
    private Long type;

    @ApiModelProperty(value = "站房分类")
    @NotNull
    @TableField
    private Integer categoryId;

    @ApiModelProperty(value = "采样高度")
    @TableField
    private Float samplingHeight;

    @ApiModelProperty(value = "站房启用日期")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd")
    @NotNull
    @TableField
    private Date enableDate;

    @ApiModelProperty(value = "站房所在地理位置的经度")
    @NotNull
    @TableField
    private Double longitude;

    @ApiModelProperty(value = "站房所在地理位置的纬度")
    @NotNull
    @TableField
    private Double latitude;

    @ApiModelProperty(value = "站房所在省份")
    @NotBlank
    @TableField
    private String province;

    @ApiModelProperty(value = "站房所在城市")
    @NotBlank
    @TableField
    private String city;

    @ApiModelProperty(value = "站房地址")
    @NotBlank
    @TableField
    private String address;

    @ApiModelProperty(value = "是否启用")
    @NotNull
    @TableField
    private Boolean status;

    @ApiModelProperty(value = "是否逻辑删除: 0.未删除 1.已删除")
    @TableField
    private Boolean isDeleted;

    @ApiModelProperty(value = "是否是工业区")
    @TableField
    private Boolean isIndustrialDistrict;

    @ApiModelProperty(value = "业主组织id")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String ownerOrgId;

    @TableField
    private Integer exportReport;

    @ApiModelProperty(value = "区域id")
    @TableField
    private String regionId;
}
