package more.ssaqms.xiamen.model.site;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.audit.AuditInfo;
import more.ssaqms.xiamen.model.data.HourData;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * @author purpose
 * @version 1.0
 * @date 2021/6/11 11:48 上午
 */
@ApiModel(value = "站房仪器数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_site_device")
public class SiteDevice {

    @ApiModelProperty(value = "站房仪器主键")
    @TableId(type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty(value = "站房ID", required = true)
    @NotBlank
    @TableField
    private String siteId;

    @ApiModelProperty(value = "站点名称", required = true)
    @TableField(exist = false)
    private String siteName;

    @ApiModelProperty(value = "站房mn号", required = true)
    @TableField
    private String siteMn;

    @ApiModelProperty(value = "仪器ID", required = true)
    @NotBlank
    @TableField
    private String deviceId;

    @ApiModelProperty(value = "仪器名称", required = true)
    @TableField(exist = false)
    private String deviceName;

    @ApiModelProperty(value = "仪器类别ID", required = true)
//    @NotBlank
    @TableField
    private Long deviceCategoryId;

    @ApiModelProperty(value = "仪器序列号", required = true)
    @NotBlank
    @TableField
    private String sn;

    @ApiModelProperty(value = "212上传仪器的编码", required = true)
    @TableField
    private String deviceCode;

    @ApiModelProperty(value = "出厂日期", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd")
    @TableField
    private Date producedDate;

    @ApiModelProperty(value = "使用日期", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd")
    @TableField
    private Date activatedDate;

    @ApiModelProperty(value = "数据频率 单位: 分钟", required = true)
    @TableField
    private Integer dataFrequency;

    @ApiModelProperty(value = "告警间隔，判断是否断站（单位小时）", required = true)
    @TableField
    private Integer alarmInterval;

    @ApiModelProperty(value = "数据最大延迟时间 单位: 分钟", required = true)
//    @NotNull
    @TableField
    private Integer maxDelayTime;

    @ApiModelProperty(value = "仪器量程下限")
//    @NotNull
    @TableField
    private Float lowRange;

    @ApiModelProperty(value = "仪器量程上限")
//    @NotNull
    @TableField
    private Float highRange;

    @ApiModelProperty(value = "运维组织ID")
//    @NotNull
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String opsOrgId;

    @ApiModelProperty(value = "站房仪器状态 true:开启, false:关闭", required = true)
    @NotNull
    @TableField
    private Boolean status;

    @ApiModelProperty(value = "是否逻辑删除: 0.未删除 1.已删除")
    @TableField
//    @NotNull
    private Boolean isDeleted;

    /**
     * 数采来源
     */
    @ApiModelProperty(value = "数采来源", required = true)
    @TableField
    private String dasId;

    /**
     * 时间标识
     */
    @ApiModelProperty(value = "时间标识", required = true)
    @TableField
    private Integer dataTimeTypeId;

    /**
     * 高度(雷达数据使用)
     */
    @ApiModelProperty(value = "高度(雷达数据使用)", required = true)
    @TableField
    private String height;

    /**
     * 审核情况
     */
    @ApiModelProperty(value = "审核情况", required = true)
    @TableField(exist = false)
    private Integer auditStage;

    /**
     * 审核id
     */
    @ApiModelProperty(value = "审核id", required = true)
    @TableField(exist = false)
    private String auditId;

    /**
     * 是否完成审核
     */
    @ApiModelProperty(value = "是否完成审核", required = true)
    @TableField(exist = false)
    private Boolean isAuditOver;

    /**
     * 是否大屏可以取数据
     */
    @ApiModelProperty(value = "是否大屏可以取数据", required = true)
    @TableField(exist = false)
    private Boolean bigScreen;

    /**
     * 数据采集率
     */
    @ApiModelProperty(value = "数据采集率", required = true)
    @TableField(exist = false)
    private List<HourData> dataHarvestRate;

    /**
     * 审核详情
     */
    @ApiModelProperty(value = "审核详情", required = true)
    @TableField(exist = false)
    private List<AuditInfo> auditInfos;

    /**
     * 站点仪器对应的因子（开启的）
     */
    @ApiModelProperty(value = "站点仪器对应的因子", required = true)
    @TableField(exist = false)
    private List<SiteDeviceFactor> siteDeviceFactors;

    /**
     * 站点仪器对应的因子（关闭的）
     */
    @ApiModelProperty(value = "站点仪器对应的因子", required = true)
    @TableField(exist = false)
    private List<SiteDeviceFactor> siteDeviceFactors0;
}
