package more.ssaqms.xiamen.model.site;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.factor.Factor;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Objects;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/6/15 10:11
 */
@ApiModel(value = "站房因子数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_site_device_factor")
public class SiteDeviceFactor {

    @ApiModelProperty(value = "主键")
    @TableId(type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty(value = "站房id", required = true)
    @NotBlank
    @TableField
    private String siteId;

    @ApiModelProperty(value = "因子id", required = true)
    @NotBlank
    @TableField
    private String factorId;

    @ApiModelProperty(value = "状态", required = true)
    @NotNull
    @TableField
    private Boolean status;

    @ApiModelProperty(value = "siteDeviceId", required = true)
    @NotBlank
    @TableField
    private String siteDeviceId;

    @ApiModelProperty(value = "是否报警（0：不报警，1：报警）")
    @NotNull
    @TableField
    private Boolean isAlarm;

    @ApiModelProperty(value = "超标报警上限")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMax;

    @ApiModelProperty(value = "超标报警下限")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMin;

    @ApiModelProperty(value = "修正下限值")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float amendLowRange;

    @ApiModelProperty(value = "修正上限值")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float amendHighRange;

    @ApiModelProperty(value = "修正值")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float amendValue;

    @ApiModelProperty(value = "超标下限")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float limitLow;

    @ApiModelProperty(value = "超标上限")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float limitHigh;

    public SiteDeviceFactor() {
    }

    public SiteDeviceFactor(@NotBlank String siteId, @NotBlank String siteDeviceId, Factor factor) {
        this.siteId = siteId;
        this.factorId = factor.getId();
        this.status = factor.getStatus();
        this.siteDeviceId = siteDeviceId;
        this.isAlarm = !Objects.isNull(factor.getIsAlarm()) && factor.getIsAlarm();
        this.alarmMax = factor.getAlarmMax();
        this.alarmMin = factor.getAlarmMin();
        this.amendLowRange = factor.getAmendLowRange();
        this.amendHighRange = factor.getAmendHighRange();
        this.amendValue = factor.getAmendValue();
        this.limitLow = factor.getLimitLow();
        this.limitHigh = factor.getLimitHigh();
    }
}
