package more.ssaqms.xiamen.model.site;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.device.Device;
import more.ssaqms.xiamen.model.device.DeviceStatus;

import javax.validation.constraints.NotBlank;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/6/15 10:59
 */
@ApiModel(value = "站房仪器状态数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_site_device_status")
public class SiteDeviceStatus {

    @ApiModelProperty(value = "id", required = true)
    @TableId(type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty(value = "siteId", required = true)
    @TableField
    @NotBlank
    private String siteId;

    @ApiModelProperty(value = "deviceStatusId", required = true)
    @TableField
    @NotBlank
    private String deviceStatusId;

    @ApiModelProperty(value = "siteDeviceId", required = true)
    @TableField
    @NotBlank
    private String siteDeviceId;

    @ApiModelProperty(value = "statusName", required = true)
    @TableField
    @NotBlank
    private String statusName;

    @ApiModelProperty(value = "status", required = true)
    @TableField
    private Boolean status;

    /**
     * 是否报警（0：不报警，1：报警）
     */
    @ApiModelProperty(value = "是否报警（0：不报警，1：报警）", required = true)
    @TableField
    private Boolean isAlarm;

    /**
     * 超标报警上限
     */
    @ApiModelProperty(value = "超标报警上限", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMax;

    /**
     * 超标报警下限
     */
    @ApiModelProperty(value = "超标报警下限", required = true)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Float alarmMin;

    /**
     * 设备状态传输码
     */
    @NotBlank
    @ApiModelProperty(value = "设备状态传输码", required = true)
    @TableField
    private String statusCode;

    /**
     * 设备传输码
     */
    @NotBlank
    @ApiModelProperty(value = "设备传输码", required = true)
    @TableField
    private String deviceCode;

    /**
     * 站点MN号
     */
    @NotBlank
    @ApiModelProperty(value = "站点MN号", required = true)
    @TableField
    private String siteMn;

    /**
     * 是否关键参数
     */
    @ApiModelProperty(value = "是否关键参数")
    @TableField
    private Boolean isImportance;

    public SiteDeviceStatus() {
    }

    public SiteDeviceStatus(SiteDevice siteDevice, DeviceStatus deviceStatus) {
        this.siteId = siteDevice.getSiteId();
        this.deviceStatusId = deviceStatus.getId();
        this.siteDeviceId = siteDevice.getId();
        this.statusName = deviceStatus.getName();
        this.status = deviceStatus.getStatus();
        this.isAlarm = false;
        this.statusCode = deviceStatus.getCode();
        this.deviceCode = siteDevice.getDeviceCode();
        this.siteMn = siteDevice.getSiteMn();
        this.isImportance = deviceStatus.getIsImportance();
        this.alarmMax = deviceStatus.getAlarmMax();
        this.alarmMin = deviceStatus.getAlarmMin();
    }

    public SiteDeviceStatus(Site site, Device device, String siteDeviceId, DeviceStatus deviceStatus) {
        this.siteId = site.getId();
        this.deviceStatusId = deviceStatus.getId();
        this.siteDeviceId = siteDeviceId;
        this.statusName = deviceStatus.getName();
        this.status = deviceStatus.getStatus();
        this.isAlarm = false;
        this.statusCode = deviceStatus.getCode();
        this.deviceCode = device.getCode();
        this.siteMn = site.getMn();
        this.isImportance = deviceStatus.getIsImportance();
        this.alarmMax = deviceStatus.getAlarmMax();
        this.alarmMin = deviceStatus.getAlarmMin();
    }
}
