package more.ssaqms.xiamen.model.statistics;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author purpose
 * @version 1.0
 * @date 2023/1/13 2:08 下午
 */
@ApiModel(value = "污染特征数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_aqi_case_library")
public class AQICaseLibrary {

    /**
     * 站房主键
     */
    @ApiModelProperty(value = "站房主键", required = true)
    @NotBlank
    @TableId
    private String siteId;

    /**
     * 超标污染日期
     */
    @ApiModelProperty(value = "污染日期", required = true)
    @NotNull(message = "时间不能为空")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd")
    @TableId
    private Date dataTime;

    /**
     * AQI污染等级
     */
    @ApiModelProperty(value = "AQI污染等级", required = true)
    @NotNull
    @TableField
    private Integer aqiGrade;

    /**
     * AQI首要污染物
     */
    @ApiModelProperty(value = "AQI首要污染物", required = true)
    @NotBlank
    @TableField
    private String primaryPollutions;

    /**
     * 污染特征描述
     */
    @ApiModelProperty(value = "污杂特征描述主键", required = true)
    @NotNull
    @TableField
    private Integer characteristicId;

    /**
     * AQI峰值
     */
    @ApiModelProperty(value = "AQI峰值", required = true)
    @TableField(exist = false)
    private Integer maxAqi;

    /**
     * 污染时间段
     */
    @ApiModelProperty(value = "污染时间段", required = true)
    @TableField(exist = false)
    private List<AQIPollutionTime> aqiPollutionTimes;


    /**
     * 将AQI数据转为此模型数据
     * @param aqiData
     */
    public void convertData(AqiData aqiData) {
        this.siteId = aqiData.getSiteId();
        this.dataTime = aqiData.getDataTime();
        this.aqiGrade = aqiData.getGrade();
        this.primaryPollutions = aqiData.getPrimaryPollutant();
    }


    /**
     * 整理数据
     * @param tsHourlyList  AQI小时数据List
     */
    public void formatData(List<AqiData> tsHourlyList) {
        AqiData firstAqiData = tsHourlyList.get(0);
        Date lastDate = firstAqiData.getDataTime();
        Long lastTS = lastDate.getTime();
        AQIPollutionTime aqiPollutionTime = new AQIPollutionTime(lastDate);
        List<AQIPollutionTime> aqiPollutionTimes = this.aqiPollutionTimes = new ArrayList<>();
        Float maxAQI = firstAqiData.getAqi();
        int size = tsHourlyList.size();

        for (int i = 1; i < size; i++) {
            AqiData aqiData = tsHourlyList.get(i);
            Date date = aqiData.getDataTime();
            Long ts = date.getTime();

            if ((ts - lastTS) > 3600000) {
                aqiPollutionTime.setEndTime(new Date(lastTS + (3600000 - 1)));
                aqiPollutionTimes.add(aqiPollutionTime);
                aqiPollutionTime = new AQIPollutionTime(date);
            }

            Float aqi = aqiData.getAqi();

            if (aqi > maxAQI) {
                maxAQI = aqi;
            }

            lastTS = ts;
        }

        aqiPollutionTime.setEndTime(new Date(lastTS + (3600000 - 1)));
        aqiPollutionTimes.add(aqiPollutionTime);
        this.maxAqi = Math.round(maxAQI);
    }
}
