package more.ssaqms.xiamen.model.statistics;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.forecast.ForecastSiteData;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/7/7 12:09
 */
@ApiModel(value = "Aqi数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_aqi_site_hour_data_statistics")
public class AqiData {
    /**
     * 数据时间
     */
    @ApiModelProperty(value = "数据时间", required = true)
    @TableId
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date dataTime;

    /**
     * 站房id
     */
    @ApiModelProperty(value = "站房id", required = true)
    @TableId
    private String siteId;

    /**
     * 因子Id
     */
    @ApiModelProperty(value = "因子Id", required = true)
    @TableId
    private String factorId;

    /**
     * rootFactorId
     */
    @ApiModelProperty(value = "rootFactorId", required = true)
    @TableId
    private String rootFactorId;

    /**
     * 值
     */
    @ApiModelProperty(value = "值", required = true)
    @TableField
    private Float value;

    /**
     * 是否审核完成
     */
    @ApiModelProperty(value = "是否审核完成", required = true)
    @TableField
    private Boolean isAuditCompleted;

    /**
     * 对应同环比所需要的时间(也可用作空气质量达标分析数据的年时间）
     */
    @ApiModelProperty(value = "对应同环比所需要的时间", hidden = true)
    @TableField(exist = false)
    private String lst;

    /**
     * 用作空气质量达标分析数据的月时间
     */
    @ApiModelProperty(value = "用作空气质量达标分析数据的月时间", hidden = true)
    @TableField(exist = false)
    private String lstMonth;

    /**
     * 空气质量AQI
     */
    @ApiModelProperty(value = "空气质量AQI", hidden = true)
    @TableField(exist = false)
    private Float aqi;

    /**
     * 空气质量等级
     */
    @ApiModelProperty(value = "空气质量等级", hidden = true)
    @TableField(exist = false)
    private Integer grade;

    /**
     * 空气质量等级
     */
    @ApiModelProperty(value = "区域id", hidden = true)
    @TableField(exist = false)
    private String regionId;

    /**
     * 首要污染物
     */
    @ApiModelProperty(value = "首要污染物", hidden = true)
    @TableField(exist = false)
    private String primaryPollutant;

    /**
     * 超标污染物
     */
    @ApiModelProperty(value = "超标污染物", hidden = true)
    @TableField(exist = false)
    private String superStandardPollutant;

    /**
     * 各因子均值LIST
     */
    @ApiModelProperty(value = "各因子均值LIST", hidden = true)
    @TableField(exist = false)
    private List<PollutantData> pollutantData;

    /**
     * 备注信息
     */
    @ApiModelProperty(value = "备注信息", hidden = true)
    @TableField(exist = false)
    private AqiRemark aqiRemark;

    @ApiModelProperty(value = "预测时间", hidden = true)
    @TableField(exist = false)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd")
    private Date forecastTime;

    public AqiData() {
    }

    /**
     * 整理AQI数据方便计算
     *
     * @param aqiData
     */
    public AqiData(AqiData aqiData) {
        this.dataTime = aqiData.dataTime;
        this.siteId = aqiData.siteId;
        List<PollutantData> pollutantDataList = new ArrayList<>();
        pollutantDataList.add(new PollutantData(aqiData));
        this.pollutantData = pollutantDataList;
    }

    public AqiData(Date dataTime, String siteId, String factorId, String rootFactorId, Float value, Boolean isAuditCompleted) {
        this.dataTime = dataTime;
        this.siteId = siteId;
        this.factorId = factorId;
        this.rootFactorId = rootFactorId;
        this.value = value;
        this.isAuditCompleted = isAuditCompleted;
    }

    public AqiData(ForecastSiteData forecastSiteData) {
        this.dataTime = forecastSiteData.getDataTime();
        this.siteId = forecastSiteData.getSiteId();
        this.factorId = forecastSiteData.getFactorId();
        this.rootFactorId = forecastSiteData.getRootFactorId();
        this.value = forecastSiteData.getValue();
        this.isAuditCompleted = true;
    }
}
