package more.ssaqms.xiamen.model.statistics;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.model.forecast.ForecastSiteAqiReceive;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/8/9 16:52
 */
@Data
@Accessors(chain = true)
public class AqiReceive {

    @ApiModelProperty(value = "开始时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    @NotNull
    private Date beginTime;

    @ApiModelProperty(value = "结束时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    @NotNull
    private Date endTime;

    @ApiModelProperty(value = "年份", required = true)
    private List<Integer> years;

    @ApiModelProperty(value = "粒度(1,24)", required = true)
    private Integer granularity;

    @ApiModelProperty(value = "month,arbitrary", required = true)
    private String type;

    @ApiModelProperty(value = "是否站点合并计算", required = true)
    private Boolean isSiteMerge;

    @ApiModelProperty(value = "是否时间合并计算(计算时间段的排名)", required = true)
    private Boolean isTimeMerge;

    @ApiModelProperty(value = "是否区域合并", hidden = true)
    private Boolean isRegion;

    @ApiModelProperty(value = "是否审核后数据(true为审核后数据,其他为全部数据)", required = true)
    private Boolean isAuditCompleted;

    @ApiModelProperty(value = "查询站点,空为所有站点", required = true)
    private List<String> siteIds;

    @ApiModelProperty(value = "综合指数保留小数位数", required = true)
    private Integer indexDecimalNum;

    public AqiReceive(Date beginTime, Date endTime, Integer granularity, List<String> siteIds, Boolean isTimeMerge, Boolean isSiteMerge) {
        this.beginTime = beginTime;
        this.endTime = endTime;
        this.granularity = granularity;
        this.siteIds = siteIds;
        this.isTimeMerge = isTimeMerge;
        this.isSiteMerge = isSiteMerge;
    }

    public AqiReceive(AqiCharacteristicReceive aqiCharacteristicReceive) {
        this.beginTime = aqiCharacteristicReceive.getBeginTime();
        this.endTime = aqiCharacteristicReceive.getEndTime();
        this.isAuditCompleted = aqiCharacteristicReceive.getIsAuditCompleted();
        this.indexDecimalNum = aqiCharacteristicReceive.getIndexDecimalNum();
    }

    public AqiReceive() {
    }

    public AqiReceive(ForecastSiteAqiReceive forecastSiteAqiReceive) {
        this.beginTime = forecastSiteAqiReceive.getBeginTime();
        this.endTime = forecastSiteAqiReceive.getEndTime();
        this.granularity = forecastSiteAqiReceive.getGranularity();
        this.isSiteMerge = forecastSiteAqiReceive.getIsSiteMerge();
        this.isTimeMerge = false;
        this.siteIds = forecastSiteAqiReceive.getSiteIds();
    }
}
