package more.ssaqms.xiamen.model.statistics;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.commons.utils.tool.DateUtils;
import more.ssaqms.xiamen.model.data.HistoricalData;
import more.ssaqms.xiamen.model.operation.integral.IntegralData;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author zuo
 * @date 2022/7/14 14:34
 */
@ApiModel(value = "均值数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_avg_data")
public class AvgData {

    /**
     * 日期（天）
     */
    @ApiModelProperty(value = "日期（天）", required = true)
    @TableId
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd")
    @NotNull(message = "日期（天）")
    private Date dataTime;

    /**
     * 数据时间，用于历史查询
     */
    @ApiModelProperty(value = "数据时间，用于历史查询", hidden = true)
    @TableField(exist = false)
    private String dataTimeStr;

    /**
     * 站房设备id
     */
    @ApiModelProperty(value = "站房设备id", required = true)
    @TableId
    @NotBlank(message = "站房设备id,不能为空")
    private String siteDeviceId;

    /**
     * 因子Id
     */
    @ApiModelProperty(value = "因子Id", required = true)
    @TableId
    @NotBlank(message = "因子Id,不能为空")
    private String factorId;

    /**
     * 因子Id
     */
    @ApiModelProperty(value = "因子名称", hidden = true)
    @TableField(exist = false)
    private String factorName;

    /**
     * 站房id
     */
    @ApiModelProperty(value = "站房id", required = true)
    @TableField
    @NotBlank(message = "站房id,不能为空")
    private String siteId;

    /**
     * 均值
     */
    @ApiModelProperty(value = "均值", required = true)
    @TableField
    private Float value;

    /**
     * 有效小时数
     */
    @ApiModelProperty(value = "有效小时数", required = true)
    @TableField
    private Integer hourCount;

    /**
     * 有效天数
     */
    @ApiModelProperty(value = "有效天数", hidden = true)
    @TableField(exist = false)
    private Integer dayCount;

    /**
     * 有效月数
     */
    @ApiModelProperty(value = "有效月数", hidden = true)
    @TableField(exist = false)
    private Integer monthCount;

    /**
     * 有效月数
     */
    @ApiModelProperty(value = "该数据是否有效，false为前端页面表颜色", hidden = true)
    @TableField(exist = false)
    private Boolean isPass;

    /**
     * 是否有效
     */
    @ApiModelProperty(value = "是否有效", required = true)
    @TableField
    private Boolean isEffective;

    /**
     * 包含重积分数据的有效小时数
     */
    @ApiModelProperty(value = "包含重积分数据的有效小时数", required = true)
    @TableField
    private Integer hourIntegralCount;

    /**
     * 包含重积分数据后判断是否有效
     */
    @ApiModelProperty(value = "包含重积分数据后判断是否有效", required = true)
    @TableField
    private Boolean isEffectiveIntegral;

    /**
     * 包含重积分数据的日均值
     */
    @ApiModelProperty(value = "包含重积分数据的日均值", required = true)
    @TableField
    private Float valueIntegral;

    /**
     * 是否包含重积分数据
     */
    @ApiModelProperty(value = "包含重积分数据的日均值")
    @TableField
    private Boolean isHaveIntegral;

    public AvgData() {
    }

    public AvgData(AvgData avgData) {
        this.dataTime = avgData.getDataTime();
        this.dataTimeStr = avgData.getDataTimeStr();
        this.siteDeviceId = avgData.getSiteDeviceId();
        this.factorId = avgData.getFactorId();
        this.siteId = avgData.getSiteId();
        this.value = avgData.getValue();
        this.hourCount = avgData.getHourCount();
        this.dayCount = avgData.getDayCount();
        this.monthCount = avgData.getMonthCount();
        this.isPass = avgData.getIsPass();
        this.isEffective = avgData.getIsEffective();
        this.hourIntegralCount = avgData.getHourIntegralCount();
        this.isEffectiveIntegral = avgData.getIsEffectiveIntegral();
        this.valueIntegral = avgData.getValueIntegral();
        this.isHaveIntegral = avgData.getIsHaveIntegral();
    }

    public AvgData(HistoricalData historicalData) {
        this.dataTimeStr = DateUtils.formatDate(historicalData.getDataTime(),"yyyy/MM/dd HH");
        this.siteId = historicalData.getSiteId();
        this.factorId = historicalData.getFactorId();
        this.value = historicalData.getValue();
        this.isHaveIntegral = false;
    }

    public AvgData(IntegralData integralData) {
        this.dataTimeStr = DateUtils.formatDate(integralData.getDataTime(),"yyyy/MM/dd HH");
        this.siteId = integralData.getSiteId();
        this.factorId = integralData.getFactorId();
        this.value = integralData.getValue();
        this.isHaveIntegral = true;
    }
}
