package more.ssaqms.xiamen.model.statistics;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;
import more.ssaqms.xiamen.util.CalculationUtils;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * @author zuo
 * @date 2022/7/14 16:14
 */
@ApiModel(value = "均值数据查询接收参数模型")
@Data
@ToString
@Accessors(chain = true)
public class AvgReceive {

    @NotNull(message = "查询开始时间,不能为空")
    @ApiModelProperty(value = "查询开始时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date beginTime;

    @NotNull(message = "查询结束时间,不能为空")
    @ApiModelProperty(value = "查询结束时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date endTime;

    @ApiModelProperty(value = "站点ids", required = true)
    @Size(min = 1, message = "站点ids,至少要有一个")
    private List<String> siteIds;

    @ApiModelProperty(value = "站点设备ids", hidden = true)
    private List<String> siteDeviceIds;

    @ApiModelProperty(value = "是否重积分数据，不传为false")
    private Boolean isMultipleIntegral;

    @ApiModelProperty(value = "因子ids", required = true)
    @Size(min = 1, message = "因子ids,至少要有一个")
    private List<String> factorIds;

    @NotBlank(message = "时间粒度,不能为空")
    @ApiModelProperty(value = "时间粒度，常规均值（year|month|day|hour），其他（year|quarter|month|day|hour）", required = true)
    @Pattern(regexp = "^year|quarter|month|day|hour|day_and_night$", message = "时间粒度参数不正确")
    private String granularity;

    @ApiModelProperty(value = "用于查询时保存原始版的granularity", hidden = true)
    private String type;

    @ApiModelProperty(value = "是否需要返回pass信息（只针对VOC和常规以外的设备有效，不传为true）-为满足龚大师报告不修改新加的参数", hidden = true)
    private Boolean isIsPass;


    public AvgReceive(IopfReceive iopfReceive) {
        this.beginTime = iopfReceive.getBeginTime();
        this.endTime = iopfReceive.getEndTime();
        this.siteIds = iopfReceive.getSiteIds();
        this.siteDeviceIds = iopfReceive.getSiteDeviceIds();
        this.isMultipleIntegral = iopfReceive.getIsMultipleIntegral();
        this.granularity = iopfReceive.getGranularity();
    }

    public AvgReceive(VocAvgGroupReceive receive) {
        this.beginTime = receive.getBeginTime();
        this.endTime = receive.getEndTime();
        this.siteIds = receive.getSiteIds();
        this.siteDeviceIds = receive.getSiteDeviceIds();
        this.factorIds = receive.getFactorIds();
        this.isMultipleIntegral = receive.getIsMultipleIntegral();
        this.granularity = CalculationUtils.timeSizeConversionForSql(receive.getGranularity());
    }

    public AvgReceive(AvgReceive avgReceive,List<String> factorIds) {
        this.beginTime = avgReceive.getBeginTime();
        this.endTime = avgReceive.getEndTime();
        this.siteIds = avgReceive.getSiteIds();
        this.siteDeviceIds = avgReceive.getSiteDeviceIds();
        this.isMultipleIntegral = avgReceive.getIsMultipleIntegral();
        this.factorIds = factorIds;
        this.granularity = avgReceive.getGranularity();
        this.type = avgReceive.getType();
        this.isIsPass = avgReceive.getIsIsPass();
    }

    public AvgReceive() {
    }
}
