package more.ssaqms.xiamen.model.statistics;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author zuo
 * @date 2022/7/12 11:58
 */
@ApiModel(value = "捕集率采集率数据模型")
@Data
@ToString
@Accessors(chain = true)
@TableName(value = "t_capture_effective_data")
public class CaptureEffectiveData {

    /**
     * 时间（审核日期)
     */
    @ApiModelProperty(value = "时间（审核日期)", required = true)
    @TableId
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd")
    @NotNull(message = "时间（审核日期),不能为空")
    private Date dataTime;

    /**
     * 数据时间，用于历史查询
     */
    @ApiModelProperty(value = "数据时间，用于历史查询", hidden = true)
    @TableField(exist = false)
    private String dataTimeStr;

    /**
     * 站房设备id
     */
    @ApiModelProperty(value = "站房设备id", required = true)
    @TableId
    @NotBlank(message = "站房设备id,不能为空")
    private String siteDeviceId;

    /**
     * 因子Id
     */
    @ApiModelProperty(value = "因子Id", required = true)
    @TableId
    @NotBlank(message = "因子Id,不能为空")
    private String factorId;

    /**
     * 站房id
     */
    @ApiModelProperty(value = "站房id", required = true)
    @TableField
    @NotBlank(message = "站房id,不能为空")
    private String siteId;

    /**
     * 捕集实到（分子）
     */
    @ApiModelProperty(value = "捕集实到（分子）", required = true)
    @TableField
    private Float captureArrival;

    /**
     * 捕集应到（分母）
     */
    @ApiModelProperty(value = "捕集应到（分母）", required = true)
    @TableField
    private Float captureShould;

    /**
     * 捕集率
     */
    @ApiModelProperty(value = "捕集率", required = true)
    @TableField
    private Float captureRate;

    /**
     * 有效实到（分子）
     */
    @ApiModelProperty(value = "有效实到（分子）", required = true)
    @TableField
    private Float effectiveArrival;

    /**
     * 有效应到（分母）
     */
    @ApiModelProperty(value = "有效应到（分母）", required = true)
    @TableField
    private Float effectiveShould;

    /**
     * 有效率
     */
    @ApiModelProperty(value = "有效率", required = true)
    @TableField
    private Float effectiveRate;

    /**
     * 包含重积分数据的有效实到（分子）
     */
    @ApiModelProperty(value = "包含重积分数据的有效实到（分子）", required = true)
    @TableField
    private Float effectiveArrivalIntegral;

    /**
     * 包含重积分数据的有效率
     */
    @ApiModelProperty(value = "包含重积分数据的有效率", required = true)
    @TableField
    private Float effectiveIntegralRate;

    /**
     * 是否包含重积分数据
     */
    @ApiModelProperty(value = "包含重积分数据的日均值")
    @TableField
    private Boolean isHaveIntegral;

    /**
     * 有效天数
     */
    @ApiModelProperty(value = "有效天数")
    @TableField(exist = false)
    private Float haveDayNumb;

    /**
     * 包换重积分有效天数
     */
    @ApiModelProperty(value = "包换重积分有效天数")
    @TableField(exist = false)
    private Float haveDayNumbIntegral;


    /**
     * 有效天数或者有效率
     */
    @ApiModelProperty(value = "有效天数或者有效率")
    @TableField(exist = false)
    private Float value;


    /**
     * 单位（对应value）
     */
    @ApiModelProperty(value = "单位（对应value）")
    @TableField(exist = false)
    private String unit;

    public CaptureEffectiveData() {
    }

    public CaptureEffectiveData(CaptureEffectiveData data) {
        this.captureArrival = data.getCaptureArrival();
        this.captureShould = data.getCaptureShould();
        this.captureRate = data.getCaptureRate();
        this.effectiveArrival = data.getEffectiveArrival();
        this.effectiveShould = data.getEffectiveShould();
        this.effectiveRate = data.getEffectiveRate();
        this.effectiveArrivalIntegral = data.getEffectiveArrivalIntegral();
        this.effectiveIntegralRate = data.getEffectiveIntegralRate();
        this.isHaveIntegral = data.getIsHaveIntegral();
        this.haveDayNumb = data.getHaveDayNumb();
        this.haveDayNumbIntegral = data.getHaveDayNumbIntegral();
        this.value = data.getValue();
        this.unit = data.getUnit();
    }
}
