package more.ssaqms.xiamen.model.statistics;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * @author zuo
 * @date 2022/7/13 12:00
 */
@ApiModel(value = "捕集率有效率数据查询接收参数模型")
@Data
@ToString
@Accessors(chain = true)
public class CaptureEffectiveReceive {

    @NotNull(message = "查询开始时间,不能为空")
    @ApiModelProperty(value = "查询开始时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date beginTime;

    @NotNull(message = "查询结束时间,不能为空")
    @ApiModelProperty(value = "查询结束时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date endTime;

    @ApiModelProperty(value = "站点ids", required = true)
    @Size(min = 1, message = "站点ids,至少要有一个")
    private List<String> siteIds;

    @ApiModelProperty(value = "站点设备ids", hidden = true)
    private List<String> siteDeviceIds;

    @ApiModelProperty(value = "因子ids", required = true)
    @Size(min = 1, message = "因子ids,至少要有一个")
    private List<String> factorIds;

    @NotBlank(message = "时间粒度,不能为空")
    @ApiModelProperty(value = "时间粒度", required = true)
    @Pattern(regexp = "^year|month|day$", message = "时间粒度参数不正确(只能是year，month，day其中一个)")
    private String granularity;
}
