package more.ssaqms.xiamen.model.statistics;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * @author zuo
 * @date 2022/7/20 17:29
 */
@Data
public class IopfReceive {

    @NotNull(message = "查询开始时间,不能为空")
    @ApiModelProperty(value = "查询开始时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date beginTime;

    @NotNull(message = "查询结束时间,不能为空")
    @ApiModelProperty(value = "查询结束时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date endTime;

    @ApiModelProperty(value = "站点ids", required = true)
    @Size(min = 1, message = "站点ids,至少要有一个")
    private List<String> siteIds;

    @ApiModelProperty(value = "unit单位", required = true)
    @NotBlank(message = "单位不能为空")
    private String unit;

    @ApiModelProperty(value = "时候查询审核后数据,不传或false为查全部", required = true)
    private Boolean isAuditCompleted;

    @ApiModelProperty(value = "是否需要无效数据")
    private Boolean isGetInvalidData;

    @ApiModelProperty(value = "站点设备ids", hidden = true)
    private List<String> siteDeviceIds;

    @NotBlank(message = "时间粒度,不能为空")
    @ApiModelProperty(value = "时间粒度，常规均值（year|month|day），其他（year|quarter|month|day）", required = true)
    @Pattern(regexp = "^year|quarter|month|day|hour$", message = "时间粒度参数不正确")
    private String granularity;

    @ApiModelProperty(value = "是否重积分数据，不传为false")
    private Boolean isMultipleIntegral;

    @ApiModelProperty(value = "是否融合数据（有审核后查询审核后去除无效数据的，繁殖查询审核前全部数据），不传为false")
    private Boolean isFuse;
}
