package more.ssaqms.xiamen.model.statistics;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import more.ssaqms.xiamen.model.data.HistoricalData;
import more.ssaqms.xiamen.model.forecast.AqiForecast;

import java.util.Date;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2020/9/12 20:01
 */
@Data
public class PollutantData {

    /**
     * 时间
     */
    @ApiModelProperty(value = "时间", required = true)
    private String dataTimeStr;

    /**
     * 时间
     */
    @ApiModelProperty(value = "时间", hidden = true)
    private Date dataTime;

    /**
     * factorId
     */
    @ApiModelProperty(value = "因子Id", required = true)
    private String factorId;

    /**
     * factorId
     */
    @ApiModelProperty(value = "站点id", required = true)
    private String siteId;
    /**
     * factorName
     */
    @ApiModelProperty(value = "因子名称", required = true)
    private String factorName;

    /**
     * value
     */
    @ApiModelProperty(value = "value", required = true)
    private Float value;

    /**
     * iOpf
     */
    @ApiModelProperty(value = "iOpf", required = true)
    private Float iOfp;

    /**
     * iOpf是否通過
     */
    @ApiModelProperty(value = "iOpf")
    private Boolean isPass;

    /**
     * iSoa
     */
    @ApiModelProperty(value = "iSoa", required = true)
    private Float iSoa;

    /**
     * iLoh
     */
    @ApiModelProperty(value = "iSoa", required = true)
    private Float iLoh;

    /**
     * iAqi
     */
    @ApiModelProperty(value = "iAqi", required = true)
    private Float iAqi;

    /**
     * rootFactorId
     */
    @ApiModelProperty(value = "污染物", required = true)
    private String rootFactorId;

    /**
     * maxValue
     */
    @ApiModelProperty(value = "maxValue",hidden = true)
    private Float maxValue;

    /**
     * valueNumb
     */
    @ApiModelProperty(value = "valueNumb",hidden = true)
    private Integer valueNumb;

    /**
     * 因子分类id
     */
    @ApiModelProperty(value = "因子分类id", required = true)
    private String groupId;

    public Float getIvalue() {
        return this.value;
    }

    /**
     * 是否包含重积分数据
     */
    @ApiModelProperty(value = "包含重积分数据的日均值")
    private Boolean isHaveIntegral;

    public PollutantData() {
    }

    public PollutantData(AqiConcentrationData aqiConcentrationData,String rootFactorId) {
        this.factorId = aqiConcentrationData.getFactorId();
        this.value = aqiConcentrationData.getValue();
        this.rootFactorId = rootFactorId;
    }

    public PollutantData(AqiData aqiData) {
        this.factorId = aqiData.getFactorId();
        this.value = aqiData.getValue();
        this.rootFactorId = aqiData.getRootFactorId();
    }

    public PollutantData(HistoricalData historicalData) {
        this.factorId = historicalData.getFactorId();
        this.value = historicalData.getValue();
        this.isHaveIntegral = historicalData.getIsHaveIntegral();
    }

    public PollutantData(AqiForecast aqiForecast, Integer granularity) {
        this.factorId = aqiForecast.getFactorId();
        this.value = aqiForecast.getValue();
        String rootFactorId = "";
        switch (factorId){
            case "1001":
                rootFactorId = "SO2" ;
                break;
            case "1003":
                rootFactorId = "NO2";
                break;
            case "1005":
                rootFactorId = "CO";
                break;
            case "1006":
                rootFactorId = (1 == granularity) ? "O3" : "H8O3";
                break;
            case "1007":
                rootFactorId = "PM10";
                break;
            case "1008":
                rootFactorId = "PM2.5";
                break;
        }
        this.rootFactorId = rootFactorId;
    }
}
