package more.ssaqms.xiamen.model.statistics;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * @author zuo
 * @date 2022/7/19 13:21
 */
@ApiModel(value = "均值voc分组数据查询接收参数模型")
@Data
@ToString
@Accessors(chain = true)
public class VocAvgGroupReceive {

    @NotNull(message = "查询开始时间,不能为空")
    @ApiModelProperty(value = "查询开始时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date beginTime;

    @NotNull(message = "查询结束时间,不能为空")
    @ApiModelProperty(value = "查询结束时间", required = true)
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy/MM/dd HH:mm:ss")
    private Date endTime;

    @ApiModelProperty(value = "站点ids", required = true)
    @Size(min = 1, message = "站点ids,至少要有一个")
    private List<String> siteIds;

    @ApiModelProperty(value = "站点设备ids", hidden = true)
    private List<String> siteDeviceIds;

    @ApiModelProperty(value = "因子ids", hidden = true)
    private List<String> factorIds;

    @ApiModelProperty(value = "因子分组ids", required = true)
    @Size(min = 1, message = "因子分组ids,至少要有一个")
    private List<String> groupIds;

    @ApiModelProperty(value = "unit单位", required = true)
    @NotBlank(message = "单位不能为空")
    private String unit;

    @NotBlank(message = "时间粒度,不能为空")
    @ApiModelProperty(value = "时间粒度，常规均值（year|month|day），其他（year|quarter|month|day|hour）", required = true)
    @Pattern(regexp = "^year|quarter|month|day|hour|day_and_night$", message = "时间粒度参数不正确")
    private String granularity;

    @ApiModelProperty(value = "是否重积分数据，不传为false")
    private Boolean isMultipleIntegral;
}
