package more.ssaqms.xiamen.model.transmission;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import more.commons.utils.response.exception.MoreException;
import org.apache.http.client.utils.DateUtils;

import java.util.Calendar;
import java.util.Date;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2020/10/30 16:46
 */

@Data
public class RadarWindSpeedData {

    /**
     * 时间
     */
    @ApiModelProperty(value = "时间", required = true)
    private String dataTime;

    /**
     * 数据
     */
    @ApiModelProperty(value = "数据", required = true)
    private String data;

    /**
     * 方位角
     */
    @ApiModelProperty(value = "方位角", required = true)
    private Float azimuth;

    /**
     * 俯仰角
     */
    @ApiModelProperty(value = "俯仰角", required = true)
    private Float elevation;

    /**
     * range
     */
    @ApiModelProperty(value = "range", required = true)
    private Float range;

    /**
     * 值
     */
    @ApiModelProperty(value = "值", required = true)
    private Float radialWindSpeed;

    /**
     * cnr
     */
    @ApiModelProperty(value = "cnr", required = true)
    private Float cnr;

    /**
     * RangeStep
     */
    @ApiModelProperty(value = "RangeStep", required = true)
    private Float rangeStep;

    public RadarWindSpeedData() {
    }

    public RadarWindSpeedData(String[] split) {
        setDataTime(dateChange(split[0]));
        this.azimuth = Float.parseFloat(split[1]);
        this.elevation = Float.parseFloat(split[2]);
        this.range = Float.parseFloat(split[3]);
        this.radialWindSpeed = Float.parseFloat(split[4]);
        this.cnr = Float.parseFloat(split[5]);
    }

    public String dateChange(String dataTime) {
        Date changeDate = null;
        try {
            String[] dateFormats = new String[]{"yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd", "yyyy年MM月dd日 HH时mm分ss秒", "yyyy年MM月dd日"};
            Date date = DateUtils.parseDate(dataTime, dateFormats);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
//            calendar.add(Calendar.HOUR_OF_DAY, 8);
            changeDate = calendar.getTime();
        } catch (RuntimeException e) {
            throw new MoreException("表中时间格式不正确");
        }
        return DateUtils.formatDate(changeDate,"yyyy-MM-dd HH:mm:ss");

    }
}
