package more.ssaqms.xiamen.util;

import com.baomidou.mybatisplus.core.toolkit.StringPool;
import more.ssaqms.xiamen.model.statistics.AqiData;
import more.ssaqms.xiamen.model.statistics.PollutantData;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2020/7/6 17:09
 */

public class AqiStatUtil {

    public static final Map<String, String> primaryPollutantNameMap = new HashMap<>();

    public static final Map<String, String> pollutantPrimaryMapBy24 = new HashMap<>();
    public static final Map<String, List<Float>> bp24h = new HashMap<>();
    public static final Map<String, List<Float>> bp1h = new HashMap<>();
    public static final List<Float> iAqi = Arrays.asList(0f, 50f, 100f, 150f, 200f, 300f, 400f, 500f);

    static {
        List<Float> so2 = Arrays.asList(0f, 50f, 150f, 475f, 800f, 1600f, 2100f, 2620f);
        List<Float> no2 = Arrays.asList(0f, 40f, 80f, 180f, 280f, 565f, 750f, 940f);
        List<Float> co = Arrays.asList(0f, 2f, 4f, 14f, 24f, 36f, 48f, 60f);
        List<Float> o3 = Arrays.asList(0f, 160f, 200f, 300f, 400f, 800f, 1000f, 1200f);
        List<Float> H8O3 = Arrays.asList(0f, 100f, 160f, 215f, 265f, 800f, 1000f, 1200f);
        List<Float> pm10 = Arrays.asList(0f, 50f, 150f, 250f, 350f, 420f, 500f, 600f);
        List<Float> pm25 = Arrays.asList(0f, 35f, 75f, 115f, 150f, 250f, 350f, 500f);
        bp24h.put(AqiStringPools.SO2, so2);
        bp24h.put(AqiStringPools.NO2, no2);
        bp24h.put(AqiStringPools.CO, co);
        bp24h.put(AqiStringPools.O3, o3);
        bp24h.put(AqiStringPools.H8O3, H8O3);
        bp24h.put(AqiStringPools.PM10, pm10);
        bp24h.put(AqiStringPools.PM25, pm25);
        so2 = Arrays.asList(0f, 150f, 500f, 650f, 800f, 1600f, 2100f, 2620f);
        no2 = Arrays.asList(0f, 100f, 200f, 700f, 1200f, 2340f, 3090f, 3840f);
        co = Arrays.asList(0f, 5f, 10f, 35f, 60f, 90f, 120f, 150f);
        bp1h.put(AqiStringPools.SO2, so2);
        bp1h.put(AqiStringPools.NO2, no2);
        bp1h.put(AqiStringPools.CO, co);
        bp1h.put(AqiStringPools.O3, o3);
        bp1h.put(AqiStringPools.H8O3, H8O3);
        bp1h.put(AqiStringPools.PM10, pm10);
        bp1h.put(AqiStringPools.PM25, pm25);

        primaryPollutantNameMap.put("SO2", "SO₂");
        primaryPollutantNameMap.put("NO2", "NO₂");
        primaryPollutantNameMap.put("O3", "O₃");
        primaryPollutantNameMap.put("H8O3", "O₃_8h");
        primaryPollutantNameMap.put("PM10", "PM₁₀");
        primaryPollutantNameMap.put("PM2.5", "PM₂.₅");


        pollutantPrimaryMapBy24.put("SO₂", "SO2");
        pollutantPrimaryMapBy24.put("NO₂", "NO2");
        pollutantPrimaryMapBy24.put("O₃", "H8O3");
        pollutantPrimaryMapBy24.put("PM₁₀", "PM10");
        pollutantPrimaryMapBy24.put("PM₂.₅", "PM2.5");
        pollutantPrimaryMapBy24.put("CO","CO");
        pollutantPrimaryMapBy24.put("SO₂-98per","SO2");
        pollutantPrimaryMapBy24.put("NO₂-98per","NO2");
        pollutantPrimaryMapBy24.put("O₃-90per","H8O3");
        pollutantPrimaryMapBy24.put("PM₁₀-95per","PM10");
        pollutantPrimaryMapBy24.put("PM₂.₅-95per","PM2.5");
        pollutantPrimaryMapBy24.put("CO-95per","CO");
    }

    //计算空气质量分指数iAqiP
    public static Float calcIAQI(Float cp, String rootFactorId, Integer granularity) {
        Float iAqiP = null;
        if (cp < 0f) {
            cp = 0f;
        }
        List<Float> bp = null;
        if (granularity == 24) {
            bp = bp24h.get(rootFactorId);
        } else {
            bp = bp1h.get(rootFactorId);
        }
        if (!Objects.isNull(bp)) {
            for (int i = 0; i < bp.size(); i++) {
                if (bp.get(i) <= cp) {
                    if (i + 1 == bp.size()) {
                        iAqiP = iAqi.get(i);
                    }else if (bp.get(i + 1) > cp) {
                        iAqiP = (iAqi.get(i + 1) - iAqi.get(i)) / (bp.get(i + 1) - bp.get(i)) * (cp - bp.get(i)) + iAqi.get(i);
                    }
                }
            }
        }
        return  CalculationUtils.digitUp(iAqiP,0);
    }

    public static void main(String[] args) {
        Float o3 = CalculationUtils.digitUp(calcIAQI(109f, "O3", 1),0);
        System.out.println(o3);
    }

    //计算综合AQI
    public static AqiData calcAQI(AqiData aqiData, Integer granularity) {
        //首要污染物
        String primaryPollutant = null;
        //超标污染物
        String superStandardPollutant = null;
        //AQI值
        Float aqi = null;
        //AQI计算物种（必须齐全6中物质）
        int iAqiNumb = 0;
        List<PollutantData> pollutantDataList = aqiData.getPollutantData();
        List<PollutantData> list = new ArrayList<>();
        for (PollutantData pollutantData : pollutantDataList) {
            String rootFactorId = pollutantData.getRootFactorId();
            Float maxValue = pollutantData.getMaxValue();
            Integer valueNumb = pollutantData.getValueNumb();
//            if (granularity == 24 && AqiStringPools.H8O3.equals(rootFactorId)) {
//                rootFactorId = AqiStringPools.O3;
//                pollutantData.setRootFactorId(AqiStringPools.O3);
//            }
            Float value = pollutantData.getValue();
            Float IAQIFloat = calcIAQI(value, rootFactorId, granularity);

            boolean b = !rootFactorId.equals(AqiStringPools.PM10_24H) && !rootFactorId.equals(AqiStringPools.PM25_24H);
            if (!Objects.isNull(IAQIFloat)) {
//                if (!(granularity == 1 && (rootFactorId.equals(AqiStringPools.PM10) || rootFactorId.equals(AqiStringPools.PM25)))
//                        && !(granularity == 24 && (rootFactorId.equals(AqiStringPools.PM10_24H) || rootFactorId.equals(AqiStringPools.PM25_24H)))) {
                IAQIFloat = CalculationUtils.digitHalfEven(IAQIFloat,0);
                if (b) {
                    pollutantData.setIAqi(IAQIFloat);
                    if (!(granularity == 1 && rootFactorId.equals(AqiStringPools.H8O3)) && !(granularity == 24 && rootFactorId.equals(AqiStringPools.O3))) {
//                        if (granularity == 24 && ((!Objects.isNull(valueNumb) && valueNumb < 14) && (!Objects.isNull(maxValue) && maxValue <= 200f)))
                        if (granularity == 24 && ((!Objects.isNull(valueNumb) && valueNumb < 14)))
                            continue;
                        iAqiNumb ++ ;
                        if (Objects.isNull(aqi)) aqi = 0f;
                        if (IAQIFloat > 100.0) {
                            String rootFactorName = rootFactorId.replaceAll("H8O3", "O3_8H");
                            superStandardPollutant = StringUtils.isEmpty(superStandardPollutant) ? rootFactorId : superStandardPollutant + StringPool.COMMA + rootFactorName;
                        }
                        if (aqi <= IAQIFloat) {
                            if (Float.toString(aqi).equals(IAQIFloat.toString()) && StringUtils.isNotEmpty(primaryPollutant)) {
//                                String s = primaryPollutantNameMap.get(rootFactorId);
//                                s = StringUtils.isNotBlank(s)? s : rootFactorId;
                                primaryPollutant = primaryPollutant + StringPool.COMMA + rootFactorId;
                            } else {
//                                String s = primaryPollutantNameMap.get(rootFactorId);
//                                s = StringUtils.isNotBlank(s)? s : rootFactorId;
                                primaryPollutant = rootFactorId;
                            }
                            aqi = IAQIFloat;
                        }
                    }
                }
            }
            if (b) {
                list.add(pollutantData);
            }
        }
        if (!Objects.isNull(aqi)){
            if (iAqiNumb >= 6 || aqi > 100f){
                aqiData.setAqi(aqi);
                aqiData.setPrimaryPollutant(aqi > 50 ? primaryPollutant : "无");
                aqiData.setSuperStandardPollutant(StringUtils.isEmpty(superStandardPollutant) ? "无" : superStandardPollutant);
            }else {
                aqiData.setPrimaryPollutant("");
                aqiData.setSuperStandardPollutant("");
            }
        }else {
            aqiData.setPrimaryPollutant("");
            aqiData.setSuperStandardPollutant("");
        }
        aqiData.setPollutantData(list);
        return aqiData;
    }

    //计算臭氧8小时滑动平均
    public static Float calcO3AVG8H(List<Float> data) {
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        int count = 8;
        int startIndex = 0;
        float O3AVG8HSum = 0F;
        List<Float> O3AVGList = new ArrayList<>();
        if (data.size() <= count) {
            Float sum = 0F;
            for (Float datum : data) {
                if (datum == null) {
                    return null;
                }
                sum += datum;
            }
            return sum / (float) data.size();
        }
        while (startIndex + count <= data.size()) {
            Float sum = 0F;
            for (int i = 0; i < count; i++) {
                sum += data.get(startIndex + i);
            }
            float O3AVG8H = sum / 8f;
            O3AVG8HSum += O3AVG8H;
            startIndex++;
        }
        return O3AVG8HSum / (float) startIndex;
    }

    //获取AQI的等级
    public static Integer GetAQIInfo(Float aqi) {
        if (aqi == null) {
//            throw new MoreException(11, "参数不全");
            return null;
        }
        if (0f <= aqi && 50f >= aqi) {
            return 1;
        }
        if (50f < aqi && 100f >= aqi) {
            return 2;
        }
        if (100f < aqi && 150f >= aqi) {
            return 3;
        }
        if (150f < aqi && 200f >= aqi) {
            return 4;
        }
        if (200f < aqi && 300f >= aqi) {
            return 5;
        }
        if (300f < aqi) {
            return 6;
        }
        return null;
    }

    public static String getAirQuality(Integer grade) {
        String airQuality = null;
        switch (grade) {
            case 1:
                airQuality = "优";
                break;
            case 2:
                airQuality = "良";
                break;
            case 3:
                airQuality = "轻度污染";
                break;
            case 4:
                airQuality = "中度污染";
                break;
            case 5:
                airQuality = "重度污染";
                break;
            case 6:
                airQuality = "严重污染";
                break;
        }
        return airQuality;
    }

    public static String getPrimaryPollutant(String primaryPollutant) {
        List<String> list = new ArrayList<>();
        String[] split = primaryPollutant.split(StringPool.COMMA);
        for (String s : split) {
            switch (s) {
                case "H8O3":
                case "O3":
                    s = "O₃";
                    break;
                case "NO2":
                    s = "NO₂";
                    break;
                case "SO2":
                    s = "SO₂";
                    break;
                case "PM10":
                case "PM10_24H":
                    s = "PM₁₀";
                    break;
                case "PM2.5":
                case "PM2.5_24H":
                    s = "PM₂.₅";
                    break;
            }
            list.add(s);
        }

        return String.join(StringPool.COMMA, list);
    }
}
