package more.ssaqms.xiamen.util;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import more.commons.utils.response.exception.MoreException;
import more.ssaqms.xiamen.model.data.DataReceive;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author zuo
 * @Version 1.0
 * @Date 2021/7/7 9:42
 */

public class CalculationUtils {

    /**
     * 百分位数值计算
     *
     * @param floats 所有的值
     * @param p      百分位数
     * @return 计算结果
     */
    public static double getPercentileEXC(List<Float> floats, double p) {
        if (CollectionUtils.isEmpty(floats))
            return 0;
        double res = 0;
        if (floats.size() == 1) {
            return floats.get(0);
        }
        floats.sort(Comparator.comparing(Float::floatValue));// 从小到大排序

        int size = floats.size();
        floats.add(floats.get(size - 1) + 1f);
        floats.add(0, (floats.get(0) - 1f));

        double x = (floats.size() - 1) * p;
        int i = (int) x; // 取出整数部分
        double j = x - i; // 还没找到最精确的取double数的小数部分的方法，有可能会有误差
        res = (1 - j) * floats.get(i) + j * floats.get(i + 1);

        floats.remove(size + 1);
        floats.remove(0);
        return res;
    }
    public static void main(String[] args) {
        System.out.println(getPercentile(Arrays.asList(1f,2f,3f,4f,5f,6f,7f), 0.9));
    }
    /**
     * 百分位数值计算
     *
     * @param floats 所有的值
     * @param p      百分位数
     * @return 计算结果
     */
    public static double getPercentile(List<Float> floats, double p) {
        if (CollectionUtils.isEmpty(floats)) return 0;
        if (1 == floats.size() && !Objects.isNull(floats.get(0)))
            return Double.parseDouble(String.valueOf(floats.get(0)));
        double res = 0;
        floats.sort(Comparator.comparing(Float::floatValue));// 从小到大排序
        double x = (floats.size() - 1) * p;
        int i = (int) x; // 取出整数部分
        double j = x - i; // 还没找到最精确的取double数的小数部分的方法，有可能会有误差
        res = (1 - j) * floats.get(i) + j * floats.get(i + 1);
        return res;
    }

//    public static void main(String[] args) {
//        List<Float> floats = Arrays.asList(106f,111f,114f,70f,34f,115f,59f,54f,127f,57f,50f,65f,150f,75f,89f,60f,64f,101f,93f,85f,90f,17f,151f,141f,120f,131f,78f,122f,102f,86f,118f);
//        List<Float> floats1 = Arrays.asList(111f,34f,127f,57f,65f,75f,60f,64f,101f,17f,141f,131f,102f,106f,114f,70f,115f,59f,54f,50f,150f,89f,93f,85f,90f,151f,120f,78f,122f,86f,118f);
//        List<Float> floats2 = Arrays.asList(17f,34f,50f,54f,57f,59f,60f,64f,65f,70f,75f,78f,85f,86f,89f,90f,93f,101f,102f,106f,111f,114f,115f,118f,120f,122f,127f,131f,141f,150f,151f);
//        List<Float> floats3 = Arrays.asList(17f);
//        System.out.println(getPercentile(floats, 0.9));
//        System.out.println(getPercentile(floats1, 0.9));
//        System.out.println(getPercentile(floats2, 0.9));
//        System.out.println(getPercentile(floats3, 0.9));
//    }

    /**
     * 标准偏差计算
     *
     * @param dataList
     * @return
     */
    public static Double standardDeviation(List<Double> dataList) {
        int num = dataList.size();
        OptionalDouble average = dataList.stream().mapToDouble(Double::doubleValue).average();
        if (!average.isPresent()) return null;
        double avg = average.getAsDouble();
        Double sum = 0.0;
        for (Double aDouble : dataList) {
            sum += Math.pow((aDouble - avg), 2);
        }
        return Math.sqrt(sum / (num - 1));
    }

    /**
     * 相关性计算(计算回归方程 和 R)
     *
     * @param values1 相关性第一列数据
     * @param values2 相关性第二列数据
     * @return 回归方程中的a和b以及R2, 并返回两列数据
     */
    public static Map<String, Double> regressionEquation(List<Double> values1, List<Double> values2) {
        Map<String, Double> map = new HashMap<>();
        int size = values1.size();
        if (size == values2.size() && size > 1) {
            double xSum = 0;
            double ySum = 0;
            double xySum = 0;
            double xxSum = 0;
            double yySum = 0;
            for (int i = 0; i < size; i++) {
                Double value1 = values1.get(i);
                Double value2 = values2.get(i);
                xSum += value1;
                ySum += value2;
                xySum += value1 * value2;
                xxSum += value1 * value1;
                //yySum为计算R2用
                yySum += value2 * value2;
            }
            //计算回归方程的A和B
            double xAvg = xSum / size;
            double yAvg = ySum / size;
            double a = (xySum - size * xAvg * yAvg) / (xxSum - size * xAvg * xAvg);
            double b = yAvg - a * xAvg;


            //计算R
            double R = (xySum - xSum * ySum / size) / Math.sqrt((xxSum - xSum * xSum / size) * (yySum - ySum * ySum / size));
            System.out.println(R);
            if (xSum != 0f) {
                map.put("a", new BigDecimal(a).setScale(5, RoundingMode.HALF_UP).doubleValue());
                map.put("b", new BigDecimal(b).setScale(5, RoundingMode.HALF_UP).doubleValue());
                map.put("R", new BigDecimal(R).setScale(5, RoundingMode.HALF_UP).doubleValue());
            }
        }
        return map;
    }

    /**
     * 时间粒度sql转换
     *
     * @param granularity
     * @param dataReceive
     * @return
     */
    public static String getGranularity(String granularity, DataReceive dataReceive) {
        switch (granularity) {
            case "year":
                return "DATE_FORMAT(`data_time`, '%Y')";
            case "month":
                return "DATE_FORMAT(`data_time`, '%Y/%m')";
            case "day":
                return "DATE_FORMAT(`data_time`, '%Y/%m/%d')";
            case "arbitrary":
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH");
                return "'" + sdf.format(dataReceive.getBeginTime()) + " 至 " + sdf.format(dataReceive.getEndTime()) + "'";
            default:
                return null;
        }
    }

    /**
     * 时间粒度sql转换
     *
     * @param granularity
     * @return
     */
    public static String timeSizeConversionForSql(String granularity) {
        switch (granularity) {
            case "year":
                return "DATE_FORMAT(`data_time`, '%Y年')";
            case "half_year":
                return "CONCAT(DATE_FORMAT(`data_time`, '%Y'),IF(FLOOR((DATE_FORMAT(`data_time`, '%m')+5)/6) < 2 ,'上半年','下半年'))";
            case "quarter":
                return "CONCAT(DATE_FORMAT(`data_time`, '%Y年'),FLOOR((DATE_FORMAT(`data_time`, '%m')+2)/3),'季度')";
            case "month":
                return "CONCAT(DATE_FORMAT(`data_time`, '%Y年'),DATE_FORMAT(`data_time`, '%m月'))";
            case "day":
                return "DATE_FORMAT(`data_time`, '%Y/%m/%d')";
            case "hour":
                return "DATE_FORMAT(`data_time`, '%Y/%m/%d %H')";
            case "arbitrary":
                return "''";
            default:
                return null;
        }
    }

    /**
     * 根据粒度获取所有时间的List<string>
     *
     * @param beginTime           开始时间
     * @param endTime             结束时间
     * @param granularity         粒度
     * @param isBeyondCurrentTime 结束时间是否能超过当前实际时间
     * @return 所有时间的List<string>
     */
    public static List<String> getGranularityTime(Date beginTime, Date endTime, String granularity, boolean isBeyondCurrentTime) {
        if (Objects.isNull(beginTime) || Objects.isNull(endTime)) return null;
        SimpleDateFormat sdf = new SimpleDateFormat();
        Integer type = null;
        boolean isHalfYearOrQuarter = false;
        Integer halfYearOrQuarterAmount = null;
        boolean isHalfYear = false;
        switch (granularity) {
            case "year":
                granularity = "yyyy年";
                type = 1;
                break;
            case "half_year":
                granularity = "yyyy";
                type = 2;
                isHalfYearOrQuarter = true;
                halfYearOrQuarterAmount = 5;
                isHalfYear = true;
                break;
            case "quarter":
                granularity = "yyyy年";
                type = 2;
                isHalfYearOrQuarter = true;
                halfYearOrQuarterAmount = 2;
                break;
            case "month":
                granularity = "yyyy年MM月";
                type = 2;
                break;
            case "day":
                granularity = "yyyy/MM/dd";
                type = 5;
                break;
            case "hour":
                granularity = "yyyy/MM/dd HH";
                type = 11;
                break;
            case "arbitrary":
                sdf.applyPattern("yyyy-MM-dd HH");
                return Collections.singletonList(sdf.format(beginTime) + " 至 " + sdf.format(endTime));
            default:
                return null;
        }
        Calendar dd = Calendar.getInstance();
        dd.setTime(beginTime);
        List<String> allTimes = new ArrayList<>();
        boolean isFirst = true;
        if (!isBeyondCurrentTime) {
            endTime = endTime.getTime() <= System.currentTimeMillis() ? endTime : new Date();
        }
        while (dd.getTime().getTime() <= endTime.getTime()) {//判断是否到结束日期
            sdf.applyPattern(granularity);
            if (isFirst) {
                allTimes.add(getGranularityTimeStr(type, isHalfYearOrQuarter, halfYearOrQuarterAmount, isHalfYear, dd, sdf));
                isFirst = false;
            }
            dd.add(type, 1);//进行当前日期月份加1
            if (dd.getTime().getTime() <= endTime.getTime()) {
                sdf.applyPattern(granularity);
                allTimes.add(getGranularityTimeStr(type, isHalfYearOrQuarter, halfYearOrQuarterAmount, isHalfYear, dd, sdf));
            }
        }
        return allTimes;
    }

    /**
     * 方法抽取 根据粒度获取所有时间的List<string>
     *
     * @param type
     * @param isHalfYearOrQuarter
     * @param halfYearOrQuarterAmount
     * @param isHalfYear
     * @param dd
     * @param sdf
     * @return
     */
    private static String getGranularityTimeStr(Integer type, boolean isHalfYearOrQuarter, Integer halfYearOrQuarterAmount, boolean isHalfYear, Calendar dd, SimpleDateFormat sdf) {
        String str = sdf.format(dd.getTime());
        if (isHalfYearOrQuarter) {
            sdf.applyPattern("MM");
            dd.add(type, halfYearOrQuarterAmount);
            int q = Integer.parseInt(sdf.format(dd.getTime())) / (halfYearOrQuarterAmount + 1);
            String s = isHalfYear ? (q == 1 ? "上半年" : "下半年") : (q + "季度");
            str = str + s;
        }
        return str;
    }

    /**
     * 时间粒度格式检查
     *
     * @param granularity 时间粒度
     * @return
     */
    public static Boolean granularityCheck(String granularity) {
        List<String> list = Arrays.asList("year", "half_year", "quarter", "month", "day", "arbitrary", "hour");
        return StringUtils.isBlank(granularity) || !list.contains(granularity);
    }

//    public static void main(String[] args) throws ParseException {
//        Date d1 = new SimpleDateFormat("yyyy").parse("2020");//定义起始日期
//        Date d2 = new SimpleDateFormat("yyyy").parse("2021");
//        List<String> allTime = getAllTime(d1, d2, "year");
//        List<String> allTime = getAllTime(d1, d2, "half_year");
//        List<String> allTime = getAllTime(d1, d2, "quarter");
//        List<String> allTime = getGranularityTime(d1, d2, "year");
//        List<String> allTime = getAllTime(d1, d2, "day");
//        System.out.println(allTime);
//    }

    /**
     * 单位转换
     *
     * @param inputValue 输入值
     * @param inputUnit  输入单位
     * @param outputUnit 输出单位
     * @param molecular  分子量
     * @return 转换后的值
     */
    public static Float UnitConverter(Float inputValue, String inputUnit, String outputUnit, Float molecular) {
        float molarGasConstant = 24.45f;
        String strCurrent = Objects.isNull(inputUnit) ? "" : inputUnit.toLowerCase();
        String strTarget = Objects.isNull(outputUnit) ? "" : outputUnit.toLowerCase();

        switch (strTarget) { //目标单位
            case "mg/m3":
                switch (strCurrent) {
                    case "ppb":
                        return (inputValue * molecular / molarGasConstant) / 1000;
                    case "ppm":
                        return inputValue * molecular / molarGasConstant;
                    case "μg/m3":
                        return inputValue / 1000;
                    default:
                        return inputValue;
                }
            case "ppb":
                switch (strCurrent) {
                    case "mg/m3":
                        return inputValue * 1000 / (molecular / molarGasConstant);
                    case "ppm":
                        return inputValue * 1000;
                    case "μg/m3":
                        return inputValue / (molecular / molarGasConstant);
                    default:
                        return inputValue;
                }
            case "ppm":
                switch (strCurrent) {
                    case "mg/m3":
                        return inputValue / (molecular / molarGasConstant);
                    case "ppb":
                        return inputValue / 1000;
                    case "μg/m3":
                        return inputValue / (molecular / molarGasConstant * 1000);
                    default:
                        return inputValue;
                }
            case "μg/m3":
                switch (strCurrent) {
                    case "mg/m3":
                        return inputValue * 1000;
                    case "ppb":
                        return inputValue * molecular / molarGasConstant;
                    case "ppm":
                        return inputValue * (molecular / molarGasConstant) * 1000;
                    case "ng/m3":
                        return inputValue / 1000;
                    default:
                        return inputValue;
                }
            case "km":
                switch (strCurrent) {
                    case "m":
                        return inputValue / 1000;
                    default:
                        return inputValue;
                }
            case "m":
                switch (strCurrent) {
                    case "km":
                        return inputValue * 1000;
                    default:
                        return inputValue;
                }
            case "kpa":
                switch (strCurrent) {
                    case "pa":
                        return inputValue / 1000;
                    case "hpa":
                        return inputValue / 100;
                    case "kpa":
                        return inputValue;
                    case "psi":
                        return inputValue * 6.895f;
                    case "mm hg":
                        return inputValue * 4 / 3 / 10;
                    default:
                        return inputValue;
                }
            case "psi":
                switch (strCurrent) {
                    case "pa":
                        return inputValue / 6895f;
                    case "hpa":
                        return inputValue / 68.95f;
                    case "kpa":
                        return inputValue / 6.895f;
                    case "mm hg":
                        return inputValue * 0.0193368f;
                    default:
                        return inputValue;
                }
            case "mm hg":
                switch (strCurrent) {
                    case "pa":
                        return inputValue * 3 / 400;
                    case "hpa":
                        return inputValue * 3 / 4;
                    case "kpa":
                        return inputValue * 30 / 4;
                    case "psi":
                        return inputValue * 51.7149f;
                    default:
                        return inputValue;
                }
            case "hpa":
                switch (strCurrent) {
                    case "pa":
                        return inputValue / 100;
                    case "hpa":
                        return inputValue;
                    case "kpa":
                        return inputValue * 10;
                    case "psi":
                        return inputValue * 68.95f;
                    case "mm hg":
                        return inputValue * 4 / 3;
                    default:
                        return inputValue;
                }
            case "pa":
                switch (strCurrent) {
                    case "pa":
                        return inputValue;
                    case "hpa":
                        return inputValue * 100;
                    case "kpa":
                        return inputValue * 1000;
                    case "psi":
                        return inputValue * 6895f;
                    case "mm hg":
                        return inputValue * 400 / 3;
                    default:
                        return inputValue;
                }
            case "m/s":
                switch (strCurrent) {
                    case "km/s":
                        return inputValue * 1000;
                    default:
                        return inputValue;
                }
            case "km/s":
                switch (strCurrent) {
                    case "m/s":
                        return inputValue / 1000;
                    default:
                        return inputValue;
                }
            case "m3/h":
                switch (strCurrent) {
                    case "m3/d":
                        return inputValue / 24;
                    case "m3/s":
                        return inputValue * 3600;
                    default:
                        return inputValue;
                }
            case "m3/d":
                switch (strCurrent) {
                    case "m3/h":
                        return inputValue * 24;
                    case "m3/s":
                        return inputValue * 3600 * 24;
                    default:
                        return inputValue;
                }
            case "m3/s":
                switch (strCurrent) {
                    case "m3/h":
                        return inputValue / 3600;
                    case "m3/d":
                        return inputValue / 3600 / 24;
                    default:
                        return inputValue;
                }

            case "kg":
                switch (strCurrent) {
                    case "g":
                        return inputValue / 1000;
                    case "mg":
                        return inputValue / 1000000;
                    case "μg":
                        return inputValue / 1000000000;
                    default:
                        return inputValue;
                }
            case "g":
                switch (strCurrent) {
                    case "kg":
                        return inputValue * 1000;
                    case "mg":
                        return inputValue / 1000;
                    case "μg":
                        return inputValue / 1000000;
                    default:
                        return inputValue;
                }

            case "mg":
                switch (strCurrent) {
                    case "kg":
                        return inputValue * 1000000;
                    case "g":
                        return inputValue * 1000;
                    case "μg":
                        return inputValue / 1000;
                    default:
                        return inputValue;
                }
            case "μg":
                switch (strCurrent) {
                    case "kg":
                        return inputValue * 1000000000;
                    case "g":
                        return inputValue * 1000000;
                    case "mg":
                        return inputValue * 1000;
                    default:
                        return inputValue;
                }


            case "kg/h":
                switch (strCurrent) {
                    case "kg/s":
                        return inputValue * 3600;
                    case "kg/d":
                        return inputValue / 24;
                    default:
                        return inputValue;
                }
            case "kg/s":
                switch (strCurrent) {
                    case "kg/h":
                        return inputValue / 3600;
                    case "kg/d":
                        return inputValue / 24 / 3600;
                    default:
                        return inputValue;
                }
            case "kg/d":
                switch (strCurrent) {
                    case "kg/h":
                        return inputValue * 24;
                    case "kg/s":
                        return inputValue * 24 * 3600;
                    default:
                        return inputValue;
                }
            default:
                return inputValue;
        }
    }

    // 表达式和某字符串比较
    public static boolean judge(String express, Map<String, Object> dataMap) throws IllegalAccessException {
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine engine = manager.getEngineByName("js");
        express = getData(engine, dataMap, express);
        Object result = null;
        try {
            result = engine.eval(express);
        } catch (ScriptException e) {
            // log.error("字符串转化运算表达式出错", e);
            e.printStackTrace();
        }
        return (boolean) result;
    }

    public static String getData(ScriptEngine engine, Map<String, Object> dataMap, String express) throws IllegalAccessException {
        for (String key : dataMap.keySet()) {
            Object obj = dataMap.get(key);
            if (!Objects.isNull(obj)) {
                engine.put(key, obj);
            } else {
                express = express.replace(key, "(1==1)||" + key);
            }
        }
        return express;
    }

    public static Integer getDayCount(String dateTimeStr, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date date = null;
        try {
            date = sdf.parse(dateTimeStr);
        } catch (ParseException ignored) {
            throw new MoreException("时间格式错误");
        }
        Calendar ca = Calendar.getInstance();
        assert date != null;
        ca.setTime(date);
        ca.set(Calendar.DATE, 1);
        ca.roll(Calendar.DATE, -1);
        return ca.get(Calendar.DATE);
    }

    public static double getAvg(List<Float> values) {
        double avg = 0;
        if (CollectionUtils.isNotEmpty(values)) {
            double sum = 0;
            for (Float value : values) {
                sum += value;
            }
            avg = sum / values.size();
        }
        return avg;
    }

    public static Map<String, Object> calculateList(List<Float> values) {
        Map<String, Object> map = new HashMap<>();
        map.put("max", "NA");
        map.put("min", "NA");
        map.put("avg", "NA");
        map.put("sum", "NA");
        if (CollectionUtils.isNotEmpty(values)) {
            Float max = Collections.max(values);
            max = BigDecimal.valueOf(max).setScale(2, RoundingMode.HALF_UP).floatValue();
            map.put("max", max);
            Float min = Collections.min(values);
            min = BigDecimal.valueOf(min).setScale(2, RoundingMode.HALF_UP).floatValue();
            map.put("min", min);
            OptionalDouble average = values.stream().mapToDouble(Float::floatValue).average();
            if (average.isPresent()) {
                //保留小数位数
                map.put("avg", BigDecimal.valueOf(average.getAsDouble()).setScale(2, RoundingMode.HALF_UP));
            }
            double sum = values.stream().mapToDouble(Float::floatValue).sum();
            map.put("sum", BigDecimal.valueOf(sum).setScale(2, RoundingMode.HALF_UP).floatValue());
        }
        return map;
    }

    /**
     * 主导风
     *
     * @param wswds
     * @return
     */
    public static double[] calcWsWdVectorAvg(List<double[]> wswds) {
        double windSpeed = 0;
        double windDirection = 0;
        double vx = 0;
        double vy = 0;
        int count = wswds.size();

        for (int i = 0; i < count; i++) {
            double ws = wswds.get(i)[0]; //数组0代表风速
            double wd = wswds.get(i)[1]; //数据1代表风向
            vx += ws * Math.sin(convertDegreesToRadians(wd));
            vy += ws * Math.cos(convertDegreesToRadians(wd));
        }
        vx = (vx / count) * -1;
        vy = (vy / count) * -1;
        windSpeed = Math.sqrt(vx * vx + vy * vy);
        windDirection = Math.atan2(vx, vy) * 180 / Math.PI;
        if (windDirection > 180) {
            windDirection -= 180;
        } else if (windDirection < 180) {
            windDirection += 180;
        }
        return new double[]{windSpeed, windDirection};
    }

    private static double convertDegreesToRadians(double degrees) {
        return (Math.PI / 180) * degrees;
    }


    public static String getWdString(double windDirection) {
        String windDirectionStr = "";
        if (windDirection <= 10 || windDirection >= 350) {
            windDirectionStr = "北风";
        } else if (windDirection > 10 && windDirection < 80) {
            windDirectionStr = "东北风";
        } else if (windDirection >= 80 && windDirection <= 100) {
            windDirectionStr = "东风";
        } else if (windDirection > 100 && windDirection < 170) {
            windDirectionStr = "东南风";
        } else if (windDirection >= 170 && windDirection <= 190) {
            windDirectionStr = "南风";
        } else if (windDirection > 190 && windDirection < 260) {
            windDirectionStr = "西南风";
        } else if (windDirection >= 260 && windDirection <= 280) {
            windDirectionStr = "西风";
        } else if (windDirection > 280) {
            windDirectionStr = "西北风";
        }
        return windDirectionStr;
    }

    public static String digit(Double d, int digit) {
        if (Objects.isNull(d)) return null;
        return new BigDecimal(d).setScale(digit, RoundingMode.HALF_UP).toString();
    }

    public static String digit(Float d, int digit) {
        if (Objects.isNull(d)) return null;
        return new BigDecimal(d).setScale(digit, RoundingMode.HALF_UP).toString();
    }

    public static Double digit2(Double d, int digit) {
        if (Objects.isNull(d)) return null;
        return new BigDecimal(d).setScale(digit, RoundingMode.HALF_UP).doubleValue();
    }

    public static Float digit2(Float d, int digit) {
        if (Objects.isNull(d)) return null;
        return new BigDecimal(d).setScale(digit, RoundingMode.HALF_UP).floatValue();
    }

    /**
     * @param value 需要科学计算的数据
     * @param digit 保留的小数位
     * @return 功能：保留小数位数后有数就进位
     */
    public static Float digitUp(Float value, int digit) {
        if (Objects.isNull(value)) return null;
        return new BigDecimal(value).setScale(digit, RoundingMode.UP).floatValue();
    }

    /**
     * @param value 需要科学计算的数据
     * @param digit 保留的小数位
     * @return 功能：四舍六入五成双计算法
     */
    public static Float digitHalfEven(Double value, int digit) {
        if (Objects.isNull(value)) return null;
        double pow10 = Math.pow(10, digit + 1);
        double pow102 = pow10 / 10;
        double newNumber = pow10 * value;
        double unitsDigit = newNumber % 10;

        if (unitsDigit < 5) {
            return toFloat(Math.floor(newNumber / 10) / pow102, digit);
        }

        if (unitsDigit > 5) {
            return toFloat(Math.ceil(newNumber / 10) / pow102, digit);
        }

        if (newNumber - Math.floor(newNumber) > 0) {
            return toFloat(Math.ceil(newNumber / 10) / pow102, digit);
        }

        newNumber = Math.floor(newNumber / 10);
        return toFloat((newNumber + (newNumber % 2)) / pow102, digit);
    }


    public static Float toFloat(double value, int digit) {
        return new BigDecimal(value).setScale(digit, RoundingMode.HALF_EVEN).floatValue();
    }

    public static Float digitHalfEven(Float value, int digit) {
        if (Objects.isNull(value)) return null;
        Double aDouble = Double.valueOf(String.valueOf(value));
        return digitHalfEven(aDouble, digit);
    }

    public static String digitHalfEvenStr(Float value, int digit) {
        if (Objects.isNull(value)) return null;
        return Objects.requireNonNull(digitHalfEven(value, digit)).toString();
    }
}
