package more.ssaqms.xiamen.util;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.web.multipart.MultipartFile;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.UUID;


/**
 * @author zuo
 * @date 2022/11/3 18:01
 */

public class FileUtils {


    public static boolean changeFileName(String oldFilePath, String newFileName) {
        File oldFile = new File(oldFilePath);
        File newFile = new File(oldFile.getParent(), newFileName);
        if (newFile.exists()) {
            delFile(newFile);
        }
        if (oldFile.exists()) {
            // 修改文件名
            if (!oldFile.renameTo(newFile)) {
                return false;
            }
        } else {
            System.out.println("文件不存在！");
            return false;
        }
        return true;
    }

    /**
     * 文件移动（替换现有文件）
     *
     * @param sourcePath      源文件路径（带文件名）
     * @param destinationPath 目标文件路径（带文件名）
     * @return
     */
    public static boolean moveFile(String sourcePath, String destinationPath) {
        Path source = Paths.get(sourcePath);
        Path destination = Paths.get(destinationPath);
        new File(destinationPath).getParentFile().mkdirs();

        try {
            Files.move(source, destination, StandardCopyOption.REPLACE_EXISTING);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("文件移动失败！");
        }
        return true;
    }

    /**
     * base64字符串转换成图片
     * imgStr base64字符串
     * path 生成图片路径
     * photoName 图片名称
     */
    public static boolean GenerateImage(String imgStr, String path, String photoName) {
        BASE64Decoder decoder = new BASE64Decoder();
        try {
            String baseValue = imgStr.replaceAll(" ", "+");
            byte[] b = decoder.decodeBuffer(baseValue.replace("data:image/jpeg;base64,", ""));
            for (int i = 0; i < b.length; ++i) {
                if (b[i] < 0) {
                    b[i] += 256;
                }
            }
            File targetFile = new File(path);
            if (!targetFile.exists()) {
                targetFile.mkdirs();
            }
            OutputStream out = Files.newOutputStream(Paths.get(path + "/" + photoName));
            out.write(b);
            out.flush();
            out.close();
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public static String getLocalHtmlStream(String path) throws Exception {
        File f = new File(path);
        if (!f.exists() && !f.getAbsolutePath().endsWith("html")) {
            return "";
        }
        InputStreamReader in = new InputStreamReader(Files.newInputStream(Paths.get(path)), "UTF-8");
        StringBuffer b = new StringBuffer();
        int c;
        while ((c = in.read()) != -1) {
            b.append((char) c);
        }
        return b.toString();
    }

    //删除文件及文件夹方法
    public static boolean delFile(File index) {
//        if (Objects.isNull(index)) return true;
        if (index.isDirectory()) {
            File[] files = index.listFiles();
            if (files != null) {
                for (File in : files) {
                    delFile(in);
                }
            }
        }
        //出现几次删除成功代表有几个文件和文本文件
        return index.delete();
    }

    /**
     * 将JSON数据格式化并保存到文件中
     *
     * @param content  要保存的内容
     * @param filePath 输出的文件地址
     * @return
     */
    public static boolean saveFile(String content, String filePath) {
        // 标记文件生成是否成功
        boolean flag = true;
        // 生成json格式文件
        try {
            // 保证创建一个新文件
            File file = new File(filePath);
            if (!file.getParentFile().exists()) { // 如果父目录不存在，创建父目录
                file.getParentFile().mkdirs();
            }
            if (file.exists()) { // 如果已存在,删除旧文件
                file.delete();
            }
            boolean t = file.createNewFile();
            // 将格式化后的字符串写入文件
            Writer write = new OutputStreamWriter(Files.newOutputStream(file.toPath()), "UTF-8");
            write.write(content);
            write.flush();
            write.close();
        } catch (Exception e) {
            flag = false;
            e.printStackTrace();
        }
        return flag;
    }

    public static String image2bast64(String path) {
        //将图片文件转化为字节数组字符串，并对其进行Base64编码处理
        InputStream inputStream = null;
        byte[] buffer = null;
        //读取图片字节数组
        try {

            inputStream = new FileInputStream(path);
            int count = 0;
            while (count == 0) {
                count = inputStream.available();
            }
            buffer = new byte[count];
            inputStream.read(buffer);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    // 关闭inputStream流
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return "data:image/jpg;base64," + new BASE64Encoder().encode(buffer);
    }

    public static String saveMultipartFile(MultipartFile file, String path, String fileName) {

        File toFile = null;
        if (file.equals("") || file.getSize() <= 0) {
            return null;
        } else {

            /*获取文件原名称*/
            String originalFilename = file.getOriginalFilename();
            /*获取文件格式*/
            String fileFormat = originalFilename.substring(originalFilename.lastIndexOf("."));

            String uuid = UUID.randomUUID().toString().trim().replaceAll("-", "");
            toFile = new File(path + File.separator + fileName + fileFormat);

            String absolutePath = null;
            try {
                absolutePath = toFile.getCanonicalPath();

                /*判断路径中的文件夹是否存在，如果不存在，先创建文件夹*/
                String dirPath = absolutePath.substring(0, absolutePath.lastIndexOf(File.separator));
                File dir = new File(dirPath);
                if (!dir.exists()) {
                    dir.mkdirs();
                }

                InputStream ins = file.getInputStream();

                inputStreamToFile(ins, toFile);
                ins.close();

            } catch (IOException e) {
                e.printStackTrace();
            }
            return absolutePath;
        }

    }

    public static String saveMultipartFile(MultipartFile file, String path) {

        File toFile = null;
        if (file.equals("") || file.getSize() <= 0) {
            return null;
        } else {

            /*获取文件原名称*/
            String originalFilename = file.getOriginalFilename();
            /*获取文件格式*/

            String uuid = UUID.randomUUID().toString().trim().replaceAll("-", "");
            toFile = new File(path + File.separator + originalFilename);

            String absolutePath = null;
            try {
                absolutePath = toFile.getCanonicalPath();

                /*判断路径中的文件夹是否存在，如果不存在，先创建文件夹*/
                String dirPath = absolutePath.substring(0, absolutePath.lastIndexOf(File.separator));
                File dir = new File(dirPath);
                if (!dir.exists()) {
                    dir.mkdirs();
                }

                InputStream ins = file.getInputStream();

                inputStreamToFile(ins, toFile);
                ins.close();

            } catch (IOException e) {
                e.printStackTrace();
            }
            return absolutePath;
        }
    }

    //获取流文件
    private static void inputStreamToFile(InputStream ins, File file) {
        try {
            OutputStream os = Files.newOutputStream(file.toPath());
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = ins.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            ins.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 删除本地临时文件
     *
     * @param file
     */
    public static void deleteTempFile(File file) {
        if (file != null) {
            File del = new File(file.toURI());
            del.delete();
        }
    }


    public static void saveJsonFile(String filePathStr, String fileNameStr, Object data) {


        try {
            File filePath = new File(filePathStr);
            if (!filePath.exists()) {
                filePath.mkdirs();
            }

//            String jsonStr = JSON.toJSONString(ResponseResult.success(data));
            ObjectMapper objectMapper = new ObjectMapper();
            String jsonStr = objectMapper.writeValueAsString(data);
            String jsonFilePath = filePathStr + "/" + fileNameStr;
            FileWriter fw = new FileWriter(jsonFilePath);

            fw.write(jsonStr);
            fw.flush();
            fw.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }


    /**
     * 在图片文件左上角写字
     * @param imagePath 图片文件路径
     * @param text      需要再文件左上角写的文字内容
     * @param savePath  保存路径
     * @throws IOException
     */
    public static Boolean imageWatermark(String imagePath, String text, String savePath) {
        try {
            // 读取图片
            BufferedImage image = ImageIO.read(new File(imagePath));

            BufferedImage resizedImage = new BufferedImage(
                    (int) (image.getWidth() / 2.0),
                    (int) (image.getHeight() / 2.0),
                    BufferedImage.TYPE_INT_RGB);

            // 创建Graphics2D对象
//            Graphics2D g2d = (Graphics2D) image.getGraphics();
            Graphics2D g2d = resizedImage.createGraphics();
            g2d.drawImage(image, 0, 0, resizedImage.getWidth(), resizedImage.getHeight(), null);

            // 设置字体和颜色
            g2d.setFont(new Font("宋体", Font.BOLD, 100)); // 字体、样式和大小
            g2d.setColor(Color.RED); // 文字颜色
            g2d.drawString(text, 100, 200); // 在图片上绘制文本

            // 释放图形上下文使用的系统资源
            g2d.dispose();

            // 保存新的图片文件
            ImageIO.write(resizedImage, "jpg", new File(savePath));
        } catch (IOException e) {
            return false;
        }
        return true;
    }

//    public static void main(String[] args) throws IOException {
//        imageWatermark("/111.jpg","02时","/222.jpg");
//    }
}
