package more.ssaqms.xiamen.util;

import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.spire.doc.*;
import com.spire.doc.documents.BorderStyle;
import com.spire.doc.documents.HorizontalAlignment;
import com.spire.doc.documents.Paragraph;
import com.spire.doc.documents.ParagraphStyle;
import com.spire.doc.fields.DocPicture;
import com.spire.doc.fields.TableOfContent;
import com.spire.doc.fields.TextRange;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.converter.WordToHtmlConverter;
import org.apache.poi.xwpf.converter.core.BasicURIResolver;
import org.apache.poi.xwpf.converter.core.FileImageExtractor;
import org.apache.poi.xwpf.converter.xhtml.XHTMLConverter;
import org.apache.poi.xwpf.converter.xhtml.XHTMLOptions;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.util.ResourceUtils;
import org.springframework.web.util.HtmlUtils;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.awt.*;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zuo
 * @date 2022/10/21 11:14
 */

public class WordTransformUtils {

    public static String docToHtml() throws Exception {
        File path = new File(ResourceUtils.getURL("classpath:").getPath());
        String imagePathStr = path.getAbsolutePath() + "\\static\\image\\";
        String sourceFileName = path.getAbsolutePath() + "\\static\\test.doc";
        String targetFileName = path.getAbsolutePath() + "\\static\\test2.html";
        File file = new File(imagePathStr);
        if (!file.exists()) {
            file.mkdirs();
        }
        HWPFDocument wordDocument = new HWPFDocument(new FileInputStream(sourceFileName));
        org.w3c.dom.Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        WordToHtmlConverter wordToHtmlConverter = new WordToHtmlConverter(document);
        //保存图片，并返回图片的相对路径
        wordToHtmlConverter.setPicturesManager((content, pictureType, name, width, height) -> {
            try (FileOutputStream out = new FileOutputStream(imagePathStr + name)) {
                out.write(content);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return "image/" + name;
        });
        wordToHtmlConverter.processDocument(wordDocument);
        org.w3c.dom.Document htmlDocument = wordToHtmlConverter.getDocument();
        DOMSource domSource = new DOMSource(htmlDocument);
        StreamResult streamResult = new StreamResult(new File(targetFileName));
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer serializer = tf.newTransformer();
        serializer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
        serializer.setOutputProperty(OutputKeys.INDENT, "yes");
        serializer.setOutputProperty(OutputKeys.METHOD, "html");
        serializer.transform(domSource, streamResult);
        return targetFileName;
    }

    public static String docxToHtml() throws Exception {
        File path = new File(ResourceUtils.getURL("classpath:").getPath());
        String imagePath = path.getAbsolutePath() + "\\static\\image";
        String sourceFileName = path.getAbsolutePath() + "\\static\\test.docx";
        String targetFileName = path.getAbsolutePath() + "\\static\\test.html";

        OutputStreamWriter outputStreamWriter = null;
        try {
            XWPFDocument document = new XWPFDocument(new FileInputStream(sourceFileName));
            XHTMLOptions options = XHTMLOptions.create();
            // 存放图片的文件夹
            options.setExtractor(new FileImageExtractor(new File(imagePath)));
            // html中图片的路径
            options.URIResolver(new BasicURIResolver("image"));
            outputStreamWriter = new OutputStreamWriter(new FileOutputStream(targetFileName), "utf-8");
            XHTMLConverter xhtmlConverter = (XHTMLConverter) XHTMLConverter.getInstance();
            xhtmlConverter.convert(document, outputStreamWriter, options);
        } finally {
            if (outputStreamWriter != null) {
                outputStreamWriter.close();
            }
        }
        return targetFileName;
    }

    public static String readfile(String filePath) {
        File file = new File(filePath);
        InputStream input = null;
        try {
            input = new FileInputStream(file);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        StringBuffer buffer = new StringBuffer();
        byte[] bytes = new byte[1024];
        try {
            for (int n; (n = input.read(bytes)) != -1; ) {
                buffer.append(new String(bytes, 0, n, "utf8"));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buffer.toString();
    }

    public static String writeWordFile(String content, String savePath, String reportPath, String fileName, Boolean isMulu, String issue) {
        content = content.replaceAll("&nbsp; ", "　");
        Map<String, Object> param = new HashMap<String, Object>();

        if (!"".equals(savePath)) {
            File fileDir = new File(savePath);
            if (!fileDir.exists()) {
                fileDir.mkdirs();
            }
            content = HtmlUtils.htmlUnescape(content);
            List<HashMap<String, String>> imgs = getImgStr(content);
            int count = 0;
            for (HashMap<String, String> img : imgs) {
                count++;
                //处理替换以“/>”结尾的img标签
                content = content.replace(img.get("img"), "${imgReplace" + count + "}");
                //处理替换以“>”结尾的img标签
                content = content.replace(img.get("img1"), "${imgReplace" + count + "}");
                Map<String, Object> header = new HashMap<String, Object>();

                try {
                    String imagePath = reportPath + img.get("src");
                    //如果没有宽高属性，默认设置为400*300
                    if (img.get("width") == null || img.get("height") == null) {
                        header.put("width", 400);
                        header.put("height", 300);
                    } else {
                        String width = img.get("width");
                        String height = img.get("height");
                        header.put("width", (int) (Double.parseDouble(width)));
                        header.put("height", (int) (Double.parseDouble(height)));
                    }
                    header.put("type", "jpg");
                    header.put("content", OfficeUtil.inputStream2ByteArray(new FileInputStream(imagePath), true));
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                }
                param.put("${imgReplace" + count + "}", header);
            }
            try {
                // 生成doc格式的word文档，需要手动改为docx
                byte[] by = content.getBytes(StandardCharsets.UTF_8);
                ByteArrayInputStream bais = new ByteArrayInputStream(by);

                ByteArrayToFile(bais, savePath + "/temp.doc");
                bais.close();

                // 临时文件（手动改好的docx文件）
                com.spire.doc.Document document = new com.spire.doc.Document();
                document.loadFromFile(savePath + "/temp.doc");

                if (isMulu) {
                    linGangMulu(document, issue);
                }

                document.saveToFile(savePath + "/temp.docx", FileFormat.Docx);
                document.close();

                CustomXWPFDocument doc = OfficeUtil.generateWord(param, savePath + "/temp.docx");
//                doc.enforceUpdateFields();

                //最终生成的带图片的word文件
                FileOutputStream fopts = new FileOutputStream(savePath + "/" + fileName + ".docx");
                doc.write(fopts);
                doc.close();
                fopts.close();
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
        return savePath + "/" + fileName + ".docx";
    }

    /**
     * 目录价页眉页脚
     *
     * @param doc
     */
    public static void linGangMulu(com.spire.doc.Document doc, String issue) {
        //加载已设置大纲级别的测试文档
//        com.spire.doc.Document doc = new com.spire.doc.Document("/mnt/super-station-aqms-report/2023/1/环境综合观测月报.docx");

        //在文档最前面插入一个段落，写入文本并格式化
        Paragraph parainserted = new Paragraph(doc);
        TextRange tr = parainserted.appendText("目 录");
        tr.getCharacterFormat().setBold(true);
        tr.getCharacterFormat().setTextColor(Color.gray);
        tr.getCharacterFormat().setFontName("宋体");
//        Paragraph paragraph1 = doc.getSections().get(0).getParagraphs().get(6);
//        paragraph1.appendBreak(BreakType.Page_Break);
        doc.getSections().get(0).getParagraphs().insert(31, parainserted);
        parainserted.getFormat().setHorizontalAlignment(HorizontalAlignment.Center);


        ParagraphStyle style = new ParagraphStyle(doc);
        style.setName("style123");
        style.getCharacterFormat().setFontName("宋体");
        doc.getStyles().add(style);

        //通过域代码添加目录表
        TableOfContent toc = new TableOfContent(doc, "{\\o \"1-3\" \\h \\z \\u}");
        Paragraph paragraph2 = doc.getSections().get(0).getParagraphs().get(30);
        paragraph2.appendTOC(1, 3);
//        paragraph2.appendBreak(BreakType.Page_Break);
//        for (int i = 0; i < 15; i++) {
//            Paragraph paragraph = doc.getSections().get(0).getParagraphs().get(i);
//            paragraph.applyStyle(style.getName());
//        }


        Section section = doc.getSections().get(0);
        insertHeaderAndFooter(section, issue);
        doc.updateTableOfContents();
        //保存文档
//        doc.saveToFile("/mnt/super-station-aqms-report/2023/1/AddToc2.docx", FileFormat.Docx_2010);
    }

    /**
     * 页眉页脚
     *
     * @param section
     */
    public static void insertHeaderAndFooter(Section section, String issue) {
        //设置首页页眉页脚不同
        section.getPageSetup().setDifferentFirstPageHeaderFooter(true);

        //添加首页页眉页脚
        HeaderFooter firstpageheader = section.getHeadersFooters().getFirstPageHeader();
        Paragraph para1 = firstpageheader.addParagraph();
        DocPicture headerPicture = para1.appendPicture("/1.jpg");
        headerPicture.setHorizontalAlignment(ShapeHorizontalAlignment.Left);
//        headerPicture.setVerticalOrigin(VerticalOrigin.Top_Margin_Area);
        headerPicture.setVerticalAlignment(ShapeVerticalAlignment.Bottom);
        Paragraph para2 = firstpageheader.addParagraph();
        TextRange textRange1 = para2.appendText("期号：" + issue);
        para2.getFormat().setHorizontalAlignment(HorizontalAlignment.Right);
        textRange1.getCharacterFormat().setBold(true);
        textRange1.getCharacterFormat().setFontSize(10f);
//        textRange1.getCharacterFormat().setTextColor(Color.GREEN);


        //从节中获取页眉页脚
        HeaderFooter header = section.getHeadersFooters().getHeader();
        HeaderFooter footer = section.getHeadersFooters().getFooter();

        //将段落添加到页眉
        Paragraph headerParagraph = header.addParagraph();

//        //将图像插入页眉并设置图像的位置
//        DocPicture headerPicture = headerParagraph.appendPicture("成都.jpg");
//        headerPicture.setHorizontalAlignment(ShapeHorizontalAlignment.Left);
//        headerPicture.setVerticalOrigin(VerticalOrigin.Top_Margin_Area);
//        headerPicture.setVerticalAlignment(ShapeVerticalAlignment.Bottom);

        //将文本插入页眉
        TextRange text = headerParagraph.appendText("临港环境空气质量月报");
        text.getCharacterFormat().setFontName("宋体");
        text.getCharacterFormat().setFontSize(10);
//        text.getCharacterFormat().setItalic(true);
//        headerParagraph.getFormat().setHorizontalAlignment(HorizontalAlignment.Right);
        headerParagraph.getFormat().setHorizontalAlignment(HorizontalAlignment.Center);

//        //设置文本环绕方式
//        headerPicture.setTextWrappingStyle(TextWrappingStyle.Behind);

//        //设置页眉段落的底部边框样式
        headerParagraph.getFormat().getBorders().getBottom().setBorderType(BorderStyle.Single);
        headerParagraph.getFormat().getBorders().getBottom().setLineWidth(1f);

        //调整页眉距离
        section.getPageSetup().setHeaderDistance(40);

        //将段落添加到页脚
        Paragraph footerParagraph = footer.addParagraph();

        //将Field_Page域和Field_Num_Pages域添加到页脚段落中
        footerParagraph.appendField("page number", FieldType.Field_Page);
//        footerParagraph.appendText(" of ");
//        footerParagraph.appendField("number of pages", FieldType.Field_Num_Pages);
        footerParagraph.getFormat().setHorizontalAlignment(HorizontalAlignment.Center);

        //设置页脚段落的顶部边框样式
        footerParagraph.getFormat().getBorders().getTop().setBorderType(BorderStyle.Single);
        footerParagraph.getFormat().getBorders().getTop().setLineWidth(1f);

        //调整页脚距离
        section.getPageSetup().setFooterDistance(40);
    }

    // 字节数组到文件的过程
    public static void ByteArrayToFile(ByteArrayInputStream bais, String newFileNmae) {
        File file = new File(newFileNmae);
        //选择流
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(file);
            int temp;
            byte[] bt = new byte[1024 * 10];
            while ((temp = bais.read(bt)) != -1) {
                fos.write(bt, 0, temp);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            //关流
            try {
                if (null != fos)
                    fos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    //获取html中的图片元素信息
    public static List<HashMap<String, String>> getImgStr(String htmlStr) {
        List<HashMap<String, String>> pics = new ArrayList<HashMap<String, String>>();

        Document doc = Jsoup.parse(htmlStr);
        Elements imgs = doc.select("img");
        for (Element img : imgs) {
            HashMap<String, String> map = new HashMap<String, String>();
            String width = img.attr("width");
            String style = img.attr("style");
            String[] split = style.split(StringPool.SEMICOLON);
            for (String s : split) {
                String[] split1 = s.split(StringPool.COLON);
                if (split1.length > 1) {
                    String imgLength = split1[1].substring(0, split1[1].length() - 2);
                    if ("width".equalsIgnoreCase(split1[0])) {
                        imgLength = String.valueOf(Double.parseDouble(imgLength) * 96.15384615384615);
                        map.put("width", imgLength);
                    } else if ("height".equalsIgnoreCase(split1[0])) {
                        imgLength = String.valueOf(Double.parseDouble(imgLength) * 96.15384615384615);
                        map.put("height", imgLength);
                    }
                }
            }
//            if (!"".equals(width)) {
//                map.put("width", width.substring(0, width.length() - 2));
//            }
//            if (!"".equals(img.attr("height"))) {
//                map.put("height", img.attr("height").substring(0, img.attr("height").length() - 2));
//            }
            map.put("img", img.toString().substring(0, img.toString().length() - 1) + "/>");
            map.put("img1", img.toString());
            map.put("src", img.attr("src"));
            pics.add(map);
        }
        return pics;
    }

    public static String dayReportHtmlAll(String dayReportHtml) {

        dayReportHtml =
                "<html>" +
                        "<head>" +
                        "<META http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" +
                        "<meta content=\"zuo\" name=\"author\">" +
                        "<style>p {text-align:justify;font-family:\"Times New Roman\";}" +
                        "</head>" +
                        "<body>" + dayReportHtml +
                        "</body>" +
                        "</html>";


        return dayReportHtml;
    }
}
